@@ -30,12 +30,12 @@ use polars_utils::aliases::{InitHashMaps, PlHashMap};
 use polars_utils::slice::GetSaferUnchecked;
 use private::Sealed;
 
-use crate::array::binview::view::{validate_binary_view, validate_utf8_only, validate_utf8_view};
+use crate::array::binview::view::{validate_binary_view, validate_utf8_only};
 use crate::array::iterator::NonNullValuesIter;
 use crate::bitmap::utils::{BitmapIter, ZipValidity};
 pub type BinaryViewArray = BinaryViewArrayGeneric<[u8]>;
 pub type Utf8ViewArray = BinaryViewArrayGeneric<str>;
-pub use view::View;
+pub use view::{validate_utf8_view, View};
 
 use super::Splitable;
 
@@ -159,6 +159,34 @@ impl<T: ViewType + ?Sized> BinaryViewArrayGeneric<T> {
         total_bytes_len: usize,
         total_buffer_len: usize,
     ) -> Self {
+        // Verify the invariants
+        #[cfg(debug_assertions)]
+        {
+            // @TODO: Enable this. This is currently bugged with concatenate.
+            // let mut actual_total_buffer_len = 0;
+            // let mut actual_total_bytes_len = 0;
+            //
+            // for buffer in buffers.iter() {
+            //     actual_total_buffer_len += buffer.len();
+            // }
+
+            for view in views.iter() {
+                // actual_total_bytes_len += view.length as usize;
+                if view.length > View::MAX_INLINE_SIZE {
+                    assert!((view.buffer_idx as usize) < (buffers.len()));
+                    assert!(
+                        view.offset as usize + view.length as usize
+                            <= buffers[view.buffer_idx as usize].len()
+                    );
+                }
+            }
+
+            // assert_eq!(actual_total_buffer_len, total_buffer_len);
+            // if (total_bytes_len as u64) != UNKNOWN_LEN {
+            //     assert_eq!(actual_total_bytes_len, total_bytes_len);
+            // }
+        }
+
         Self {
             data_type,
             views,
@@ -232,6 +260,29 @@ impl<T: ViewType + ?Sized> BinaryViewArrayGeneric<T> {
         )
     }
 
+    /// Apply a function over the views. This can be used to update views in operations like slicing.
+    ///
+    /// # Safety
+    /// Update the views. All invariants of the views apply.
+    pub unsafe fn apply_views<F: FnMut(View, &T) -> View>(&self, mut update_view: F) -> Self {
+        let arr = self.clone();
+        let (views, buffers, validity, total_bytes_len, total_buffer_len) = arr.into_inner();
+
+        let mut views = views.make_mut();
+        for v in views.iter_mut() {
+            let str_slice = T::from_bytes_unchecked(v.get_slice_unchecked(&buffers));
+            *v = update_view(*v, str_slice);
+        }
+        Self::new_unchecked(
+            self.data_type.clone(),
+            views.into(),
+            buffers,
+            validity,
+            total_bytes_len,
+            total_buffer_len,
+        )
+    }
+
     pub fn try_new(
         data_type: ArrowDataType,
         views: Buffer<View>,
@@ -384,7 +435,7 @@ impl<T: ViewType + ?Sized> BinaryViewArrayGeneric<T> {
         let buffers = self.buffers.as_ref();
 
         for view in self.views.as_ref() {
-            unsafe { mutable.push_view_copied_unchecked(*view, buffers) }
+            unsafe { mutable.push_view_unchecked(*view, buffers) }
         }
         mutable.freeze().with_validity(self.validity)
     }
@@ -540,6 +591,13 @@ impl<T: ViewType + ?Sized> Array for BinaryViewArrayGeneric<T> {
     }
 
     fn with_validity(&self, validity: Option<Bitmap>) -> Box<dyn Array> {
+        debug_assert!(
+            validity.as_ref().map_or(true, |v| v.len() == self.len()),
+            "{} != {}",
+            validity.as_ref().unwrap().len(),
+            self.len()
+        );
+
         let mut new = self.clone();
         new.validity = validity;
         Box::new(new)
