@@ -21,6 +21,11 @@ use crate::trusted_len::TrustedLen;
 const DEFAULT_BLOCK_SIZE: usize = 8 * 1024;
 const MAX_EXP_BLOCK_SIZE: usize = 16 * 1024 * 1024;
 
+// Invariants:
+//
+// - Each view must point to a valid slice of a buffer
+// - `total_buffer_len` must be equal to `completed_buffers.iter().map(Vec::len).sum()`
+// - `total_bytes_len` must be equal to `views.iter().map(View::len).sum()`
 pub struct MutableBinaryViewArray<T: ViewType + ?Sized> {
     pub(crate) views: Vec<View>,
     pub(crate) completed_buffers: Vec<Buffer<u8>>,
@@ -97,11 +102,41 @@ impl<T: ViewType + ?Sized> MutableBinaryViewArray<T> {
         }
     }
 
+    /// Get a mutable reference to the [`Vec`] of [`View`]s in this [`MutableBinaryViewArray`].
+    ///
+    /// # Safety
+    ///
+    /// This is safe as long as any mutation of the [`Vec`] does not break any invariants of the
+    /// [`MutableBinaryViewArray`] before it is read again.
     #[inline]
-    pub fn views_mut(&mut self) -> &mut Vec<View> {
+    pub unsafe fn views_mut(&mut self) -> &mut Vec<View> {
         &mut self.views
     }
 
+    /// Set the `total_bytes_len` of the [`MutableBinaryViewArray`]
+    ///
+    /// # Safety
+    ///
+    /// This should not break invariants of the [`MutableBinaryViewArray`]
+    #[inline]
+    pub unsafe fn set_total_bytes_len(&mut self, value: usize) {
+        #[cfg(debug_assertions)]
+        {
+            let actual_length: usize = self.views().iter().map(|v| v.length as usize).sum();
+            assert_eq!(value, actual_length);
+        }
+
+        self.total_bytes_len = value;
+    }
+
+    pub fn total_bytes_len(&self) -> usize {
+        self.total_bytes_len
+    }
+
+    pub fn total_buffer_len(&self) -> usize {
+        self.total_buffer_len
+    }
+
     #[inline]
     pub fn views(&self) -> &[View] {
         &self.views
@@ -144,7 +179,7 @@ impl<T: ViewType + ?Sized> MutableBinaryViewArray<T> {
     /// - caller must allocate enough capacity
     /// - caller must ensure the view and buffers match.
     /// - The array must not have validity.
-    pub(crate) unsafe fn push_view_copied_unchecked(&mut self, v: View, buffers: &[Buffer<u8>]) {
+    pub(crate) unsafe fn push_view_unchecked(&mut self, v: View, buffers: &[Buffer<u8>]) {
         let len = v.length;
         self.total_bytes_len += len as usize;
         if len <= 12 {
@@ -165,7 +200,7 @@ impl<T: ViewType + ?Sized> MutableBinaryViewArray<T> {
     /// - caller must ensure the view and buffers match.
     /// - The array must not have validity.
     /// - caller must not mix use this function with other push functions.
-    pub unsafe fn push_view_unchecked(&mut self, mut v: View, buffers: &[Buffer<u8>]) {
+    pub unsafe fn push_view_unchecked_dedupe(&mut self, mut v: View, buffers: &[Buffer<u8>]) {
         let len = v.length;
         self.total_bytes_len += len as usize;
         if len <= 12 {
@@ -268,12 +303,10 @@ impl<T: ViewType + ?Sized> MutableBinaryViewArray<T> {
 
     #[inline]
     pub fn push_buffer(&mut self, buffer: Buffer<u8>) -> u32 {
-        if !self.in_progress_buffer.is_empty() {
-            self.completed_buffers
-                .push(Buffer::from(std::mem::take(&mut self.in_progress_buffer)));
-        }
+        self.finish_in_progress();
 
         let buffer_idx = self.completed_buffers.len();
+        self.total_buffer_len += buffer.len();
         self.completed_buffers.push(buffer);
         buffer_idx as u32
     }
@@ -438,14 +471,17 @@ impl<T: ViewType + ?Sized> MutableBinaryViewArray<T> {
     /// # Safety
     /// Same as `push_view_unchecked()`.
     #[inline]
-    pub unsafe fn extend_non_null_views_trusted_len_unchecked<I>(
+    pub unsafe fn extend_non_null_views_unchecked_dedupe<I>(
         &mut self,
         iterator: I,
         buffers: &[Buffer<u8>],
     ) where
-        I: TrustedLen<Item = View>,
+        I: Iterator<Item = View>,
     {
-        self.extend_non_null_views_unchecked(iterator, buffers);
+        self.reserve(iterator.size_hint().0);
+        for v in iterator {
+            self.push_view_unchecked_dedupe(v, buffers);
+        }
     }
 
     #[inline]
@@ -563,11 +599,142 @@ impl MutableBinaryViewArray<[u8]> {
             )?
         }
         // Restore in-progress buffer as we don't want to get too small buffers
-        if let (true, Some(last)) = (pushed, self.completed_buffers.pop()) {
-            self.in_progress_buffer = last.into_mut().right().unwrap();
+        if pushed {
+            if let Some(last) = self.completed_buffers.pop() {
+                self.in_progress_buffer = last.into_mut().right().unwrap();
+            }
         }
         Ok(())
     }
+
+    /// Extend from a `buffer` and `length` of items given some statistics about the lengths.
+    ///
+    /// This will attempt to dispatch to several optimized implementations.
+    ///
+    /// # Safety
+    ///
+    /// This is safe if the statistics are correct.
+    pub unsafe fn extend_from_lengths_with_stats(
+        &mut self,
+        buffer: &[u8],
+        lengths_iterator: impl Clone + ExactSizeIterator<Item = usize>,
+        min_length: usize,
+        max_length: usize,
+        sum_length: usize,
+    ) {
+        let num_items = lengths_iterator.len();
+
+        if num_items == 0 {
+            return;
+        }
+
+        #[cfg(debug_assertions)]
+        {
+            let (min, max, sum) = lengths_iterator.clone().map(|v| (v, v, v)).fold(
+                (usize::MAX, usize::MIN, 0usize),
+                |(cmin, cmax, csum), (emin, emax, esum)| {
+                    (cmin.min(emin), cmax.max(emax), csum + esum)
+                },
+            );
+
+            assert_eq!(min, min_length);
+            assert_eq!(max, max_length);
+            assert_eq!(sum, sum_length);
+        }
+
+        assert!(sum_length <= buffer.len());
+
+        let mut buffer_offset = 0;
+        if min_length > View::MAX_INLINE_SIZE as usize
+            && (num_items == 1 || sum_length + self.in_progress_buffer.len() <= u32::MAX as usize)
+        {
+            let buffer_idx = self.completed_buffers().len() as u32;
+            let in_progress_buffer_offset = self.in_progress_buffer.len();
+
+            self.total_bytes_len += sum_length;
+            self.total_buffer_len += sum_length;
+
+            self.in_progress_buffer
+                .extend_from_slice(&buffer[..sum_length]);
+            self.views.extend(lengths_iterator.map(|length| {
+                // SAFETY: We asserted before that the sum of all lengths is smaller or equal to
+                // the buffer length.
+                let view_buffer =
+                    unsafe { buffer.get_unchecked(buffer_offset..buffer_offset + length) };
+
+                // SAFETY: We know that the minimum length > View::MAX_INLINE_SIZE. Therefore, this
+                // length is > View::MAX_INLINE_SIZE.
+                let view = unsafe {
+                    View::new_noninline_unchecked(
+                        view_buffer,
+                        buffer_idx,
+                        (buffer_offset + in_progress_buffer_offset) as u32,
+                    )
+                };
+                buffer_offset += length;
+                view
+            }));
+        } else if max_length <= View::MAX_INLINE_SIZE as usize {
+            self.total_bytes_len += sum_length;
+
+            // If the min and max are the same, we can dispatch to the optimized SIMD
+            // implementation.
+            if min_length == max_length {
+                let length = min_length;
+                if length == 0 {
+                    self.views
+                        .resize(self.views.len() + num_items, View::new_inline(&[]));
+                } else {
+                    View::extend_with_inlinable_strided(
+                        &mut self.views,
+                        &buffer[..length * num_items],
+                        length as u8,
+                    );
+                }
+            } else {
+                self.views.extend(lengths_iterator.map(|length| {
+                    // SAFETY: We asserted before that the sum of all lengths is smaller or equal
+                    // to the buffer length.
+                    let view_buffer =
+                        unsafe { buffer.get_unchecked(buffer_offset..buffer_offset + length) };
+
+                    // SAFETY: We know that each view has a length <= View::MAX_INLINE_SIZE because
+                    // the maximum length is <= View::MAX_INLINE_SIZE
+                    let view = unsafe { View::new_inline_unchecked(view_buffer) };
+
+                    buffer_offset += length;
+
+                    view
+                }));
+            }
+        } else {
+            // If all fails, just fall back to a base implementation.
+            self.reserve(num_items);
+            for length in lengths_iterator {
+                let value = &buffer[buffer_offset..buffer_offset + length];
+                buffer_offset += length;
+                self.push_value(value);
+            }
+        }
+    }
+
+    /// Extend from a `buffer` and `length` of items.
+    ///
+    /// This will attempt to dispatch to several optimized implementations.
+    #[inline]
+    pub fn extend_from_lengths(
+        &mut self,
+        buffer: &[u8],
+        lengths_iterator: impl Clone + ExactSizeIterator<Item = usize>,
+    ) {
+        let (min, max, sum) = lengths_iterator.clone().map(|v| (v, v, v)).fold(
+            (usize::MAX, usize::MIN, 0usize),
+            |(cmin, cmax, csum), (emin, emax, esum)| (cmin.min(emin), cmax.max(emax), csum + esum),
+        );
+
+        // SAFETY: We just collected the right stats.
+        unsafe { self.extend_from_lengths_with_stats(buffer, lengths_iterator, min, max, sum) }
+    }
 }
 
 impl<T: ViewType + ?Sized, P: AsRef<T>> Extend<Option<P>> for MutableBinaryViewArray<T> {
@@ -641,3 +808,54 @@ impl<T: ViewType + ?Sized, P: AsRef<T>> TryPush<Option<P>> for MutableBinaryView
         Ok(())
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    fn roundtrip(values: &[&[u8]]) -> bool {
+        let buffer = values
+            .iter()
+            .flat_map(|v| v.iter().copied())
+            .collect::<Vec<u8>>();
+        let lengths = values.iter().map(|v| v.len()).collect::<Vec<usize>>();
+        let mut bv = MutableBinaryViewArray::<[u8]>::with_capacity(values.len());
+
+        bv.extend_from_lengths(&buffer[..], lengths.into_iter());
+
+        &bv.values_iter().collect::<Vec<&[u8]>>()[..] == values
+    }
+
+    #[test]
+    fn extend_with_lengths_basic() {
+        assert!(roundtrip(&[]));
+        assert!(roundtrip(&[b"abc"]));
+        assert!(roundtrip(&[
+            b"a_very_very_long_string_that_is_not_inlinable"
+        ]));
+        assert!(roundtrip(&[
+            b"abc",
+            b"a_very_very_long_string_that_is_not_inlinable"
+        ]));
+    }
+
+    #[test]
+    fn extend_with_inlinable_fastpath() {
+        assert!(roundtrip(&[b"abc", b"defg", b"hix"]));
+        assert!(roundtrip(&[b"abc", b"defg", b"hix", b"xyza1234abcd"]));
+    }
+
+    #[test]
+    fn extend_with_inlinable_eq_len_fastpath() {
+        assert!(roundtrip(&[b"abc", b"def", b"hix"]));
+        assert!(roundtrip(&[b"abc", b"def", b"hix", b"xyz"]));
+    }
+
+    #[test]
+    fn extend_with_not_inlinable_fastpath() {
+        assert!(roundtrip(&[
+            b"a_very_long_string123",
+            b"a_longer_string_than_the_previous"
+        ]));
+    }
+}
