@@ -1,7 +1,7 @@
-use super::Array;
+use super::{Array, Splitable};
 use crate::bitmap::Bitmap;
 use crate::buffer::Buffer;
-use crate::datatypes::DataType;
+use crate::datatypes::ArrowDataType;
 
 #[cfg(feature = "arrow_rs")]
 mod data;
@@ -17,7 +17,7 @@ use polars_error::{polars_bail, polars_ensure, PolarsResult};
 #[derive(Clone)]
 pub struct FixedSizeBinaryArray {
     size: usize, // this is redundant with `data_type`, but useful to not have to deconstruct the data_type.
-    data_type: DataType,
+    data_type: ArrowDataType,
     values: Buffer<u8>,
     validity: Option<Bitmap>,
 }
@@ -31,7 +31,7 @@ impl FixedSizeBinaryArray {
     /// * The length of `values` is not a multiple of `size` in `data_type`
     /// * the validity's length is not equal to `values.len() / size`.
     pub fn try_new(
-        data_type: DataType,
+        data_type: ArrowDataType,
         values: Buffer<u8>,
         validity: Option<Bitmap>,
     ) -> PolarsResult<Self> {
@@ -67,17 +67,17 @@ impl FixedSizeBinaryArray {
     /// * The `data_type`'s physical type is not [`crate::datatypes::PhysicalType::FixedSizeBinary`]
     /// * The length of `values` is not a multiple of `size` in `data_type`
     /// * the validity's length is not equal to `values.len() / size`.
-    pub fn new(data_type: DataType, values: Buffer<u8>, validity: Option<Bitmap>) -> Self {
+    pub fn new(data_type: ArrowDataType, values: Buffer<u8>, validity: Option<Bitmap>) -> Self {
         Self::try_new(data_type, values, validity).unwrap()
     }
 
     /// Returns a new empty [`FixedSizeBinaryArray`].
-    pub fn new_empty(data_type: DataType) -> Self {
+    pub fn new_empty(data_type: ArrowDataType) -> Self {
         Self::new(data_type, Buffer::new(), None)
     }
 
     /// Returns a new null [`FixedSizeBinaryArray`].
-    pub fn new_null(data_type: DataType, length: usize) -> Self {
+    pub fn new_null(data_type: ArrowDataType, length: usize) -> Self {
         let size = Self::maybe_get_size(&data_type).unwrap();
         Self::new(
             data_type,
@@ -105,6 +105,7 @@ impl FixedSizeBinaryArray {
     /// Slices this [`FixedSizeBinaryArray`].
     /// # Implementation
     /// This operation is `O(1)`.
+    ///
     /// # Safety
     /// The caller must ensure that `offset + length <= self.len()`.
     pub unsafe fn slice_unchecked(&mut self, offset: usize, length: usize) {
@@ -151,6 +152,7 @@ impl FixedSizeBinaryArray {
     }
 
     /// Returns the element at index `i` as &str
+    ///
     /// # Safety
     /// Assumes that the `i < self.len`.
     #[inline]
@@ -178,12 +180,12 @@ impl FixedSizeBinaryArray {
     /// # Panics
     /// Panics iff the data_type is not supported for the physical type.
     #[inline]
-    pub fn to(self, data_type: DataType) -> Self {
+    pub fn to(self, data_type: ArrowDataType) -> Self {
         match (
             data_type.to_logical_type(),
             self.data_type().to_logical_type(),
         ) {
-            (DataType::FixedSizeBinary(size_a), DataType::FixedSizeBinary(size_b))
+            (ArrowDataType::FixedSizeBinary(size_a), ArrowDataType::FixedSizeBinary(size_b))
                 if size_a == size_b => {},
             _ => panic!("Wrong DataType"),
         }
@@ -203,19 +205,19 @@ impl FixedSizeBinaryArray {
 }
 
 impl FixedSizeBinaryArray {
-    pub(crate) fn maybe_get_size(data_type: &DataType) -> PolarsResult<usize> {
+    pub(crate) fn maybe_get_size(data_type: &ArrowDataType) -> PolarsResult<usize> {
         match data_type.to_logical_type() {
-            DataType::FixedSizeBinary(size) => {
+            ArrowDataType::FixedSizeBinary(size) => {
                 polars_ensure!(*size != 0, ComputeError: "FixedSizeBinaryArray expects a positive size");
                 Ok(*size)
             },
-            _ => {
-                polars_bail!(ComputeError: "FixedSizeBinaryArray expects DataType::FixedSizeBinary")
+            other => {
+                polars_bail!(ComputeError: "FixedSizeBinaryArray expects DataType::FixedSizeBinary. found {other:?}")
             },
         }
     }
 
-    pub fn get_size(data_type: &DataType) -> usize {
+    pub fn get_size(data_type: &ArrowDataType) -> usize {
         Self::maybe_get_size(data_type).unwrap()
     }
 }
@@ -233,6 +235,34 @@ impl Array for FixedSizeBinaryArray {
     }
 }
 
+impl Splitable for FixedSizeBinaryArray {
+    fn check_bound(&self, offset: usize) -> bool {
+        offset < self.len()
+    }
+
+    unsafe fn _split_at_unchecked(&self, offset: usize) -> (Self, Self) {
+        let (lhs_values, rhs_values) = unsafe { self.values.split_at_unchecked(offset) };
+        let (lhs_validity, rhs_validity) = unsafe { self.validity.split_at_unchecked(offset) };
+
+        let size = self.size;
+
+        (
+            Self {
+                data_type: self.data_type.clone(),
+                values: lhs_values,
+                validity: lhs_validity,
+                size,
+            },
+            Self {
+                data_type: self.data_type.clone(),
+                values: rhs_values,
+                validity: rhs_validity,
+                size,
+            },
+        )
+    }
+}
+
 impl FixedSizeBinaryArray {
     /// Creates a [`FixedSizeBinaryArray`] from an fallible iterator of optional `[u8]`.
     pub fn try_from_iter<P: AsRef<[u8]>, I: IntoIterator<Item = Option<P>>>(
@@ -255,7 +285,7 @@ impl FixedSizeBinaryArray {
     /// Creates a [`FixedSizeBinaryArray`] from a slice of arrays of bytes
     pub fn from_slice<const N: usize, P: AsRef<[[u8; N]]>>(a: P) -> Self {
         let values = a.as_ref().iter().flatten().copied().collect::<Vec<_>>();
-        Self::new(DataType::FixedSizeBinary(N), values.into(), None)
+        Self::new(ArrowDataType::FixedSizeBinary(N), values.into(), None)
     }
 
     /// Creates a new [`FixedSizeBinaryArray`] from a slice of optional `[u8]`.
@@ -264,20 +294,3 @@ impl FixedSizeBinaryArray {
         MutableFixedSizeBinaryArray::from(slice).into()
     }
 }
-
-pub trait FixedSizeBinaryValues {
-    fn values(&self) -> &[u8];
-    fn size(&self) -> usize;
-}
-
-impl FixedSizeBinaryValues for FixedSizeBinaryArray {
-    #[inline]
-    fn values(&self) -> &[u8] {
-        &self.values
-    }
-
-    #[inline]
-    fn size(&self) -> usize {
-        self.size
-    }
-}
