@@ -130,9 +130,8 @@ impl<T: NativeType> MutablePrimitiveArray<T> {
     #[inline]
     pub fn push_value(&mut self, value: T) {
         self.values.push(value);
-        match &mut self.validity {
-            Some(validity) => validity.push(true),
-            None => {},
+        if let Some(validity) = &mut self.validity {
+            validity.push(true)
         }
     }
 
