@@ -1,4 +1,3 @@
-use std::iter::FromIterator;
 use std::sync::Arc;
 
 use polars_error::{polars_bail, PolarsResult};
@@ -8,7 +7,7 @@ use crate::array::physical_binary::*;
 use crate::array::{Array, MutableArray, TryExtend, TryExtendFromSelf, TryPush};
 use crate::bitmap::utils::{BitmapIter, ZipValidity};
 use crate::bitmap::{Bitmap, MutableBitmap};
-use crate::datatypes::DataType;
+use crate::datatypes::ArrowDataType;
 use crate::offset::{Offset, Offsets};
 use crate::trusted_len::TrustedLen;
 
@@ -57,7 +56,7 @@ impl<O: Offset> MutableUtf8Array<O> {
     /// # Implementation
     /// This function is `O(N)` - checking utf8 is `O(N)`
     pub fn try_new(
-        data_type: DataType,
+        data_type: ArrowDataType,
         offsets: Offsets<O>,
         values: Vec<u8>,
         validity: Option<MutableBitmap>,
@@ -75,6 +74,7 @@ impl<O: Offset> MutableUtf8Array<O> {
     }
 
     /// Create a [`MutableUtf8Array`] out of low-end APIs.
+    ///
     /// # Safety
     /// The caller must ensure that every value between offsets is a valid utf8.
     /// # Panics
@@ -82,7 +82,7 @@ impl<O: Offset> MutableUtf8Array<O> {
     /// * The `offsets` and `values` are inconsistent
     /// * The validity is not `None` and its length is different from `offsets`'s length minus one.
     pub unsafe fn new_unchecked(
-        data_type: DataType,
+        data_type: ArrowDataType,
         offsets: Offsets<O>,
         values: Vec<u8>,
         validity: Option<MutableBitmap>,
@@ -100,7 +100,7 @@ impl<O: Offset> MutableUtf8Array<O> {
         Self::from_trusted_len_iter(slice.as_ref().iter().map(|x| x.as_ref()))
     }
 
-    fn default_data_type() -> DataType {
+    fn default_data_type() -> ArrowDataType {
         Utf8Array::<O>::default_data_type()
     }
 
@@ -145,14 +145,13 @@ impl<O: Offset> MutableUtf8Array<O> {
     }
 
     /// Returns the value of the element at index `i`, ignoring the array's validity.
-    /// # Safety
-    /// This function is safe iff `i < self.len`.
     #[inline]
     pub fn value(&self, i: usize) -> &str {
         self.values.value(i)
     }
 
     /// Returns the value of the element at index `i`, ignoring the array's validity.
+    ///
     /// # Safety
     /// This function is safe iff `i < self.len`.
     #[inline]
@@ -198,7 +197,7 @@ impl<O: Offset> MutableUtf8Array<O> {
     }
 
     /// Extract the low-end APIs from the [`MutableUtf8Array`].
-    pub fn into_data(self) -> (DataType, Offsets<O>, Vec<u8>, Option<MutableBitmap>) {
+    pub fn into_data(self) -> (ArrowDataType, Offsets<O>, Vec<u8>, Option<MutableBitmap>) {
         let (data_type, offsets, values) = self.values.into_inner();
         (data_type, offsets, values, self.validity)
     }
@@ -261,11 +260,11 @@ impl<O: Offset> MutableArray for MutableUtf8Array<O> {
         array.arced()
     }
 
-    fn data_type(&self) -> &DataType {
+    fn data_type(&self) -> &ArrowDataType {
         if O::IS_LARGE {
-            &DataType::LargeUtf8
+            &ArrowDataType::LargeUtf8
         } else {
-            &DataType::Utf8
+            &ArrowDataType::Utf8
         }
     }
 
@@ -329,6 +328,7 @@ impl<O: Offset> MutableUtf8Array<O> {
     /// Extends the [`MutableUtf8Array`] from an iterator of values of trusted len.
     /// This differs from `extended_trusted_len_unchecked` which accepts iterator of optional
     /// values.
+    ///
     /// # Safety
     /// The iterator must be trusted len.
     #[inline]
@@ -357,6 +357,7 @@ impl<O: Offset> MutableUtf8Array<O> {
     }
 
     /// Extends [`MutableUtf8Array`] from an iterator of trusted len.
+    ///
     /// # Safety
     /// The iterator must be trusted len.
     #[inline]
@@ -376,6 +377,7 @@ impl<O: Offset> MutableUtf8Array<O> {
     }
 
     /// Creates a [`MutableUtf8Array`] from an iterator of trusted length.
+    ///
     /// # Safety
     /// The iterator must be [`TrustedLen`](https://doc.rust-lang.org/std/iter/trait.TrustedLen.html).
     /// I.e. that `size_hint().1` correctly reports its length.
@@ -404,6 +406,7 @@ impl<O: Offset> MutableUtf8Array<O> {
     }
 
     /// Creates a [`MutableUtf8Array`] from an iterator of trusted length of `&str`.
+    ///
     /// # Safety
     /// The iterator must be [`TrustedLen`](https://doc.rust-lang.org/std/iter/trait.TrustedLen.html).
     /// I.e. that `size_hint().1` correctly reports its length.
@@ -440,6 +443,7 @@ impl<O: Offset> MutableUtf8Array<O> {
     }
 
     /// Creates a [`MutableUtf8Array`] from an falible iterator of trusted length.
+    ///
     /// # Safety
     /// The iterator must be [`TrustedLen`](https://doc.rust-lang.org/std/iter/trait.TrustedLen.html).
     /// I.e. that `size_hint().1` correctly reports its length.
@@ -518,9 +522,8 @@ impl<O: Offset, T: AsRef<str>> TryPush<Option<T>> for MutableUtf8Array<O> {
             Some(value) => {
                 self.values.try_push(value.as_ref())?;
 
-                match &mut self.validity {
-                    Some(validity) => validity.push(true),
-                    None => {},
+                if let Some(validity) = &mut self.validity {
+                    validity.push(true)
                 }
             },
             None => {
