@@ -270,6 +270,16 @@ fn eq(lhs: &Bitmap, rhs: &Bitmap) -> bool {
     lhs_remainder.zip(rhs_remainder).all(|(x, y)| x == y)
 }
 
+pub fn num_intersections_with(lhs: &Bitmap, rhs: &Bitmap) -> usize {
+    binary_fold(
+        lhs,
+        rhs,
+        |lhs, rhs| (lhs & rhs).count_ones() as usize,
+        0,
+        |lhs, rhs| lhs + rhs,
+    )
+}
+
 pub fn intersects_with(lhs: &Bitmap, rhs: &Bitmap) -> bool {
     binary_fold(
         lhs,
@@ -290,6 +300,38 @@ pub fn intersects_with_mut(lhs: &MutableBitmap, rhs: &MutableBitmap) -> bool {
     )
 }
 
+pub fn num_edges(lhs: &Bitmap) -> usize {
+    if lhs.is_empty() {
+        return 0;
+    }
+
+    // @TODO: If is probably quite inefficient to do it like this because now either one is not
+    // aligned. Maybe, we can implement a smarter way to do this.
+    binary_fold(
+        &unsafe { lhs.clone().sliced_unchecked(0, lhs.len() - 1) },
+        &unsafe { lhs.clone().sliced_unchecked(1, lhs.len() - 1) },
+        |l, r| (l ^ r).count_ones() as usize,
+        0,
+        |acc, v| acc + v,
+    )
+}
+
+/// Compute `out[i] = if selector[i] { truthy[i] } else { falsy }`.
+pub fn select_constant(selector: &Bitmap, truthy: &Bitmap, falsy: bool) -> Bitmap {
+    let falsy_mask: u64 = if falsy {
+        0xFFFF_FFFF_FFFF_FFFF
+    } else {
+        0x0000_0000_0000_0000
+    };
+
+    binary(selector, truthy, |s, t| (s & t) | (!s & falsy_mask))
+}
+
+/// Compute `out[i] = if selector[i] { truthy[i] } else { falsy[i] }`.
+pub fn select(selector: &Bitmap, truthy: &Bitmap, falsy: &Bitmap) -> Bitmap {
+    ternary(selector, truthy, falsy, |s, t, f| (s & t) | (!s & f))
+}
+
 impl PartialEq for Bitmap {
     fn eq(&self, other: &Self) -> bool {
         eq(self, other)
