@@ -8,10 +8,10 @@ use crate::bitmap::{Bitmap, MutableBitmap};
 use crate::datatypes::ArrowDataType;
 use crate::types::NativeType;
 
-/// Applies an unary and infallible function to a [`PrimitiveArray`]. This is the
-/// fastest way to perform an operation on a [`PrimitiveArray`] when the benefits
-/// of a vectorized operation outweighs the cost of branching nulls and
-/// non-nulls.
+/// Applies an unary and infallible function to a [`PrimitiveArray`].
+///
+/// This is the /// fastest way to perform an operation on a [`PrimitiveArray`] when the benefits
+/// of a vectorized operation outweighs the cost of branching nulls and non-nulls.
 ///
 /// # Implementation
 /// This will apply the function for all values, including those on null slots.
@@ -105,9 +105,6 @@ where
 {
     let mut mut_bitmap = MutableBitmap::with_capacity(array.len());
 
-    // TODO: Clippy lint is broken, remove attr once fixed.
-    // https://github.com/rust-lang/rust-clippy/issues/12580
-    #[cfg_attr(feature = "nightly", allow(clippy::manual_unwrap_or_default))]
     let values = array
         .values()
         .iter()
@@ -134,11 +131,14 @@ where
     PrimitiveArray::<O>::new(data_type, values, validity)
 }
 
-/// Applies a binary operations to two primitive arrays. This is the fastest
-/// way to perform an operation on two primitive array when the benefits of a
+/// Applies a binary operations to two primitive arrays.
+///
+/// This is the fastest way to perform an operation on two primitive array when the benefits of a
 /// vectorized operation outweighs the cost of branching nulls and non-nulls.
+///
 /// # Errors
 /// This function errors iff the arrays have a different length.
+///
 /// # Implementation
 /// This will apply the function for all values, including those on null slots.
 /// This implies that the operation must be infallible for any value of the
@@ -257,9 +257,6 @@ where
 
     let mut mut_bitmap = MutableBitmap::with_capacity(lhs.len());
 
-    // TODO: Clippy lint is broken, remove attr once fixed.
-    // https://github.com/rust-lang/rust-clippy/issues/12580
-    #[cfg_attr(feature = "nightly", allow(clippy::manual_unwrap_or_default))]
     let values = lhs
         .values()
         .iter()
