@@ -1,6 +1,17 @@
+use std::sync::atomic::{AtomicBool, Ordering};
+
 use atoi::FromRadix10SignedChecked;
 use num_traits::Euclid;
 
+static TRIM_DECIMAL_ZEROS: AtomicBool = AtomicBool::new(false);
+
+pub fn get_trim_decimal_zeros() -> bool {
+    TRIM_DECIMAL_ZEROS.load(Ordering::Relaxed)
+}
+pub fn set_trim_decimal_zeros(trim: Option<bool>) {
+    TRIM_DECIMAL_ZEROS.store(trim.unwrap_or(false), Ordering::Relaxed)
+}
+
 /// Count the number of b'0's at the beginning of a slice.
 fn leading_zeros(bytes: &[u8]) -> u8 {
     bytes.iter().take_while(|byte| **byte == b'0').count() as u8
