@@ -1,8 +1,8 @@
 use std::io::Read;
 
-use ahash::AHashMap;
 use arrow_format::ipc::planus::ReadAsRoot;
 use polars_error::{polars_bail, polars_err, PolarsError, PolarsResult};
+use polars_utils::aliases::PlHashMap;
 
 use super::super::CONTINUATION_MARKER;
 use super::common::*;
@@ -11,7 +11,7 @@ use super::{Dictionaries, OutOfSpecKind};
 use crate::array::Array;
 use crate::datatypes::ArrowSchema;
 use crate::io::ipc::IpcSchema;
-use crate::record_batch::RecordBatch;
+use crate::record_batch::RecordBatchT;
 
 /// Metadata of an Arrow IPC stream, written at the start of the stream
 #[derive(Debug, Clone)]
@@ -67,7 +67,7 @@ pub enum StreamState {
     /// A live stream without data
     Waiting,
     /// Next item in the stream
-    Some(RecordBatch<Box<dyn Array>>),
+    Some(RecordBatchT<Box<dyn Array>>),
 }
 
 impl StreamState {
@@ -76,7 +76,7 @@ impl StreamState {
     /// # Panics
     ///
     /// If the `StreamState` was `Waiting`.
-    pub fn unwrap(self) -> RecordBatch<Box<dyn Array>> {
+    pub fn unwrap(self) -> RecordBatchT<Box<dyn Array>> {
         if let StreamState::Some(batch) = self {
             batch
         } else {
@@ -93,7 +93,7 @@ fn read_next<R: Read>(
     dictionaries: &mut Dictionaries,
     message_buffer: &mut Vec<u8>,
     data_buffer: &mut Vec<u8>,
-    projection: &Option<(Vec<usize>, AHashMap<usize, usize>, ArrowSchema)>,
+    projection: &Option<(Vec<usize>, PlHashMap<usize, usize>, ArrowSchema)>,
     scratch: &mut Vec<u8>,
 ) -> PolarsResult<Option<StreamState>> {
     // determine metadata length
@@ -238,7 +238,7 @@ pub struct StreamReader<R: Read> {
     finished: bool,
     data_buffer: Vec<u8>,
     message_buffer: Vec<u8>,
-    projection: Option<(Vec<usize>, AHashMap<usize, usize>, ArrowSchema)>,
+    projection: Option<(Vec<usize>, PlHashMap<usize, usize>, ArrowSchema)>,
     scratch: Vec<u8>,
 }
 
