@@ -8,13 +8,14 @@
 use arrow::types::NativeType;
 
 pub mod arithmetic;
+pub mod arity;
 pub mod comparisons;
 pub mod filter;
 pub mod float_sum;
 pub mod if_then_else;
 pub mod min_max;
-
-pub mod arity;
+pub mod size;
+pub mod unique;
 
 // Trait to enable the scalar blanket implementation.
 pub trait NotSimdPrimitive: NativeType {}
