@@ -1,12 +1,10 @@
 //! The typed heart of every Series column.
 use std::iter::Map;
-use std::marker::PhantomData;
-use std::sync::Arc;
+use std::sync::{Arc, RwLockReadGuard, RwLockWriteGuard};
 
 use arrow::array::*;
 use arrow::bitmap::Bitmap;
-#[cfg(feature = "serde")]
-use serde::{Deserialize, Serialize};
+use polars_compute::filter::filter_with_bitmap;
 
 use crate::prelude::*;
 
@@ -19,6 +17,7 @@ pub mod collect;
 pub mod comparison;
 pub mod float;
 pub mod iterator;
+pub mod metadata;
 #[cfg(feature = "ndarray")]
 pub(crate) mod ndarray;
 
@@ -37,6 +36,8 @@ pub(crate) mod logical;
 pub mod object;
 #[cfg(feature = "random")]
 mod random;
+#[cfg(feature = "dtype-struct")]
+mod struct_;
 #[cfg(any(
     feature = "temporal",
     feature = "dtype-datetime",
@@ -51,8 +52,13 @@ use std::slice::Iter;
 
 use arrow::legacy::kernels::concatenate::concatenate_owned_unchecked;
 use arrow::legacy::prelude::*;
-use bitflags::bitflags;
+#[cfg(feature = "dtype-struct")]
+pub use struct_::StructChunked;
 
+use self::metadata::{
+    IMMetadata, Metadata, MetadataFlags, MetadataMerge, MetadataProperties, MetadataReadGuard,
+    MetadataTrait,
+};
 use crate::series::IsSorted;
 use crate::utils::{first_non_null, last_non_null};
 
@@ -69,8 +75,8 @@ pub type ChunkLenIter<'a> = std::iter::Map<std::slice::Iter<'a, ArrayRef>, fn(&A
 ///
 /// ```rust
 /// # use polars_core::prelude::*;
-/// fn apply_cosine_and_cast(ca: &Float32Chunked) -> Float64Chunked {
-///     ca.apply_values_generic(|v| v.cos() as f64)
+/// fn apply_cosine_and_cast(ca: &Float32Chunked) -> Float32Chunked {
+///     ca.apply_values(|v| v.cos())
 /// }
 /// ```
 ///
@@ -138,93 +144,266 @@ pub type ChunkLenIter<'a> = std::iter::Map<std::slice::Iter<'a, ArrayRef>, fn(&A
 pub struct ChunkedArray<T: PolarsDataType> {
     pub(crate) field: Arc<Field>,
     pub(crate) chunks: Vec<ArrayRef>,
-    phantom: PhantomData<T>,
-    pub(crate) bit_settings: Settings,
+
+    // While it might be temping to make Arc<...> into Option<Arc<...>>, it is very difficult to
+    // combine with the interior mutability that IMMetadata provides.
+    pub(crate) md: Arc<IMMetadata<T>>,
+
     length: IdxSize,
     null_count: IdxSize,
 }
 
-bitflags! {
-    #[derive(Default, Debug, Clone, Copy,PartialEq)]
-    #[cfg_attr(feature = "serde", derive(Serialize, Deserialize), serde(transparent))]
-    pub struct Settings: u8 {
-        const SORTED_ASC = 0x01;
-        const SORTED_DSC = 0x02;
-        const FAST_EXPLODE_LIST = 0x04;
-    }
-}
-
-impl Settings {
-    pub fn set_sorted_flag(&mut self, sorted: IsSorted) {
-        match sorted {
-            IsSorted::Not => {
-                self.remove(Settings::SORTED_ASC | Settings::SORTED_DSC);
-            },
-            IsSorted::Ascending => {
-                self.remove(Settings::SORTED_DSC);
-                self.insert(Settings::SORTED_ASC)
-            },
-            IsSorted::Descending => {
-                self.remove(Settings::SORTED_ASC);
-                self.insert(Settings::SORTED_DSC)
-            },
-        }
-    }
-
-    pub fn get_sorted_flag(&self) -> IsSorted {
-        if self.contains(Settings::SORTED_ASC) {
-            IsSorted::Ascending
-        } else if self.contains(Settings::SORTED_DSC) {
-            IsSorted::Descending
-        } else {
-            IsSorted::Not
-        }
+impl<T: PolarsDataType> ChunkedArray<T>
+where
+    Metadata<T>: MetadataTrait,
+{
+    /// Attempt to get a reference to the trait object containing the [`ChunkedArray`]'s [`Metadata`]
+    ///
+    /// This fails if there is a need to block.
+    pub fn metadata_dyn(&self) -> Option<RwLockReadGuard<dyn MetadataTrait>> {
+        self.md.as_ref().upcast().try_read().ok()
     }
 }
 
 impl<T: PolarsDataType> ChunkedArray<T> {
+    fn should_rechunk(&self) -> bool {
+        self.chunks.len() > 1 && self.chunks.len() > self.len() / 3
+    }
+
+    fn optional_rechunk(self) -> Self {
+        // Rechunk if we have many small chunks.
+        if self.should_rechunk() {
+            self.rechunk()
+        } else {
+            self
+        }
+    }
+
+    /// Create a new [`ChunkedArray`] and compute its `length` and `null_count`.
+    ///
+    /// If you want to explicitly the `length` and `null_count`, look at
+    /// [`ChunkedArray::new_with_dims`]
+    fn new_with_compute_len(field: Arc<Field>, chunks: Vec<ArrayRef>) -> Self {
+        unsafe {
+            let mut chunked_arr = Self::new_with_dims(field, chunks, 0, 0);
+            chunked_arr.compute_len();
+            chunked_arr
+        }
+    }
+
+    /// Create a new [`ChunkedArray`] and explicitly set its `length` and `null_count`.
+    /// # Safety
+    /// The length and null_count must be correct.
+    pub unsafe fn new_with_dims(
+        field: Arc<Field>,
+        chunks: Vec<ArrayRef>,
+        length: IdxSize,
+        null_count: IdxSize,
+    ) -> Self {
+        Self {
+            field,
+            chunks,
+            md: Arc::new(IMMetadata::default()),
+
+            length,
+            null_count,
+        }
+    }
+
+    /// Get a guard to read the [`ChunkedArray`]'s [`Metadata`]
+    pub fn metadata(&self) -> MetadataReadGuard<T> {
+        self.md.as_ref().try_read().map_or(
+            MetadataReadGuard::Locked(&Metadata::DEFAULT),
+            MetadataReadGuard::Unlocked,
+        )
+    }
+
+    /// Get a guard to read/write the [`ChunkedArray`]'s [`Metadata`]
+    pub fn interior_mut_metadata(&self) -> RwLockWriteGuard<Metadata<T>> {
+        self.md.as_ref().write()
+    }
+
+    /// Get a reference to [`Arc`] that contains the [`ChunkedArray`]'s [`Metadata`]
+    pub fn metadata_arc(&self) -> &Arc<IMMetadata<T>> {
+        &self.md
+    }
+
+    /// Get a [`Arc`] that contains the [`ChunkedArray`]'s [`Metadata`]
+    pub fn metadata_owned_arc(&self) -> Arc<IMMetadata<T>> {
+        self.md.clone()
+    }
+
+    /// Get a mutable reference to the [`Arc`] that contains the [`ChunkedArray`]'s [`Metadata`]
+    pub fn metadata_mut(&mut self) -> &mut Arc<IMMetadata<T>> {
+        &mut self.md
+    }
+
     pub(crate) fn is_sorted_ascending_flag(&self) -> bool {
-        self.bit_settings.contains(Settings::SORTED_ASC)
+        self.metadata().is_sorted_ascending()
     }
 
     pub(crate) fn is_sorted_descending_flag(&self) -> bool {
-        self.bit_settings.contains(Settings::SORTED_DSC)
+        self.metadata().is_sorted_descending()
     }
 
     /// Whether `self` is sorted in any direction.
     pub(crate) fn is_sorted_any(&self) -> bool {
-        self.is_sorted_ascending_flag() || self.is_sorted_descending_flag()
+        self.metadata().is_sorted_any()
     }
 
     pub fn unset_fast_explode_list(&mut self) {
-        self.bit_settings.remove(Settings::FAST_EXPLODE_LIST)
+        self.set_fast_explode_list(false)
     }
 
-    pub fn get_flags(&self) -> Settings {
-        self.bit_settings
+    pub fn set_fast_explode_list(&mut self, value: bool) {
+        Arc::make_mut(self.metadata_mut())
+            .get_mut()
+            .set_fast_explode_list(value)
+    }
+
+    pub fn get_fast_explode_list(&self) -> bool {
+        self.get_flags().get_fast_explode_list()
+    }
+
+    pub fn get_flags(&self) -> MetadataFlags {
+        self.metadata().get_flags()
     }
 
     /// Set flags for the [`ChunkedArray`]
-    pub(crate) fn set_flags(&mut self, flags: Settings) {
-        self.bit_settings = flags;
+    pub(crate) fn set_flags(&mut self, flags: MetadataFlags) {
+        // @TODO: This should probably just not be here
+        let md = Arc::make_mut(self.metadata_mut());
+        md.get_mut().set_flags(flags);
     }
 
     pub fn is_sorted_flag(&self) -> IsSorted {
-        self.bit_settings.get_sorted_flag()
+        self.metadata().is_sorted()
     }
 
     /// Set the 'sorted' bit meta info.
     pub fn set_sorted_flag(&mut self, sorted: IsSorted) {
-        self.bit_settings.set_sorted_flag(sorted)
+        Arc::make_mut(self.metadata_mut())
+            .get_mut()
+            .set_sorted_flag(sorted)
     }
 
     /// Set the 'sorted' bit meta info.
     pub fn with_sorted_flag(&self, sorted: IsSorted) -> Self {
         let mut out = self.clone();
-        out.bit_settings.set_sorted_flag(sorted);
+        out.set_sorted_flag(sorted);
         out
     }
 
+    pub fn get_min_value(&self) -> Option<T::OwnedPhysical> {
+        self.metadata().get_min_value().cloned()
+    }
+
+    pub fn get_max_value(&self) -> Option<T::OwnedPhysical> {
+        self.metadata().get_max_value().cloned()
+    }
+
+    pub fn get_distinct_count(&self) -> Option<IdxSize> {
+        self.metadata().get_distinct_count()
+    }
+
+    pub fn merge_metadata(&mut self, md: Metadata<T>) {
+        let self_md = self.metadata_mut();
+        let self_md = self_md.as_ref();
+        let self_md = self_md.read();
+
+        match self_md.merge(md) {
+            MetadataMerge::Keep => {},
+            MetadataMerge::New(md) => {
+                let md = Arc::new(IMMetadata::new(md));
+                drop(self_md);
+                self.md = md;
+            },
+            MetadataMerge::Conflict => {
+                panic!("Trying to merge metadata, but got conflicting information")
+            },
+        }
+    }
+
+    /// Copies [`Metadata`] properties specified by `props`  from `other` with different underlying [`PolarsDataType`] into
+    /// `self`.
+    ///
+    /// This does not copy the properties with a different type between the [`Metadata`]s (e.g.
+    /// `min_value` and `max_value`) and will panic on debug builds if that is attempted.
+    #[inline(always)]
+    pub fn copy_metadata_cast<O: PolarsDataType>(
+        &mut self,
+        other: &ChunkedArray<O>,
+        props: MetadataProperties,
+    ) {
+        use MetadataProperties as P;
+
+        // If you add a property, add it here and below to ensure that metadata is copied
+        // properly.
+        debug_assert!(
+            {
+                props
+                    - (P::SORTED
+                        | P::FAST_EXPLODE_LIST
+                        | P::MIN_VALUE
+                        | P::MAX_VALUE
+                        | P::DISTINCT_COUNT)
+            }
+            .is_empty(),
+            "A MetadataProperty was not added to the copy_metadata_cast check"
+        );
+
+        debug_assert!(!props.contains(P::MIN_VALUE));
+        debug_assert!(!props.contains(P::MAX_VALUE));
+
+        // We add a fast path here for if both metadatas are empty, as this is quite a common case.
+        if props.is_empty() {
+            return;
+        }
+
+        let other_md = other.metadata();
+
+        if other_md.is_empty() {
+            return;
+        }
+
+        let other_md = other_md.filter_props_cast(props);
+        self.merge_metadata(other_md);
+    }
+
+    /// Copies [`Metadata`] properties specified by `props` from `other` into `self`.
+    #[inline(always)]
+    pub fn copy_metadata(&mut self, other: &Self, props: MetadataProperties) {
+        use MetadataProperties as P;
+
+        // If you add a property add it here and below to ensure that metadata is copied properly.
+        debug_assert!(
+            {
+                props
+                    - (P::SORTED
+                        | P::FAST_EXPLODE_LIST
+                        | P::MIN_VALUE
+                        | P::MAX_VALUE
+                        | P::DISTINCT_COUNT)
+            }
+            .is_empty(),
+            "A MetadataProperty was not added to the copy_metadata check"
+        );
+
+        // We add a fast path here for if both metadatas are empty, as this is quite a common case.
+        if props.is_empty() {
+            return;
+        }
+
+        let other_md = other.metadata();
+
+        if other_md.is_empty() {
+            return;
+        }
+
+        let other_md = other_md.filter_props(props);
+        self.merge_metadata(other_md);
+    }
+
     /// Get the index of the first non null value in this [`ChunkedArray`].
     pub fn first_non_null(&self) -> Option<usize> {
         if self.null_count() == self.len() {
@@ -283,6 +462,31 @@ impl<T: PolarsDataType> ChunkedArray<T> {
         }
     }
 
+    pub fn drop_nulls(&self) -> Self {
+        if self.null_count() == 0 {
+            self.clone()
+        } else {
+            let chunks = self
+                .downcast_iter()
+                .map(|arr| {
+                    if arr.null_count() == 0 {
+                        arr.to_boxed()
+                    } else {
+                        filter_with_bitmap(arr, arr.validity().unwrap())
+                    }
+                })
+                .collect();
+            unsafe {
+                Self::new_with_dims(
+                    self.field.clone(),
+                    chunks,
+                    (self.len() - self.null_count()) as IdxSize,
+                    0,
+                )
+            }
+        }
+    }
+
     /// Get the buffer of bits representing null values
     #[inline]
     #[allow(clippy::type_complexity)]
@@ -294,10 +498,9 @@ impl<T: PolarsDataType> ChunkedArray<T> {
     }
 
     #[inline]
-    /// Return if any the chunks in this [`ChunkedArray`] have a validity bitmap.
-    /// no bitmap means no null values.
-    pub fn has_validity(&self) -> bool {
-        self.iter_validities().any(|valid| valid.is_some())
+    /// Return if any the chunks in this [`ChunkedArray`] have nulls.
+    pub fn has_nulls(&self) -> bool {
+        self.null_count > 0
     }
 
     /// Shrink the capacity of this array to fit its length.
@@ -307,15 +510,16 @@ impl<T: PolarsDataType> ChunkedArray<T> {
 
     pub fn clear(&self) -> Self {
         // SAFETY: we keep the correct dtype
-        unsafe {
-            self.copy_with_chunks(
-                vec![new_empty_array(
-                    self.chunks.first().unwrap().data_type().clone(),
-                )],
-                true,
-                true,
-            )
-        }
+        let mut ca = unsafe {
+            self.copy_with_chunks(vec![new_empty_array(
+                self.chunks.first().unwrap().data_type().clone(),
+            )])
+        };
+
+        use MetadataProperties as P;
+        ca.copy_metadata(self, P::SORTED | P::FAST_EXPLODE_LIST);
+
+        ca
     }
 
     /// Unpack a [`Series`] to the same physical type.
@@ -389,19 +593,8 @@ impl<T: PolarsDataType> ChunkedArray<T> {
     ///
     /// # Safety
     /// The caller must ensure the dtypes of the chunks are correct
-    unsafe fn copy_with_chunks(
-        &self,
-        chunks: Vec<ArrayRef>,
-        keep_sorted: bool,
-        keep_fast_explode: bool,
-    ) -> Self {
-        Self::from_chunks_and_metadata(
-            chunks,
-            self.field.clone(),
-            self.bit_settings,
-            keep_sorted,
-            keep_fast_explode,
-        )
+    unsafe fn copy_with_chunks(&self, chunks: Vec<ArrayRef>) -> Self {
+        Self::new_with_compute_len(self.field.clone(), chunks)
     }
 
     /// Get data type of [`ChunkedArray`].
@@ -690,8 +883,7 @@ impl<T: PolarsDataType> Clone for ChunkedArray<T> {
         ChunkedArray {
             field: self.field.clone(),
             chunks: self.chunks.clone(),
-            phantom: PhantomData,
-            bit_settings: self.bit_settings,
+            md: self.md.clone(),
             length: self.length,
             null_count: self.null_count,
         }
@@ -740,7 +932,11 @@ pub(crate) fn to_primitive<T: PolarsNumericType>(
     values: Vec<T::Native>,
     validity: Option<Bitmap>,
 ) -> PrimitiveArray<T::Native> {
-    PrimitiveArray::new(T::get_dtype().to_arrow(true), values.into(), validity)
+    PrimitiveArray::new(
+        T::get_dtype().to_arrow(CompatLevel::newest()),
+        values.into(),
+        validity,
+    )
 }
 
 pub(crate) fn to_array<T: PolarsNumericType>(
@@ -755,8 +951,7 @@ impl<T: PolarsDataType> Default for ChunkedArray<T> {
         ChunkedArray {
             field: Arc::new(Field::new("default", DataType::Null)),
             chunks: Default::default(),
-            phantom: PhantomData,
-            bit_settings: Default::default(),
+            md: Arc::new(IMMetadata::default()),
             length: 0,
             null_count: 0,
         }
@@ -858,7 +1053,7 @@ pub(crate) mod test {
     fn slice() {
         let mut first = UInt32Chunked::new("first", &[0, 1, 2]);
         let second = UInt32Chunked::new("second", &[3, 4, 5]);
-        first.append(&second);
+        first.append(&second).unwrap();
         assert_slice_equal(&first.slice(0, 3), &[0, 1, 2]);
         assert_slice_equal(&first.slice(0, 4), &[0, 1, 2, 3]);
         assert_slice_equal(&first.slice(1, 4), &[1, 2, 3, 4]);
