@@ -15,6 +15,8 @@ mod is_valid;
 mod iterator;
 pub mod registry;
 
+pub use extension::set_polars_allow_extension;
+
 #[derive(Debug, Clone)]
 pub struct ObjectArray<T>
 where
@@ -186,6 +188,16 @@ where
         self.offset = offset;
     }
 
+    fn split_at_boxed(&self, offset: usize) -> (Box<dyn Array>, Box<dyn Array>) {
+        let (lhs, rhs) = Splitable::split_at(self, offset);
+        (Box::new(lhs), Box::new(rhs))
+    }
+
+    unsafe fn split_at_boxed_unchecked(&self, offset: usize) -> (Box<dyn Array>, Box<dyn Array>) {
+        let (lhs, rhs) = unsafe { Splitable::split_at_unchecked(self, offset) };
+        (Box::new(lhs), Box::new(rhs))
+    }
+
     fn len(&self) -> usize {
         self.len
     }
@@ -214,6 +226,29 @@ where
     }
 }
 
+impl<T: PolarsObject> Splitable for ObjectArray<T> {
+    fn check_bound(&self, offset: usize) -> bool {
+        offset <= self.len()
+    }
+
+    unsafe fn _split_at_unchecked(&self, offset: usize) -> (Self, Self) {
+        (
+            Self {
+                values: self.values.clone(),
+                null_bitmap: self.null_bitmap.clone(),
+                len: offset,
+                offset: self.offset,
+            },
+            Self {
+                values: self.values.clone(),
+                null_bitmap: self.null_bitmap.clone(),
+                len: self.len() - offset,
+                offset: self.offset + offset,
+            },
+        )
+    }
+}
+
 impl<T: PolarsObject> StaticArray for ObjectArray<T> {
     type ValueT<'a> = &'a T;
     type ZeroableValueT<'a> = Option<&'a T>;
