@@ -17,6 +17,7 @@ pub use var::*;
 use super::float_sorted_arg_max::{
     float_arg_max_sorted_ascending, float_arg_max_sorted_descending,
 };
+use crate::chunked_array::metadata::MetadataEnv;
 use crate::chunked_array::ChunkedArray;
 use crate::datatypes::{BooleanChunked, PolarsNumericType};
 use crate::prelude::*;
@@ -91,8 +92,10 @@ where
         if self.null_count() == self.len() {
             return None;
         }
+
         // There is at least one non-null value.
-        match self.is_sorted_flag() {
+
+        let result = match self.is_sorted_flag() {
             IsSorted::Ascending => {
                 let idx = self.first_non_null().unwrap();
                 unsafe { self.get_unchecked(idx) }
@@ -105,7 +108,13 @@ where
                 .downcast_iter()
                 .filter_map(MinMaxKernel::min_ignore_nan_kernel)
                 .reduce(MinMax::min_ignore_nan),
+        };
+
+        if MetadataEnv::experimental_enabled() {
+            self.interior_mut_metadata().set_min_value(result);
         }
+
+        result
     }
 
     fn max(&self) -> Option<T::Native> {
@@ -113,7 +122,8 @@ where
             return None;
         }
         // There is at least one non-null value.
-        match self.is_sorted_flag() {
+
+        let result = match self.is_sorted_flag() {
             IsSorted::Ascending => {
                 let idx = if T::get_dtype().is_float() {
                     float_arg_max_sorted_ascending(self)
@@ -136,7 +146,13 @@ where
                 .downcast_iter()
                 .filter_map(MinMaxKernel::max_ignore_nan_kernel)
                 .reduce(MinMax::max_ignore_nan),
+        };
+
+        if MetadataEnv::experimental_enabled() {
+            self.interior_mut_metadata().set_max_value(result);
         }
+
+        result
     }
 
     fn min_max(&self) -> Option<(T::Native, T::Native)> {
@@ -144,7 +160,8 @@ where
             return None;
         }
         // There is at least one non-null value.
-        match self.is_sorted_flag() {
+
+        let result = match self.is_sorted_flag() {
             IsSorted::Ascending => {
                 let min = unsafe { self.get_unchecked(self.first_non_null().unwrap()) };
                 let max = {
@@ -181,7 +198,21 @@ where
                         MinMax::max_ignore_nan(max1, max2),
                     )
                 }),
+        };
+
+        if MetadataEnv::experimental_enabled() {
+            let (min, max) = match result {
+                Some((min, max)) => (Some(min), Some(max)),
+                None => (None, None),
+            };
+
+            let mut md = self.interior_mut_metadata();
+
+            md.set_min_value(min);
+            md.set_max_value(max);
         }
+
+        result
     }
 
     fn mean(&self) -> Option<f64> {
@@ -448,9 +479,6 @@ impl StringChunked {
 }
 
 impl ChunkAggSeries for StringChunked {
-    fn sum_reduce(&self) -> Scalar {
-        Scalar::new(DataType::String, AnyValue::Null)
-    }
     fn max_reduce(&self) -> Scalar {
         let av: AnyValue = self.max_str().into();
         Scalar::new(DataType::String, av.into_static().unwrap())
