@@ -1,9 +1,8 @@
-use arrow::bitmap::MutableBitmap;
+use arrow::bitmap::{Bitmap, MutableBitmap};
 use arrow::legacy::kernels::set::set_at_nulls;
-use arrow::legacy::trusted_len::FromIteratorReversed;
-use arrow::legacy::utils::FromTrustedLenIterator;
 use bytemuck::Zeroable;
 use num_traits::{Bounded, NumCast, One, Zero};
+use polars_utils::itertools::Itertools;
 
 use crate::prelude::*;
 
@@ -65,236 +64,79 @@ impl Series {
     /// example();
     /// ```
     pub fn fill_null(&self, strategy: FillNullStrategy) -> PolarsResult<Series> {
-        let logical_type = self.dtype();
-        let s = self.to_physical_repr();
+        // Nothing to fill.
+        let nc = self.null_count();
+        if nc == 0
+            || (nc == self.len()
+                && matches!(
+                    strategy,
+                    FillNullStrategy::Forward(_)
+                        | FillNullStrategy::Backward(_)
+                        | FillNullStrategy::Max
+                        | FillNullStrategy::Min
+                        | FillNullStrategy::MaxBound
+                        | FillNullStrategy::MinBound
+                        | FillNullStrategy::Mean
+                ))
+        {
+            return Ok(self.clone());
+        }
 
-        use DataType::*;
-        let out = match s.dtype() {
-            Boolean => fill_null_bool(s.bool().unwrap(), strategy),
-            String => {
-                let s = unsafe { s.cast_unchecked(&Binary)? };
-                let out = s.fill_null(strategy)?;
-                return unsafe { out.cast_unchecked(&String) };
+        let physical_type = self.dtype().to_physical();
+
+        match strategy {
+            FillNullStrategy::Forward(None) if !physical_type.is_numeric() => {
+                fill_forward_gather(self)
             },
-            Binary => {
-                let ca = s.binary().unwrap();
-                fill_null_binary(ca, strategy).map(|ca| ca.into_series())
+            FillNullStrategy::Forward(Some(limit)) => fill_forward_gather_limit(self, limit),
+            FillNullStrategy::Backward(None) if !physical_type.is_numeric() => {
+                fill_backward_gather(self)
             },
-            List(_) => {
-                let ca = s.list().unwrap();
-                fill_null_list(ca, strategy).map(|ca| ca.into_series())
+            FillNullStrategy::Backward(Some(limit)) => fill_backward_gather_limit(self, limit),
+            _ => {
+                let logical_type = self.dtype();
+                let s = self.to_physical_repr();
+                use DataType::*;
+                let out = match s.dtype() {
+                    Boolean => fill_null_bool(s.bool().unwrap(), strategy),
+                    String => {
+                        let s = unsafe { s.cast_unchecked(&Binary)? };
+                        let out = s.fill_null(strategy)?;
+                        return unsafe { out.cast_unchecked(&String) };
+                    },
+                    Binary => {
+                        let ca = s.binary().unwrap();
+                        fill_null_binary(ca, strategy).map(|ca| ca.into_series())
+                    },
+                    dt if dt.is_numeric() => {
+                        with_match_physical_numeric_polars_type!(dt, |$T| {
+                            let ca: &ChunkedArray<$T> = s.as_ref().as_ref().as_ref();
+                                fill_null_numeric(ca, strategy).map(|ca| ca.into_series())
+                        })
+                    },
+                    dt => {
+                        polars_bail!(InvalidOperation: "fill null strategy not yet supported for dtype: {}", dt)
+                    },
+                }?;
+                unsafe { out.cast_unchecked(logical_type) }
             },
-            dt if dt.is_numeric() => {
-                with_match_physical_numeric_polars_type!(dt, |$T| {
-                    let ca: &ChunkedArray<$T> = s.as_ref().as_ref().as_ref();
-                        fill_null_numeric(ca, strategy).map(|ca| ca.into_series())
-                })
-            },
-            _ => todo!(),
-        }?;
-        unsafe { out.cast_unchecked(logical_type) }
+        }
     }
 }
 
-// Utility trait to make generics work
-trait LocalCopy {
-    fn cheap_clone(&self) -> Self;
-}
-
-impl<T: Copy> LocalCopy for T {
-    #[inline]
-    fn cheap_clone(&self) -> Self {
-        *self
-    }
-}
-
-impl LocalCopy for Series {
-    #[inline]
-    fn cheap_clone(&self) -> Self {
-        self.clone()
-    }
-}
-
-fn fill_forward_limit<'a, T, K, I>(ca: &'a ChunkedArray<T>, limit: IdxSize) -> ChunkedArray<T>
-where
-    &'a ChunkedArray<T>: IntoIterator<IntoIter = I>,
-    K: LocalCopy,
-    I: TrustedLen<Item = Option<K>>,
-    T: PolarsDataType,
-    ChunkedArray<T>: FromTrustedLenIterator<Option<K>>,
-{
-    let mut cnt = 0;
-    let mut previous = None;
-    ca.into_iter()
-        .map(|opt_v| match opt_v {
-            Some(v) => {
-                cnt = 0;
-                previous = Some(v.cheap_clone());
-                Some(v)
-            },
-            None => {
-                if cnt < limit {
-                    cnt += 1;
-                    previous.as_ref().map(|v| v.cheap_clone())
-                } else {
-                    None
-                }
-            },
-        })
-        .collect_trusted()
-}
-
-fn fill_backward_limit<'a, T, K, I>(ca: &'a ChunkedArray<T>, limit: IdxSize) -> ChunkedArray<T>
-where
-    &'a ChunkedArray<T>: IntoIterator<IntoIter = I>,
-    K: LocalCopy,
-    I: TrustedLen<Item = Option<K>> + DoubleEndedIterator,
-    T: PolarsDataType,
-    ChunkedArray<T>: FromIteratorReversed<Option<K>>,
-{
-    let mut cnt = 0;
-    let mut previous = None;
-    ca.into_iter()
-        .rev()
-        .map(|opt_v| match opt_v {
-            Some(v) => {
-                cnt = 0;
-                previous = Some(v.cheap_clone());
-                Some(v)
-            },
-            None => {
-                if cnt < limit {
-                    cnt += 1;
-                    previous.as_ref().map(|v| v.cheap_clone())
-                } else {
-                    None
-                }
-            },
-        })
-        .collect_reversed()
-}
-
-fn fill_backward_limit_binary(ca: &BinaryChunked, limit: IdxSize) -> BinaryChunked {
-    let mut cnt = 0;
-    let mut previous = None;
-    let out: BinaryChunked = ca
-        .into_iter()
-        .rev()
-        .map(|opt_v| match opt_v {
-            Some(v) => {
-                cnt = 0;
-                previous = Some(v);
-                Some(v)
-            },
-            None => {
-                if cnt < limit {
-                    cnt += 1;
-                    previous
-                } else {
-                    None
-                }
-            },
-        })
-        .collect_trusted();
-    out.into_iter().rev().collect_trusted()
-}
-
-fn fill_forward<'a, T, K, I>(ca: &'a ChunkedArray<T>) -> ChunkedArray<T>
-where
-    &'a ChunkedArray<T>: IntoIterator<IntoIter = I>,
-    K: LocalCopy,
-    I: TrustedLen<Item = Option<K>>,
-    T: PolarsDataType,
-    ChunkedArray<T>: FromTrustedLenIterator<Option<K>>,
-{
-    ca.into_iter()
-        .scan(None, |previous, opt_v| match opt_v {
-            Some(value) => {
-                *previous = Some(value.cheap_clone());
-                Some(Some(value))
-            },
-            None => Some(previous.as_ref().map(|v| v.cheap_clone())),
-        })
-        .collect_trusted()
-}
-
-fn fill_backward<'a, T, K, I>(ca: &'a ChunkedArray<T>) -> ChunkedArray<T>
-where
-    &'a ChunkedArray<T>: IntoIterator<IntoIter = I>,
-    K: Copy,
-    I: TrustedLen<Item = Option<K>> + DoubleEndedIterator,
-    T: PolarsDataType,
-    ChunkedArray<T>: FromIteratorReversed<Option<K>>,
-{
-    ca.into_iter()
-        .rev()
-        .scan(None, |previous, opt_v| match opt_v {
-            Some(value) => {
-                *previous = Some(value);
-                Some(Some(value))
-            },
-            None => Some(*previous),
-        })
-        .collect_reversed()
-}
-
-macro_rules! impl_fill_backward {
-    ($ca:ident, $ChunkedArray:ty) => {{
-        let ca: $ChunkedArray = $ca
-            .into_iter()
-            .rev()
-            .scan(None, |previous, opt_v| match opt_v {
-                Some(value) => {
-                    *previous = Some(value.cheap_clone());
-                    Some(Some(value))
-                },
-                None => Some(previous.as_ref().map(|s| s.cheap_clone())),
-            })
-            .collect_trusted();
-        ca.into_iter().rev().collect_trusted()
-    }};
-}
-
-macro_rules! impl_fill_backward_limit {
-    ($ca:ident, $ChunkedArray:ty, $limit:expr) => {{
-        let mut cnt = 0;
-        let mut previous = None;
-        let out: $ChunkedArray = $ca
-            .into_iter()
-            .rev()
-            .map(|opt_v| match opt_v {
-                Some(v) => {
-                    cnt = 0;
-                    previous = Some(v.cheap_clone());
-                    Some(v)
-                },
-                None => {
-                    if cnt < $limit {
-                        cnt += 1;
-                        previous.as_ref().map(|s| s.cheap_clone())
-                    } else {
-                        None
-                    }
-                },
-            })
-            .collect_trusted();
-        out.into_iter().rev().collect_trusted()
-    }};
-}
-
 fn fill_forward_numeric<'a, T, I>(ca: &'a ChunkedArray<T>) -> ChunkedArray<T>
 where
     T: PolarsDataType,
     &'a ChunkedArray<T>: IntoIterator<IntoIter = I>,
     I: TrustedLen + Iterator<Item = Option<T::Physical<'a>>>,
-    T::ZeroablePhysical<'a>: LocalCopy,
+    T::ZeroablePhysical<'a>: Copy,
 {
     // Compute values.
     let values: Vec<T::ZeroablePhysical<'a>> = ca
         .into_iter()
         .scan(T::ZeroablePhysical::zeroed(), |prev, v| {
-            *prev = v.map(|v| v.into()).unwrap_or(prev.cheap_clone());
-            Some(prev.cheap_clone())
+            *prev = v.map(|v| v.into()).unwrap_or(*prev);
+            Some(*prev)
         })
         .collect_trusted();
 
@@ -317,15 +159,15 @@ where
     T: PolarsDataType,
     &'a ChunkedArray<T>: IntoIterator<IntoIter = I>,
     I: TrustedLen + Iterator<Item = Option<T::Physical<'a>>> + DoubleEndedIterator,
-    T::ZeroablePhysical<'a>: LocalCopy,
+    T::ZeroablePhysical<'a>: Copy,
 {
     // Compute values.
     let values: Vec<T::ZeroablePhysical<'a>> = ca
         .into_iter()
         .rev()
         .scan(T::ZeroablePhysical::zeroed(), |prev, v| {
-            *prev = v.map(|v| v.into()).unwrap_or(prev.cheap_clone());
-            Some(prev.cheap_clone())
+            *prev = v.map(|v| v.into()).unwrap_or(*prev);
+            Some(*prev)
         })
         .collect_reversed();
 
@@ -355,14 +197,7 @@ where
     ChunkedArray<T>: ChunkAgg<T::Native>,
 {
     // Nothing to fill.
-    if ca.null_count() == 0 {
-        return Ok(ca.clone());
-    }
     let mut out = match strategy {
-        FillNullStrategy::Forward(None) => fill_forward_numeric(ca),
-        FillNullStrategy::Forward(Some(limit)) => fill_forward_limit(ca, limit),
-        FillNullStrategy::Backward(None) => fill_backward_numeric(ca),
-        FillNullStrategy::Backward(Some(limit)) => fill_backward_limit(ca, limit),
         FillNullStrategy::Min => {
             ca.fill_null_with_values(ChunkAgg::min(ca).ok_or_else(err_fill_null)?)?
         },
@@ -378,33 +213,123 @@ where
         FillNullStrategy::Zero => return ca.fill_null_with_values(Zero::zero()),
         FillNullStrategy::MinBound => return ca.fill_null_with_values(Bounded::min_value()),
         FillNullStrategy::MaxBound => return ca.fill_null_with_values(Bounded::max_value()),
+        FillNullStrategy::Forward(None) => fill_forward_numeric(ca),
+        FillNullStrategy::Backward(None) => fill_backward_numeric(ca),
+        // Handled earlier
+        FillNullStrategy::Forward(_) => unreachable!(),
+        FillNullStrategy::Backward(_) => unreachable!(),
     };
     out.rename(ca.name());
     Ok(out)
 }
 
+fn fill_with_gather<F: Fn(&Bitmap) -> Vec<IdxSize>>(
+    s: &Series,
+    bits_to_idx: F,
+) -> PolarsResult<Series> {
+    let s = s.rechunk();
+    let arr = s.chunks()[0].clone();
+    let validity = arr.validity().expect("nulls");
+
+    let idx = bits_to_idx(validity);
+
+    Ok(unsafe { s.take_unchecked_from_slice(&idx) })
+}
+
+fn fill_forward_gather(s: &Series) -> PolarsResult<Series> {
+    fill_with_gather(s, |validity| {
+        let mut last_valid = 0;
+        validity
+            .iter()
+            .enumerate_idx()
+            .map(|(i, v)| {
+                if v {
+                    last_valid = i;
+                    i
+                } else {
+                    last_valid
+                }
+            })
+            .collect::<Vec<_>>()
+    })
+}
+
+fn fill_forward_gather_limit(s: &Series, limit: IdxSize) -> PolarsResult<Series> {
+    fill_with_gather(s, |validity| {
+        let mut last_valid = 0;
+        let mut conseq_invalid_count = 0;
+        validity
+            .iter()
+            .enumerate_idx()
+            .map(|(i, v)| {
+                if v {
+                    last_valid = i;
+                    conseq_invalid_count = 0;
+                    i
+                } else if conseq_invalid_count < limit {
+                    conseq_invalid_count += 1;
+                    last_valid
+                } else {
+                    i
+                }
+            })
+            .collect::<Vec<_>>()
+    })
+}
+
+fn fill_backward_gather(s: &Series) -> PolarsResult<Series> {
+    fill_with_gather(s, |validity| {
+        let last = validity.len() as IdxSize - 1;
+        let mut last_valid = last;
+        unsafe {
+            validity
+                .iter()
+                .rev()
+                .enumerate_idx()
+                .map(|(i, v)| {
+                    if v {
+                        last_valid = last - i;
+                        last - i
+                    } else {
+                        last_valid
+                    }
+                })
+                .trust_my_length((last + 1) as usize)
+                .collect_reversed::<Vec<_>>()
+        }
+    })
+}
+
+fn fill_backward_gather_limit(s: &Series, limit: IdxSize) -> PolarsResult<Series> {
+    fill_with_gather(s, |validity| {
+        let last = validity.len() as IdxSize - 1;
+        let mut last_valid = last;
+        let mut conseq_invalid_count = 0;
+        unsafe {
+            validity
+                .iter()
+                .rev()
+                .enumerate_idx()
+                .map(|(i, v)| {
+                    if v {
+                        last_valid = last - i;
+                        conseq_invalid_count = 0;
+                        last - i
+                    } else if conseq_invalid_count < limit {
+                        conseq_invalid_count += 1;
+                        last_valid
+                    } else {
+                        last - i
+                    }
+                })
+                .trust_my_length((last + 1) as usize)
+                .collect_reversed()
+        }
+    })
+}
+
 fn fill_null_bool(ca: &BooleanChunked, strategy: FillNullStrategy) -> PolarsResult<Series> {
-    // Nothing to fill.
-    if ca.null_count() == 0 {
-        return Ok(ca.clone().into_series());
-    }
     match strategy {
-        FillNullStrategy::Forward(limit) => {
-            let mut out: BooleanChunked = match limit {
-                Some(limit) => fill_forward_limit(ca, limit),
-                None => fill_forward(ca),
-            };
-            out.rename(ca.name());
-            Ok(out.into_series())
-        },
-        FillNullStrategy::Backward(limit) => {
-            let mut out: BooleanChunked = match limit {
-                None => fill_backward(ca),
-                Some(limit) => fill_backward_limit(ca, limit),
-            };
-            out.rename(ca.name());
-            Ok(out.into_series())
-        },
         FillNullStrategy::Min => ca
             .fill_null_with_values(ca.min().ok_or_else(err_fill_null)?)
             .map(|ca| ca.into_series()),
@@ -418,31 +343,13 @@ fn fill_null_bool(ca: &BooleanChunked, strategy: FillNullStrategy) -> PolarsResu
         FillNullStrategy::Zero | FillNullStrategy::MinBound => {
             ca.fill_null_with_values(false).map(|ca| ca.into_series())
         },
+        FillNullStrategy::Forward(_) => unreachable!(),
+        FillNullStrategy::Backward(_) => unreachable!(),
     }
 }
 
 fn fill_null_binary(ca: &BinaryChunked, strategy: FillNullStrategy) -> PolarsResult<BinaryChunked> {
-    // Nothing to fill.
-    if ca.null_count() == 0 {
-        return Ok(ca.clone());
-    }
     match strategy {
-        FillNullStrategy::Forward(limit) => {
-            let mut out: BinaryChunked = match limit {
-                Some(limit) => fill_forward_limit(ca, limit),
-                None => fill_forward(ca),
-            };
-            out.rename(ca.name());
-            Ok(out)
-        },
-        FillNullStrategy::Backward(limit) => {
-            let mut out = match limit {
-                None => impl_fill_backward!(ca, BinaryChunked),
-                Some(limit) => fill_backward_limit_binary(ca, limit),
-            };
-            out.rename(ca.name());
-            Ok(out)
-        },
         FillNullStrategy::Min => {
             ca.fill_null_with_values(ca.min_binary().ok_or_else(err_fill_null)?)
         },
@@ -450,32 +357,8 @@ fn fill_null_binary(ca: &BinaryChunked, strategy: FillNullStrategy) -> PolarsRes
             ca.fill_null_with_values(ca.max_binary().ok_or_else(err_fill_null)?)
         },
         FillNullStrategy::Zero => ca.fill_null_with_values(&[]),
-        strat => polars_bail!(InvalidOperation: "fill-null strategy {:?} is not supported", strat),
-    }
-}
-
-fn fill_null_list(ca: &ListChunked, strategy: FillNullStrategy) -> PolarsResult<ListChunked> {
-    // Nothing to fill.
-    if ca.null_count() == 0 {
-        return Ok(ca.clone());
-    }
-    match strategy {
-        FillNullStrategy::Forward(limit) => {
-            let mut out: ListChunked = match limit {
-                Some(limit) => fill_forward_limit(ca, limit),
-                None => fill_forward(ca),
-            };
-            out.rename(ca.name());
-            Ok(out)
-        },
-        FillNullStrategy::Backward(limit) => {
-            let mut out: ListChunked = match limit {
-                None => impl_fill_backward!(ca, ListChunked),
-                Some(limit) => impl_fill_backward_limit!(ca, ListChunked, limit),
-            };
-            out.rename(ca.name());
-            Ok(out)
-        },
+        FillNullStrategy::Forward(_) => unreachable!(),
+        FillNullStrategy::Backward(_) => unreachable!(),
         strat => polars_bail!(InvalidOperation: "fill-null strategy {:?} is not supported", strat),
     }
 }
