@@ -9,40 +9,6 @@ use polars_utils::sync::SyncPtr;
 use polars_utils::total_ord::ToTotalOrd;
 use polars_utils::unwrap::UnwrapUncheckedRelease;
 
-#[derive(Clone)]
-pub struct Scalar {
-    dtype: DataType,
-    value: AnyValue<'static>,
-}
-
-impl Scalar {
-    pub fn new(dtype: DataType, value: AnyValue<'static>) -> Self {
-        Self { dtype, value }
-    }
-
-    pub fn value(&self) -> &AnyValue<'static> {
-        &self.value
-    }
-
-    pub fn as_any_value(&self) -> AnyValue {
-        self.value
-            .strict_cast(&self.dtype)
-            .unwrap_or_else(|| self.value.clone())
-    }
-
-    pub fn into_series(self, name: &str) -> Series {
-        Series::from_any_values_and_dtype(name, &[self.as_any_value()], &self.dtype, true).unwrap()
-    }
-
-    pub fn dtype(&self) -> &DataType {
-        &self.dtype
-    }
-
-    pub fn update(&mut self, value: AnyValue<'static>) {
-        self.value = value;
-    }
-}
-
 use super::*;
 #[cfg(feature = "dtype-struct")]
 use crate::prelude::any_value::arr_to_any_value;
@@ -128,7 +94,7 @@ pub enum AnyValue<'a> {
     StringOwned(smartstring::alias::String),
     Binary(&'a [u8]),
     BinaryOwned(Vec<u8>),
-    /// A 128-bit fixed point decimal number.
+    /// A 128-bit fixed point decimal number with a scale.
     #[cfg(feature = "dtype-decimal")]
     Decimal(i128, usize),
 }
@@ -163,7 +129,9 @@ impl Serialize for AnyValue<'_> {
             AnyValue::BinaryOwned(v) => {
                 serializer.serialize_newtype_variant(name, 14, "BinaryOwned", v)
             },
-            _ => todo!(),
+            _ => Err(serde::ser::Error::custom(
+                "Unknown data type. Cannot serialize",
+            )),
         }
     }
 }
@@ -569,7 +537,13 @@ impl<'a> AnyValue<'a> {
 
             // to string
             (av, DataType::String) => {
-                AnyValue::StringOwned(format_smartstring!("{}", av.extract::<i64>()?))
+                if av.is_unsigned_integer() {
+                    AnyValue::StringOwned(format_smartstring!("{}", av.extract::<u64>()?))
+                } else if av.is_float() {
+                    AnyValue::StringOwned(format_smartstring!("{}", av.extract::<f64>()?))
+                } else {
+                    AnyValue::StringOwned(format_smartstring!("{}", av.extract::<i64>()?))
+                }
             },
 
             // to binary
@@ -654,6 +628,35 @@ impl<'a> AnyValue<'a> {
                 *tu_r,
             ),
 
+            // to decimal
+            #[cfg(feature = "dtype-decimal")]
+            (av, DataType::Decimal(prec, scale)) if av.is_integer() => {
+                let value = av.try_extract::<i128>().unwrap();
+                let scale = scale.unwrap_or(0);
+                let factor = 10_i128.pow(scale as _); // Conversion to u32 is safe, max value is 38.
+                let converted = value.checked_mul(factor)?;
+
+                // Check if the converted value fits into the specified precision
+                let prec = prec.unwrap_or(38) as u32;
+                let num_digits = (converted.abs() as f64).log10().ceil() as u32;
+                if num_digits > prec {
+                    return None;
+                }
+
+                AnyValue::Decimal(converted, scale)
+            },
+            #[cfg(feature = "dtype-decimal")]
+            (AnyValue::Decimal(value, scale_av), DataType::Decimal(_, scale)) => {
+                let Some(scale) = scale else {
+                    return Some(self.clone());
+                };
+                // TODO: Allow lossy conversion?
+                let scale_diff = scale.checked_sub(*scale_av)?;
+                let factor = 10_i128.pow(scale_diff as _); // Conversion is safe, max value is 38.
+                let converted = value.checked_mul(factor)?;
+                AnyValue::Decimal(converted, *scale)
+            },
+
             // to self
             (av, dtype) if av.dtype() == *dtype => self.clone(),
 
@@ -825,8 +828,8 @@ impl<'a> AnyValue<'a> {
     pub fn add(&self, rhs: &AnyValue) -> AnyValue<'static> {
         use AnyValue::*;
         match (self, rhs) {
-            (Null, _) => Null,
-            (_, Null) => Null,
+            (Null, r) => r.clone().into_static().unwrap(),
+            (l, Null) => l.clone().into_static().unwrap(),
             (Int32(l), Int32(r)) => Int32(l + r),
             (Int64(l), Int64(r)) => Int64(l + r),
             (UInt32(l), UInt32(r)) => UInt32(l + r),
