@@ -14,15 +14,12 @@ impl private::PrivateSeries for SeriesWrap<BooleanChunked> {
     fn _dtype(&self) -> &DataType {
         self.0.ref_field().data_type()
     }
-    fn _get_flags(&self) -> Settings {
+    fn _get_flags(&self) -> MetadataFlags {
         self.0.get_flags()
     }
-    fn _set_flags(&mut self, flags: Settings) {
+    fn _set_flags(&mut self, flags: MetadataFlags) {
         self.0.set_flags(flags)
     }
-    fn explode_by_offsets(&self, offsets: &[i64]) -> Series {
-        self.0.explode_by_offsets(offsets)
-    }
 
     unsafe fn equal_element(&self, idx_self: usize, idx_other: usize, other: &Series) -> bool {
         self.0.equal_element(idx_self, idx_other, other)
@@ -39,12 +36,16 @@ impl private::PrivateSeries for SeriesWrap<BooleanChunked> {
         (&self.0).into_total_ord_inner()
     }
 
-    fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) -> PolarsResult<()> {
+    fn vec_hash(&self, random_state: PlRandomState, buf: &mut Vec<u64>) -> PolarsResult<()> {
         self.0.vec_hash(random_state, buf)?;
         Ok(())
     }
 
-    fn vec_hash_combine(&self, build_hasher: RandomState, hashes: &mut [u64]) -> PolarsResult<()> {
+    fn vec_hash_combine(
+        &self,
+        build_hasher: PlRandomState,
+        hashes: &mut [u64],
+    ) -> PolarsResult<()> {
         self.0.vec_hash_combine(build_hasher, hashes)?;
         Ok(())
     }
@@ -71,14 +72,14 @@ impl private::PrivateSeries for SeriesWrap<BooleanChunked> {
     #[cfg(feature = "algorithm_group_by")]
     unsafe fn agg_std(&self, groups: &GroupsProxy, _ddof: u8) -> Series {
         self.0
-            .cast(&DataType::Float64)
+            .cast_with_options(&DataType::Float64, CastOptions::Overflowing)
             .unwrap()
             .agg_std(groups, _ddof)
     }
     #[cfg(feature = "algorithm_group_by")]
     unsafe fn agg_var(&self, groups: &GroupsProxy, _ddof: u8) -> Series {
         self.0
-            .cast(&DataType::Float64)
+            .cast_with_options(&DataType::Float64, CastOptions::Overflowing)
             .unwrap()
             .agg_var(groups, _ddof)
     }
@@ -101,6 +102,10 @@ impl private::PrivateSeries for SeriesWrap<BooleanChunked> {
 }
 
 impl SeriesTrait for SeriesWrap<BooleanChunked> {
+    fn get_metadata(&self) -> Option<RwLockReadGuard<dyn MetadataTrait>> {
+        self.metadata_dyn()
+    }
+
     fn bitxor(&self, other: &Series) -> PolarsResult<Series> {
         let other = self.0.unpack_series_matching_type(other)?;
         Ok((&self.0).bitxor(other).into_series())
@@ -140,16 +145,20 @@ impl SeriesTrait for SeriesWrap<BooleanChunked> {
     fn slice(&self, offset: i64, length: usize) -> Series {
         self.0.slice(offset, length).into_series()
     }
+    fn split_at(&self, offset: i64) -> (Series, Series) {
+        let (a, b) = self.0.split_at(offset);
+        (a.into_series(), b.into_series())
+    }
 
     fn append(&mut self, other: &Series) -> PolarsResult<()> {
         polars_ensure!(self.0.dtype() == other.dtype(), append);
-        self.0.append(other.as_ref().as_ref());
+        self.0.append(other.as_ref().as_ref())?;
         Ok(())
     }
 
     fn extend(&mut self, other: &Series) -> PolarsResult<()> {
         polars_ensure!(self.0.dtype() == other.dtype(), extend);
-        self.0.extend(other.as_ref().as_ref());
+        self.0.extend(other.as_ref().as_ref())?;
         Ok(())
     }
 
@@ -189,8 +198,8 @@ impl SeriesTrait for SeriesWrap<BooleanChunked> {
         ChunkExpandAtIndex::new_from_index(&self.0, index, length).into_series()
     }
 
-    fn cast(&self, data_type: &DataType) -> PolarsResult<Series> {
-        self.0.cast(data_type)
+    fn cast(&self, data_type: &DataType, options: CastOptions) -> PolarsResult<Series> {
+        self.0.cast_with_options(data_type, options)
     }
 
     fn get(&self, index: usize) -> PolarsResult<AnyValue> {
@@ -214,8 +223,8 @@ impl SeriesTrait for SeriesWrap<BooleanChunked> {
         self.0.null_count()
     }
 
-    fn has_validity(&self) -> bool {
-        self.0.has_validity()
+    fn has_nulls(&self) -> bool {
+        self.0.has_nulls()
     }
 
     #[cfg(feature = "algorithm_group_by")]
@@ -263,21 +272,30 @@ impl SeriesTrait for SeriesWrap<BooleanChunked> {
         Ok(ChunkAggSeries::min_reduce(&self.0))
     }
     fn median_reduce(&self) -> PolarsResult<Scalar> {
-        let ca = self.0.cast(&DataType::Int8).unwrap();
+        let ca = self
+            .0
+            .cast_with_options(&DataType::Int8, CastOptions::Overflowing)
+            .unwrap();
         let sc = ca.median_reduce()?;
         let v = sc.value().cast(&DataType::Float64);
         Ok(Scalar::new(DataType::Float64, v))
     }
     /// Get the variance of the Series as a new Series of length 1.
     fn var_reduce(&self, _ddof: u8) -> PolarsResult<Scalar> {
-        let ca = self.0.cast(&DataType::Int8).unwrap();
+        let ca = self
+            .0
+            .cast_with_options(&DataType::Int8, CastOptions::Overflowing)
+            .unwrap();
         let sc = ca.var_reduce(_ddof)?;
         let v = sc.value().cast(&DataType::Float64);
         Ok(Scalar::new(DataType::Float64, v))
     }
     /// Get the standard deviation of the Series as a new Series of length 1.
     fn std_reduce(&self, _ddof: u8) -> PolarsResult<Scalar> {
-        let ca = self.0.cast(&DataType::Int8).unwrap();
+        let ca = self
+            .0
+            .cast_with_options(&DataType::Int8, CastOptions::Overflowing)
+            .unwrap();
         let sc = ca.std_reduce(_ddof)?;
         let v = sc.value().cast(&DataType::Float64);
         Ok(Scalar::new(DataType::Float64, v))
