@@ -17,16 +17,12 @@ macro_rules! impl_dyn_series {
                 self.0.ref_field().data_type()
             }
 
-            fn _set_flags(&mut self, flags: Settings) {
+            fn _set_flags(&mut self, flags: MetadataFlags) {
                 self.0.set_flags(flags)
             }
-            fn _get_flags(&self) -> Settings {
+            fn _get_flags(&self) -> MetadataFlags {
                 self.0.get_flags()
             }
-            fn explode_by_offsets(&self, offsets: &[i64]) -> Series {
-                self.0.explode_by_offsets(offsets)
-            }
-
             unsafe fn equal_element(
                 &self,
                 idx_self: usize,
@@ -52,14 +48,18 @@ macro_rules! impl_dyn_series {
                 (&self.0).into_total_ord_inner()
             }
 
-            fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) -> PolarsResult<()> {
+            fn vec_hash(
+                &self,
+                random_state: PlRandomState,
+                buf: &mut Vec<u64>,
+            ) -> PolarsResult<()> {
                 self.0.vec_hash(random_state, buf)?;
                 Ok(())
             }
 
             fn vec_hash_combine(
                 &self,
-                build_hasher: RandomState,
+                build_hasher: PlRandomState,
                 hashes: &mut [u64],
             ) -> PolarsResult<()> {
                 self.0.vec_hash_combine(build_hasher, hashes)?;
@@ -97,18 +97,48 @@ macro_rules! impl_dyn_series {
             }
 
             fn subtract(&self, rhs: &Series) -> PolarsResult<Series> {
+                polars_ensure!(
+                    self.dtype() == rhs.dtype(),
+                    opq = sub,
+                    self.dtype(),
+                    rhs.dtype()
+                );
                 NumOpsDispatch::subtract(&self.0, rhs)
             }
             fn add_to(&self, rhs: &Series) -> PolarsResult<Series> {
+                polars_ensure!(
+                    self.dtype() == rhs.dtype(),
+                    opq = add,
+                    self.dtype(),
+                    rhs.dtype()
+                );
                 NumOpsDispatch::add_to(&self.0, rhs)
             }
             fn multiply(&self, rhs: &Series) -> PolarsResult<Series> {
+                polars_ensure!(
+                    self.dtype() == rhs.dtype(),
+                    opq = mul,
+                    self.dtype(),
+                    rhs.dtype()
+                );
                 NumOpsDispatch::multiply(&self.0, rhs)
             }
             fn divide(&self, rhs: &Series) -> PolarsResult<Series> {
+                polars_ensure!(
+                    self.dtype() == rhs.dtype(),
+                    opq = div,
+                    self.dtype(),
+                    rhs.dtype()
+                );
                 NumOpsDispatch::divide(&self.0, rhs)
             }
             fn remainder(&self, rhs: &Series) -> PolarsResult<Series> {
+                polars_ensure!(
+                    self.dtype() == rhs.dtype(),
+                    opq = rem,
+                    self.dtype(),
+                    rhs.dtype()
+                );
                 NumOpsDispatch::remainder(&self.0, rhs)
             }
             #[cfg(feature = "algorithm_group_by")]
@@ -135,6 +165,10 @@ macro_rules! impl_dyn_series {
                 ChunkRollApply::rolling_map(&self.0, _f, _options).map(|ca| ca.into_series())
             }
 
+            fn get_metadata(&self) -> Option<RwLockReadGuard<dyn MetadataTrait>> {
+                self.metadata_dyn()
+            }
+
             fn rename(&mut self, name: &str) {
                 self.0.rename(name);
             }
@@ -160,15 +194,20 @@ macro_rules! impl_dyn_series {
                 return self.0.slice(offset, length).into_series();
             }
 
+            fn split_at(&self, offset: i64) -> (Series, Series) {
+                let (a, b) = self.0.split_at(offset);
+                (a.into_series(), b.into_series())
+            }
+
             fn append(&mut self, other: &Series) -> PolarsResult<()> {
                 polars_ensure!(self.0.dtype() == other.dtype(), append);
-                self.0.append(other.as_ref().as_ref());
+                self.0.append(other.as_ref().as_ref())?;
                 Ok(())
             }
 
             fn extend(&mut self, other: &Series) -> PolarsResult<()> {
                 polars_ensure!(self.0.dtype() == other.dtype(), extend);
-                self.0.extend(other.as_ref().as_ref());
+                self.0.extend(other.as_ref().as_ref())?;
                 Ok(())
             }
 
@@ -220,8 +259,12 @@ macro_rules! impl_dyn_series {
                 ChunkExpandAtIndex::new_from_index(&self.0, index, length).into_series()
             }
 
-            fn cast(&self, data_type: &DataType) -> PolarsResult<Series> {
-                self.0.cast(data_type)
+            fn cast(
+                &self,
+                data_type: &DataType,
+                cast_options: CastOptions,
+            ) -> PolarsResult<Series> {
+                self.0.cast_with_options(data_type, cast_options)
             }
 
             fn get(&self, index: usize) -> PolarsResult<AnyValue> {
@@ -245,8 +288,8 @@ macro_rules! impl_dyn_series {
                 self.0.null_count()
             }
 
-            fn has_validity(&self) -> bool {
-                self.0.has_validity()
+            fn has_nulls(&self) -> bool {
+                self.0.has_nulls()
             }
 
             #[cfg(feature = "algorithm_group_by")]
