@@ -20,7 +20,7 @@ pub(crate) mod null;
 mod object;
 mod string;
 #[cfg(feature = "dtype-struct")]
-mod struct__;
+mod struct_;
 #[cfg(feature = "dtype-time")]
 mod time;
 
@@ -29,15 +29,12 @@ use std::borrow::Cow;
 use std::ops::{BitAnd, BitOr, BitXor};
 use std::sync::RwLockReadGuard;
 
-use ahash::RandomState;
-
 use super::*;
 use crate::chunked_array::comparison::*;
 use crate::chunked_array::metadata::MetadataTrait;
 use crate::chunked_array::ops::compare_inner::{
     IntoTotalEqInner, IntoTotalOrdInner, TotalEqInner, TotalOrdInner,
 };
-use crate::chunked_array::ops::explode::ExplodeByOffsets;
 use crate::chunked_array::AsSinglePtr;
 
 // Utility wrapper struct
@@ -92,10 +89,6 @@ macro_rules! impl_dyn_series {
                 self.0.set_flags(flags)
             }
 
-            fn explode_by_offsets(&self, offsets: &[i64]) -> Series {
-                self.0.explode_by_offsets(offsets)
-            }
-
             unsafe fn equal_element(
                 &self,
                 idx_self: usize,
@@ -121,14 +114,18 @@ macro_rules! impl_dyn_series {
                 (&self.0).into_total_ord_inner()
             }
 
-            fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) -> PolarsResult<()> {
+            fn vec_hash(
+                &self,
+                random_state: PlRandomState,
+                buf: &mut Vec<u64>,
+            ) -> PolarsResult<()> {
                 self.0.vec_hash(random_state, buf)?;
                 Ok(())
             }
 
             fn vec_hash_combine(
                 &self,
-                build_hasher: RandomState,
+                build_hasher: PlRandomState,
                 hashes: &mut [u64],
             ) -> PolarsResult<()> {
                 self.0.vec_hash_combine(build_hasher, hashes)?;
@@ -307,13 +304,13 @@ macro_rules! impl_dyn_series {
 
             fn append(&mut self, other: &Series) -> PolarsResult<()> {
                 polars_ensure!(self.0.dtype() == other.dtype(), append);
-                self.0.append(other.as_ref().as_ref());
+                self.0.append(other.as_ref().as_ref())?;
                 Ok(())
             }
 
             fn extend(&mut self, other: &Series) -> PolarsResult<()> {
                 polars_ensure!(self.0.dtype() == other.dtype(), extend);
-                self.0.extend(other.as_ref().as_ref());
+                self.0.extend(other.as_ref().as_ref())?;
                 Ok(())
             }
 
