@@ -3,7 +3,6 @@ use std::any::Any;
 use polars_error::constants::LENGTH_LIMIT_MSG;
 
 use crate::prelude::compare_inner::{IntoTotalEqInner, TotalEqInner};
-use crate::prelude::explode::ExplodeByOffsets;
 use crate::prelude::*;
 use crate::series::private::{PrivateSeries, PrivateSeriesNumeric};
 use crate::series::*;
@@ -80,10 +79,6 @@ impl PrivateSeries for NullChunked {
 
         Ok(Self::new(self.name().into(), len).into_series())
     }
-    fn explode_by_offsets(&self, offsets: &[i64]) -> Series {
-        ExplodeByOffsets::explode_by_offsets(self, offsets)
-    }
-
     fn subtract(&self, _rhs: &Series) -> PolarsResult<Series> {
         null_arithmetic(self, _rhs, "subtract")
     }
@@ -122,12 +117,16 @@ impl PrivateSeries for NullChunked {
         MetadataFlags::empty()
     }
 
-    fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) -> PolarsResult<()> {
+    fn vec_hash(&self, random_state: PlRandomState, buf: &mut Vec<u64>) -> PolarsResult<()> {
         VecHash::vec_hash(self, random_state, buf)?;
         Ok(())
     }
 
-    fn vec_hash_combine(&self, build_hasher: RandomState, hashes: &mut [u64]) -> PolarsResult<()> {
+    fn vec_hash_combine(
+        &self,
+        build_hasher: PlRandomState,
+        hashes: &mut [u64],
+    ) -> PolarsResult<()> {
         VecHash::vec_hash_combine(self, build_hasher, hashes)?;
         Ok(())
     }
