@@ -39,10 +39,6 @@ impl private::PrivateSeries for SeriesWrap<TimeChunked> {
         self.0.set_flags(flags)
     }
 
-    fn explode_by_offsets(&self, offsets: &[i64]) -> Series {
-        self.0.explode_by_offsets(offsets).into_time().into_series()
-    }
-
     #[cfg(feature = "zip_with")]
     fn zip_with_same_type(&self, mask: &BooleanChunked, other: &Series) -> PolarsResult<Series> {
         let other = other.to_physical_repr().into_owned();
@@ -51,12 +47,16 @@ impl private::PrivateSeries for SeriesWrap<TimeChunked> {
             .map(|ca| ca.into_time().into_series())
     }
 
-    fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) -> PolarsResult<()> {
+    fn vec_hash(&self, random_state: PlRandomState, buf: &mut Vec<u64>) -> PolarsResult<()> {
         self.0.vec_hash(random_state, buf)?;
         Ok(())
     }
 
-    fn vec_hash_combine(&self, build_hasher: RandomState, hashes: &mut [u64]) -> PolarsResult<()> {
+    fn vec_hash_combine(
+        &self,
+        build_hasher: PlRandomState,
+        hashes: &mut [u64],
+    ) -> PolarsResult<()> {
         self.0.vec_hash_combine(build_hasher, hashes)?;
         Ok(())
     }
@@ -160,7 +160,7 @@ impl SeriesTrait for SeriesWrap<TimeChunked> {
         // ref Cow
         // ref SeriesTrait
         // ref ChunkedArray
-        self.0.append(other.as_ref().as_ref().as_ref());
+        self.0.append(other.as_ref().as_ref().as_ref())?;
         Ok(())
     }
 
@@ -171,7 +171,7 @@ impl SeriesTrait for SeriesWrap<TimeChunked> {
         // ref SeriesTrait
         // ref ChunkedArray
         let other = other.to_physical_repr();
-        self.0.extend(other.as_ref().as_ref().as_ref());
+        self.0.extend(other.as_ref().as_ref().as_ref())?;
         Ok(())
     }
 
