@@ -1,3 +1,7 @@
+use arrow::bitmap::Bitmap;
+
+#[cfg(feature = "object")]
+use crate::chunked_array::object::registry::get_object_builder;
 use crate::prelude::*;
 
 impl Series {
@@ -51,15 +55,28 @@ impl Series {
                     .iter()
                     .map(|fld| Series::full_null(fld.name(), size, fld.data_type()))
                     .collect::<Vec<_>>();
-                StructChunked::from_series(name, &fields)
-                    .unwrap()
-                    .into_series()
+                let ca = StructChunked::from_series(name, &fields).unwrap();
+
+                if !fields.is_empty() {
+                    ca.with_outer_validity(Some(Bitmap::new_zeroed(size)))
+                        .into_series()
+                } else {
+                    ca.into_series()
+                }
             },
             DataType::Null => Series::new_null(name, size),
             DataType::Unknown(kind) => {
                 let dtype = kind.materialize().expect("expected known type");
                 Series::full_null(name, size, &dtype)
             },
+            #[cfg(feature = "object")]
+            DataType::Object(_, _) => {
+                let mut builder = get_object_builder(name, size);
+                for _ in 0..size {
+                    builder.append_null();
+                }
+                builder.to_series()
+            },
             _ => {
                 macro_rules! primitive {
                     ($type:ty) => {{
