@@ -46,8 +46,6 @@ pub enum BitRepr {
 }
 
 pub(crate) mod private {
-    use ahash::RandomState;
-
     use super::*;
     use crate::chunked_array::metadata::MetadataFlags;
     use crate::chunked_array::ops::compare_inner::{TotalEqInner, TotalOrdInner};
@@ -81,10 +79,6 @@ pub(crate) mod private {
 
         fn _set_flags(&mut self, flags: MetadataFlags);
 
-        fn explode_by_offsets(&self, _offsets: &[i64]) -> Series {
-            invalid_operation_panic!(explode_by_offsets, self)
-        }
-
         unsafe fn equal_element(
             &self,
             _idx_self: usize,
@@ -101,12 +95,12 @@ pub(crate) mod private {
         fn into_total_ord_inner<'a>(&'a self) -> Box<dyn TotalOrdInner + 'a> {
             invalid_operation_panic!(into_total_ord_inner, self)
         }
-        fn vec_hash(&self, _build_hasher: RandomState, _buf: &mut Vec<u64>) -> PolarsResult<()> {
+        fn vec_hash(&self, _build_hasher: PlRandomState, _buf: &mut Vec<u64>) -> PolarsResult<()> {
             polars_bail!(opq = vec_hash, self._dtype());
         }
         fn vec_hash_combine(
             &self,
-            _build_hasher: RandomState,
+            _build_hasher: PlRandomState,
             _hashes: &mut [u64],
         ) -> PolarsResult<()> {
             polars_bail!(opq = vec_hash_combine, self._dtype());
