@@ -45,10 +45,15 @@ impl PhysicalExpr for FilterExpr {
 
         let (ac_s, ac_predicate) = POOL.install(|| rayon::join(ac_s_f, ac_predicate_f));
         let (mut ac_s, mut ac_predicate) = (ac_s?, ac_predicate?);
+        // Check if the groups are still equal, otherwise aggregate.
+        // TODO! create a special group iters that don't materialize
+        if ac_s.groups.as_ref() as *const _ != ac_predicate.groups.as_ref() as *const _ {
+            let _ = ac_s.aggregated();
+            let _ = ac_predicate.aggregated();
+        }
 
         if ac_predicate.is_aggregated() || ac_s.is_aggregated() {
-            // SAFETY: unstable series never lives longer than the iterator.
-            let preds = unsafe { ac_predicate.iter_groups(false) };
+            let preds = ac_predicate.iter_groups(false);
             let s = ac_s.aggregated();
             let ca = s.list()?;
             let out = if ca.is_empty() {
