@@ -6,6 +6,7 @@ mod cast;
 mod column;
 mod count;
 mod filter;
+mod gather;
 mod group_iter;
 mod literal;
 #[cfg(feature = "dynamic_group_by")]
@@ -13,7 +14,6 @@ mod rolling;
 mod slice;
 mod sort;
 mod sortby;
-mod take;
 mod ternary;
 mod window;
 
@@ -30,18 +30,18 @@ pub(crate) use cast::*;
 pub(crate) use column::*;
 pub(crate) use count::*;
 pub(crate) use filter::*;
+pub(crate) use gather::*;
 pub(crate) use literal::*;
 use polars_core::prelude::*;
 use polars_io::predicates::PhysicalIoExpr;
-use polars_ops::prelude::*;
 use polars_plan::prelude::*;
 #[cfg(feature = "dynamic_group_by")]
 pub(crate) use rolling::RollingExpr;
 pub(crate) use slice::*;
 pub(crate) use sort::*;
 pub(crate) use sortby::*;
-pub(crate) use take::*;
 pub(crate) use ternary::*;
+pub use window::window_function_format_order_by;
 pub(crate) use window::*;
 
 use crate::state::ExecutionState;
@@ -121,7 +121,7 @@ impl<'a> AggregationContext<'a> {
     pub(crate) fn dtype(&self) -> DataType {
         match &self.state {
             AggState::Literal(s) => s.dtype().clone(),
-            AggState::AggregatedList(s) => s.list().unwrap().inner_dtype(),
+            AggState::AggregatedList(s) => s.list().unwrap().inner_dtype().clone(),
             AggState::AggregatedScalar(s) => s.dtype().clone(),
             AggState::NotAggregated(s) => s.dtype().clone(),
         }
@@ -196,7 +196,7 @@ impl<'a> AggregationContext<'a> {
 
     /// # Arguments
     /// - `aggregated` sets if the Series is a list due to aggregation (could also be a list because its
-    /// the columns dtype)
+    ///   the columns dtype)
     fn new(
         series: Series,
         groups: Cow<'a, GroupsProxy>,
@@ -289,8 +289,7 @@ impl<'a> AggregationContext<'a> {
                 });
             },
             _ => {
-                // SAFETY: unstable series never lives longer than the iterator.
-                let groups = unsafe {
+                let groups = {
                     self.series()
                         .list()
                         .expect("impl error, should be a list at this point")
@@ -319,7 +318,7 @@ impl<'a> AggregationContext<'a> {
 
     /// # Arguments
     /// - `aggregated` sets if the Series is a list due to aggregation (could also be a list because its
-    /// the columns dtype)
+    ///   the columns dtype)
     pub(crate) fn with_series(
         &mut self,
         series: Series,
@@ -379,9 +378,11 @@ impl<'a> AggregationContext<'a> {
 
     /// Update the group tuples
     pub(crate) fn with_groups(&mut self, groups: GroupsProxy) -> &mut Self {
-        // In case of new groups, a series always needs to be flattened
-        self.with_series(self.flat_naive().into_owned(), false, None)
-            .unwrap();
+        if let AggState::AggregatedList(_) = self.agg_state() {
+            // In case of new groups, a series always needs to be flattened
+            self.with_series(self.flat_naive().into_owned(), false, None)
+                .unwrap();
+        }
         self.groups = Cow::Owned(groups);
         // make sure that previous setting is not used
         self.update_groups = UpdateGroups::No;
@@ -420,7 +421,9 @@ impl<'a> AggregationContext<'a> {
                 self.groups();
                 let rows = self.groups.len();
                 let s = s.new_from_index(0, rows);
-                s.reshape_list(&[rows as i64, -1]).unwrap()
+                let out = s.reshape_list(&[rows as i64, -1]).unwrap();
+                self.state = AggState::AggregatedList(out.clone());
+                out
             },
         }
     }
@@ -612,6 +615,10 @@ impl PhysicalIoExpr for PhysicalIoHelper {
         self.expr.evaluate(df, &state)
     }
 
+    fn live_variables(&self) -> Option<Vec<Arc<str>>> {
+        Some(expr_to_leaf_column_names(self.expr.as_expression()?))
+    }
+
     #[cfg(feature = "parquet")]
     fn as_stats_evaluator(&self) -> Option<&dyn polars_io::predicates::StatsEvaluator> {
         self.expr.as_stats_evaluator()
