@@ -161,26 +161,6 @@ impl ExpressionConversionState {
     }
 }
 
-pub fn create_physical_expr_streaming(
-    expr_ir: &ExprIR,
-    expr_arena: &Arena<AExpr>,
-    schema: Option<&SchemaRef>,
-    state: &mut ExpressionConversionState,
-) -> PolarsResult<Arc<dyn PhysicalExpr>> {
-    let phys_expr =
-        create_physical_expr_inner(expr_ir.node(), Context::Default, expr_arena, schema, state)?;
-
-    if let Some(name) = expr_ir.get_alias() {
-        Ok(Arc::new(AliasExpr::new(
-            phys_expr,
-            name.clone(),
-            node_to_expr(expr_ir.node(), expr_arena),
-        )))
-    } else {
-        Ok(phys_expr)
-    }
-}
-
 pub fn create_physical_expr(
     expr_ir: &ExprIR,
     ctxt: Context,
@@ -406,7 +386,7 @@ fn create_physical_expr_inner(
                         I::Std(_, ddof) => GBM::Std(*ddof),
                         I::Var(_, ddof) => GBM::Var(*ddof),
                         I::AggGroups(_) => {
-                            panic!("agg groups expression only supported in aggregation context")
+                            polars_bail!(InvalidOperation: "agg groups expression only supported in aggregation context")
                         },
                     };
 
@@ -496,8 +476,8 @@ fn create_physical_expr_inner(
                     .ok()
             });
 
-            let is_reducing_aggregation =
-                options.returns_scalar && matches!(options.collect_groups, ApplyOptions::GroupWise);
+            let is_reducing_aggregation = options.flags.contains(FunctionFlags::RETURNS_SCALAR)
+                && matches!(options.collect_groups, ApplyOptions::GroupWise);
             // Will be reset in the function so get that here.
             let has_window = state.local.has_window;
             let input = create_physical_expressions_check_state(
@@ -534,8 +514,8 @@ fn create_physical_expr_inner(
                     .to_dtype(schema, Context::Default, expr_arena)
                     .ok()
             });
-            let is_reducing_aggregation =
-                options.returns_scalar && matches!(options.collect_groups, ApplyOptions::GroupWise);
+            let is_reducing_aggregation = options.flags.contains(FunctionFlags::RETURNS_SCALAR)
+                && matches!(options.collect_groups, ApplyOptions::GroupWise);
             // Will be reset in the function so get that here.
             let has_window = state.local.has_window;
             let input = create_physical_expressions_check_state(
@@ -596,11 +576,5 @@ fn create_physical_expr_inner(
                 node_to_expr(*input, expr_arena),
             )))
         },
-        Wildcard => {
-            polars_bail!(ComputeError: "wildcard column selection not supported at this point")
-        },
-        Nth(n) => {
-            polars_bail!(ComputeError: "nth column selection not supported at this point (n={})", n)
-        },
     }
 }
