@@ -1,11 +1,12 @@
 mod convert;
 mod extrema;
+mod len;
 mod mean;
 mod sum;
 
 use std::any::Any;
 
-pub use convert::{can_convert_into_reduction, into_reduction};
+pub use convert::into_reduction;
 use polars_core::prelude::*;
 
 #[allow(dead_code)]
