@@ -7,7 +7,7 @@ use polars_core::error::to_compute_err;
 use polars_core::prelude::*;
 pub use Compression as AvroCompression;
 
-use crate::shared::SerWriter;
+use crate::shared::{schema_to_arrow_checked, SerWriter};
 
 /// Write a [`DataFrame`] to [Apache Avro] format
 ///
@@ -64,12 +64,12 @@ where
     }
 
     fn finish(&mut self, df: &mut DataFrame) -> PolarsResult<()> {
-        let schema = df.schema().to_arrow(false);
+        let schema = schema_to_arrow_checked(&df.schema(), CompatLevel::oldest(), "avro")?;
         let record = write::to_record(&schema, self.name.clone())?;
 
         let mut data = vec![];
         let mut compressed_block = avro_schema::file::CompressedBlock::default();
-        for chunk in df.iter_chunks(false) {
+        for chunk in df.iter_chunks(CompatLevel::oldest(), true) {
             let mut serializers = chunk
                 .iter()
                 .zip(record.fields.iter())
