@@ -45,22 +45,6 @@ pub(crate) fn get_file_chunks(
     offsets
 }
 
-// magic numbers
-const GZIP: [u8; 2] = [31, 139];
-const ZLIB0: [u8; 2] = [0x78, 0x01];
-const ZLIB1: [u8; 2] = [0x78, 0x9C];
-const ZLIB2: [u8; 2] = [0x78, 0xDA];
-const ZSTD: [u8; 4] = [0x28, 0xB5, 0x2F, 0xFD];
-
-/// check if csv file is compressed
-pub fn is_compressed(bytes: &[u8]) -> bool {
-    bytes.starts_with(&ZLIB0)
-        || bytes.starts_with(&ZLIB1)
-        || bytes.starts_with(&ZLIB2)
-        || bytes.starts_with(&GZIP)
-        || bytes.starts_with(&ZSTD)
-}
-
 #[cfg(any(feature = "decompress", feature = "decompress-fast"))]
 fn decompress_impl<R: Read>(
     decoder: &mut R,
@@ -145,6 +129,7 @@ pub(crate) fn decompress(
     quote_char: Option<u8>,
     eol_char: u8,
 ) -> Option<Vec<u8>> {
+    use crate::utils::compression::magic::*;
     if bytes.starts_with(&GZIP) {
         let mut decoder = flate2::read::MultiGzDecoder::new(bytes);
         decompress_impl(&mut decoder, n_rows, separator, quote_char, eol_char)
