@@ -25,7 +25,7 @@
 //! let file = Cursor::new(basic_json);
 //! let df = JsonReader::new(file)
 //! .with_json_format(JsonFormat::JsonLines)
-//! .infer_schema_len(Some(3))
+//! .infer_schema_len(NonZeroUsize::new(3))
 //! .with_batch_size(NonZeroUsize::new(3).unwrap())
 //! .finish()
 //! .unwrap();
@@ -71,6 +71,7 @@ use std::ops::Deref;
 use arrow::legacy::conversion::chunk_to_struct;
 use polars_core::error::to_compute_err;
 use polars_core::prelude::*;
+use polars_error::{polars_bail, PolarsResult};
 use polars_json::json::write::FallibleStreamingIterator;
 #[cfg(feature = "serde")]
 use serde::{Deserialize, Serialize};
@@ -86,9 +87,11 @@ pub struct JsonWriterOptions {
     pub maintain_order: bool,
 }
 
-/// The format to use to write the DataFrame to JSON: `Json` (a JSON array) or `JsonLines` (each row output on a
-/// separate line). In either case, each row is serialized as a JSON object whose keys are the column names and whose
-/// values are the row's corresponding values.
+/// The format to use to write the DataFrame to JSON: `Json` (a JSON array)
+/// or `JsonLines` (each row output on a separate line).
+///
+/// In either case, each row is serialized as a JSON object whose keys are the column names and
+/// whose values are the row's corresponding values.
 pub enum JsonFormat {
     /// A single JSON array containing each DataFrame row as an object. The length of the array is the number of rows in
     /// the DataFrame.
@@ -143,10 +146,14 @@ where
         df.align_chunks();
         let fields = df
             .iter()
-            .map(|s| s.field().to_arrow(true))
-            .collect::<Vec<_>>();
+            .map(|s| {
+                #[cfg(feature = "object")]
+                polars_ensure!(!matches!(s.dtype(), DataType::Object(_, _)), ComputeError: "cannot write 'Object' datatype to json");
+                Ok(s.field().to_arrow(CompatLevel::newest()))
+            })
+            .collect::<PolarsResult<Vec<_>>>()?;
         let batches = df
-            .iter_chunks(true)
+            .iter_chunks(CompatLevel::newest(), false)
             .map(|chunk| Ok(Box::new(chunk_to_struct(chunk, fields.clone())) as ArrayRef));
 
         match self.json_format {
@@ -184,9 +191,13 @@ where
     pub fn write_batch(&mut self, df: &DataFrame) -> PolarsResult<()> {
         let fields = df
             .iter()
-            .map(|s| s.field().to_arrow(true))
-            .collect::<Vec<_>>();
-        let chunks = df.iter_chunks(true);
+            .map(|s| {
+                #[cfg(feature = "object")]
+                polars_ensure!(!matches!(s.dtype(), DataType::Object(_, _)), ComputeError: "cannot write 'Object' datatype to json");
+                Ok(s.field().to_arrow(CompatLevel::newest()))
+            })
+            .collect::<PolarsResult<Vec<_>>>()?;
+        let chunks = df.iter_chunks(CompatLevel::newest(), false);
         let batches =
             chunks.map(|chunk| Ok(Box::new(chunk_to_struct(chunk, fields.clone())) as ArrayRef));
         let mut serializer = polars_json::ndjson::write::Serializer::new(batches, vec![]);
@@ -206,7 +217,7 @@ where
     reader: R,
     rechunk: bool,
     ignore_errors: bool,
-    infer_schema_len: Option<usize>,
+    infer_schema_len: Option<NonZeroUsize>,
     batch_size: NonZeroUsize,
     projection: Option<Vec<String>>,
     schema: Option<SchemaRef>,
@@ -214,6 +225,17 @@ where
     json_format: JsonFormat,
 }
 
+pub fn remove_bom(bytes: &[u8]) -> PolarsResult<&[u8]> {
+    if bytes.starts_with(&[0xEF, 0xBB, 0xBF]) {
+        // UTF-8 BOM
+        Ok(&bytes[3..])
+    } else if bytes.starts_with(&[0xFE, 0xFF]) || bytes.starts_with(&[0xFF, 0xFE]) {
+        // UTF-16 BOM
+        polars_bail!(ComputeError: "utf-16 not supported")
+    } else {
+        Ok(bytes)
+    }
+}
 impl<'a, R> SerReader<R> for JsonReader<'a, R>
 where
     R: MmapBytesReader,
@@ -223,7 +245,7 @@ where
             reader,
             rechunk: true,
             ignore_errors: false,
-            infer_schema_len: Some(100),
+            infer_schema_len: Some(NonZeroUsize::new(100).unwrap()),
             batch_size: NonZeroUsize::new(8192).unwrap(),
             projection: None,
             schema: None,
@@ -242,9 +264,10 @@ where
     /// Because JSON values specify their types (number, string, etc), no upcasting or conversion is performed between
     /// incompatible types in the input. In the event that a column contains mixed dtypes, is it unspecified whether an
     /// error is returned or whether elements of incompatible dtypes are replaced with `null`.
-    fn finish(self) -> PolarsResult<DataFrame> {
-        let rb: ReaderBytes = (&self.reader).into();
-
+    fn finish(mut self) -> PolarsResult<DataFrame> {
+        let pre_rb: ReaderBytes = (&mut self.reader).into();
+        let bytes = remove_bom(pre_rb.deref())?;
+        let rb = ReaderBytes::Borrowed(bytes);
         let out = match self.json_format {
             JsonFormat::Json => {
                 polars_ensure!(!self.ignore_errors, InvalidOperation: "'ignore_errors' only supported in ndjson");
@@ -259,15 +282,16 @@ where
                         overwrite_schema(mut_schema, overwrite)?;
                     }
 
-                    DataType::Struct(schema.iter_fields().collect()).to_arrow(true)
+                    DataType::Struct(schema.iter_fields().collect()).to_arrow(CompatLevel::newest())
                 } else {
                     // infer
                     let inner_dtype = if let BorrowedValue::Array(values) = &json_value {
                         infer::json_values_to_supertype(
                             values,
-                            self.infer_schema_len.unwrap_or(usize::MAX),
+                            self.infer_schema_len
+                                .unwrap_or(NonZeroUsize::new(usize::MAX).unwrap()),
                         )?
-                        .to_arrow(true)
+                        .to_arrow(CompatLevel::newest())
                     } else {
                         polars_json::json::infer(&json_value)?
                     };
@@ -286,7 +310,7 @@ where
                                 .map(|(name, dt)| Field::new(&name, dt))
                                 .collect(),
                         )
-                        .to_arrow(true)
+                        .to_arrow(CompatLevel::newest())
                     } else {
                         inner_dtype
                     }
@@ -318,6 +342,9 @@ where
                     false,
                     self.infer_schema_len,
                     self.ignore_errors,
+                    None,
+                    None,
+                    None,
                 )?;
                 let mut df: DataFrame = json_reader.as_df()?;
                 if self.rechunk {
@@ -360,7 +387,7 @@ where
     ///
     /// It is an error to pass `max_records = Some(0)`, as a schema cannot be inferred from 0 records when deserializing
     /// from JSON (unlike CSVs, there is no header row to inspect for column names).
-    pub fn infer_schema_len(mut self, max_records: Option<usize>) -> Self {
+    pub fn infer_schema_len(mut self, max_records: Option<NonZeroUsize>) -> Self {
         self.infer_schema_len = max_records;
         self
     }
