@@ -14,11 +14,11 @@ use tokio::sync::Mutex;
 
 use super::mmap::ColumnStore;
 use super::predicates::read_this_row_group;
-use super::read_impl::compute_row_group_range;
 use crate::cloud::{
-    build_object_store, object_path_from_string, CloudLocation, CloudOptions, PolarsObjectStore,
+    build_object_store, object_path_from_str, CloudLocation, CloudOptions, PolarsObjectStore,
 };
 use crate::parquet::metadata::FileMetaDataRef;
+use crate::parquet::read::metadata::PartitionedColumnChunkMD;
 use crate::pl_async::get_runtime;
 use crate::predicates::PhysicalIoExpr;
 
@@ -39,17 +39,8 @@ impl ParquetObjectStore {
         options: Option<&CloudOptions>,
         metadata: Option<FileMetaDataRef>,
     ) -> PolarsResult<Self> {
-        let (
-            CloudLocation {
-                prefix, expansion, ..
-            },
-            store,
-        ) = build_object_store(uri, options).await?;
-
-        // Any wildcards should already have been resolved here. Without this assertion they would
-        // be ignored.
-        debug_assert!(expansion.is_none(), "path should not contain wildcards");
-        let path = object_path_from_string(prefix)?;
+        let (CloudLocation { prefix, .. }, store) = build_object_store(uri, options, false).await?;
+        let path = object_path_from_str(&prefix)?;
 
         Ok(ParquetObjectStore {
             store: PolarsObjectStore::new(store),
@@ -271,8 +262,8 @@ impl FetchRowGroupsFromObjectStore {
         schema: ArrowSchemaRef,
         projection: Option<&[usize]>,
         predicate: Option<Arc<dyn PhysicalIoExpr>>,
+        row_group_range: Range<usize>,
         row_groups: &[RowGroupMetaData],
-        limit: usize,
     ) -> PolarsResult<Self> {
         let projected_fields: Option<Arc<[SmartString]>> = projection.map(|projection| {
             projection
@@ -281,26 +272,33 @@ impl FetchRowGroupsFromObjectStore {
                 .collect()
         });
 
-        let row_groups_end = compute_row_group_range(0, row_groups.len(), limit, row_groups);
-        let row_groups = &row_groups[0..row_groups_end];
-
         let mut prefetched: PlHashMap<usize, DownloadedRowGroup> = PlHashMap::new();
 
         let mut row_groups = if let Some(pred) = predicate.as_deref() {
-            row_groups
-                .iter()
-                .enumerate()
-                .filter(|(i, rg)| {
+            row_group_range
+                .filter_map(|i| {
+                    let rg = &row_groups[i];
+
+                    // TODO!
+                    // Optimize this. Now we partition the predicate columns twice. (later on reading as well)
+                    // I think we must add metadata context where we can cache and amortize the partitioning.
+                    let mut part_md = PartitionedColumnChunkMD::new(rg);
+                    let live = pred.live_variables();
+                    part_md.set_partitions(
+                        live.as_ref()
+                            .map(|vars| vars.iter().map(|s| s.as_ref()).collect::<PlHashSet<_>>())
+                            .as_ref(),
+                    );
                     let should_be_read =
-                        matches!(read_this_row_group(Some(pred), rg, &schema), Ok(true));
+                        matches!(read_this_row_group(Some(pred), &part_md, &schema), Ok(true));
 
                     // Already add the row groups that will be skipped to the prefetched data.
                     if !should_be_read {
-                        prefetched.insert(*i, Default::default());
+                        prefetched.insert(i, Default::default());
                     }
-                    should_be_read
+
+                    should_be_read.then(|| (i, rg.clone()))
                 })
-                .map(|(i, rg)| (i, rg.clone()))
                 .collect::<Vec<_>>()
         } else {
             row_groups.iter().cloned().enumerate().collect()
