@@ -2,12 +2,19 @@ use std::borrow::Cow;
 use std::collections::VecDeque;
 use std::ops::{Deref, Range};
 
-use arrow::array::new_empty_array;
+use arrow::array::BooleanArray;
+use arrow::bitmap::{Bitmap, MutableBitmap};
 use arrow::datatypes::ArrowSchemaRef;
 use polars_core::prelude::*;
 use polars_core::utils::{accumulate_dataframes_vertical, split_df};
 use polars_core::POOL;
-use polars_parquet::read::{self, ArrayIter, FileMetaData, PhysicalType, RowGroupMetaData};
+use polars_parquet::parquet::error::ParquetResult;
+use polars_parquet::parquet::statistics::Statistics;
+use polars_parquet::read::{
+    self, ColumnChunkMetaData, FileMetaData, Filter, PhysicalType, RowGroupMetaData,
+};
+use polars_utils::mmap::MemSlice;
+use polars_utils::vec::inplace_zip_filtermap;
 use rayon::prelude::*;
 
 #[cfg(feature = "cloud")]
@@ -20,43 +27,47 @@ use super::{mmap, ParallelStrategy};
 use crate::hive::materialize_hive_partitions;
 use crate::mmap::{MmapBytesReader, ReaderBytes};
 use crate::parquet::metadata::FileMetaDataRef;
+use crate::parquet::read::metadata::PartitionedColumnChunkMD;
+use crate::parquet::read::ROW_COUNT_OVERFLOW_ERR;
 use crate::predicates::{apply_predicate, PhysicalIoExpr};
 use crate::utils::get_reader_bytes;
+use crate::utils::slice::split_slice_at_file;
 use crate::RowIndex;
 
 #[cfg(debug_assertions)]
 // Ensure we get the proper polars types from schema inference
 // This saves unneeded casts.
 fn assert_dtypes(data_type: &ArrowDataType) {
+    use ArrowDataType as D;
+
     match data_type {
-        ArrowDataType::Utf8 => {
-            unreachable!()
-        },
-        ArrowDataType::Binary => {
-            unreachable!()
-        },
-        ArrowDataType::List(_) => {
-            unreachable!()
-        },
-        ArrowDataType::LargeList(inner) => {
-            assert_dtypes(&inner.data_type);
-        },
-        ArrowDataType::Struct(fields) => {
-            for fld in fields {
-                assert_dtypes(fld.data_type())
-            }
-        },
+        // These should all be casted to the BinaryView / Utf8View variants
+        D::Utf8 | D::Binary | D::LargeUtf8 | D::LargeBinary => unreachable!(),
+
+        // This should have been converted to a LargeList
+        D::List(_) => unreachable!(),
+
+        // This should have been converted to a LargeList(Struct(_))
+        D::Map(_, _) => unreachable!(),
+
+        // Recursive checks
+        D::Dictionary(_, data_type, _) => assert_dtypes(data_type),
+        D::Extension(_, data_type, _) => assert_dtypes(data_type),
+        D::LargeList(inner) => assert_dtypes(&inner.data_type),
+        D::FixedSizeList(inner, _) => assert_dtypes(&inner.data_type),
+        D::Struct(fields) => fields.iter().for_each(|f| assert_dtypes(f.data_type())),
+
         _ => {},
     }
 }
 
 fn column_idx_to_series(
     column_i: usize,
-    md: &RowGroupMetaData,
-    remaining_rows: usize,
+    // The metadata belonging to this column
+    field_md: &[&ColumnChunkMetaData],
+    filter: Option<Filter>,
     file_schema: &ArrowSchema,
     store: &mmap::ColumnStore,
-    chunk_size: usize,
 ) -> PolarsResult<Series> {
     let field = &file_schema.fields[column_i];
 
@@ -64,19 +75,26 @@ fn column_idx_to_series(
     {
         assert_dtypes(field.data_type())
     }
+    let columns = mmap_columns(store, field_md);
+    let stats = columns
+        .iter()
+        .map(|(col_md, _)| col_md.statistics().transpose())
+        .collect::<ParquetResult<Vec<Option<Statistics>>>>();
+    let array = mmap::to_deserializer(columns, field.clone(), filter)?;
+    let mut series = Series::try_from((field, array))?;
 
-    let columns = mmap_columns(store, md.columns(), &field.name);
-    let iter = mmap::to_deserializer(columns, field.clone(), remaining_rows, Some(chunk_size))?;
-
-    let mut series = if remaining_rows < md.num_rows() {
-        array_iter_to_series(iter, field, Some(remaining_rows))
-    } else {
-        array_iter_to_series(iter, field, None)
-    }?;
+    // We cannot really handle nested metadata at the moment. Just skip it.
+    use ArrowDataType as AD;
+    match field.data_type() {
+        AD::List(_) | AD::LargeList(_) | AD::Struct(_) | AD::FixedSizeList(_, _) => {
+            return Ok(series)
+        },
+        _ => {},
+    }
 
     // See if we can find some statistics for this series. If we cannot find anything just return
     // the series as is.
-    let Some(Ok(stats)) = md.columns()[column_i].statistics() else {
+    let Ok(Some(stats)) = stats.map(|mut s| s.pop().flatten()) else {
         return Ok(series);
     };
 
@@ -118,45 +136,13 @@ fn column_idx_to_series(
     Ok(series)
 }
 
-pub(super) fn array_iter_to_series(
-    iter: ArrayIter,
-    field: &ArrowField,
-    num_rows: Option<usize>,
-) -> PolarsResult<Series> {
-    let mut total_count = 0;
-    let chunks = match num_rows {
-        None => iter.collect::<PolarsResult<Vec<_>>>()?,
-        Some(n) => {
-            let mut out = Vec::with_capacity(2);
-
-            for arr in iter {
-                let arr = arr?;
-                let len = arr.len();
-                out.push(arr);
-
-                total_count += len;
-                if total_count >= n {
-                    break;
-                }
-            }
-            out
-        },
-    };
-    if chunks.is_empty() {
-        let arr = new_empty_array(field.data_type.clone());
-        Series::try_from((field, arr))
-    } else {
-        Series::try_from((field, chunks))
-    }
-}
-
 #[allow(clippy::too_many_arguments)]
 fn rg_to_dfs(
     store: &mmap::ColumnStore,
     previous_row_count: &mut IdxSize,
     row_group_start: usize,
     row_group_end: usize,
-    remaining_rows: &mut usize,
+    slice: (usize, usize),
     file_metadata: &FileMetaData,
     schema: &ArrowSchemaRef,
     predicate: Option<&dyn PhysicalIoExpr>,
@@ -166,13 +152,36 @@ fn rg_to_dfs(
     use_statistics: bool,
     hive_partition_columns: Option<&[Series]>,
 ) -> PolarsResult<Vec<DataFrame>> {
-    if let ParallelStrategy::Columns | ParallelStrategy::None = parallel {
-        rg_to_dfs_optionally_par_over_columns(
+    use ParallelStrategy as S;
+
+    if parallel == S::Prefiltered {
+        if let Some(predicate) = predicate {
+            if let Some(live_variables) = predicate.live_variables() {
+                return rg_to_dfs_prefiltered(
+                    store,
+                    previous_row_count,
+                    row_group_start,
+                    row_group_end,
+                    file_metadata,
+                    schema,
+                    live_variables,
+                    predicate,
+                    row_index,
+                    projection,
+                    use_statistics,
+                    hive_partition_columns,
+                );
+            }
+        }
+    }
+
+    match parallel {
+        S::Columns | S::None => rg_to_dfs_optionally_par_over_columns(
             store,
             previous_row_count,
             row_group_start,
             row_group_end,
-            remaining_rows,
+            slice,
             file_metadata,
             schema,
             predicate,
@@ -181,14 +190,13 @@ fn rg_to_dfs(
             projection,
             use_statistics,
             hive_partition_columns,
-        )
-    } else {
-        rg_to_dfs_par_over_rg(
+        ),
+        _ => rg_to_dfs_par_over_rg(
             store,
             row_group_start,
             row_group_end,
             previous_row_count,
-            remaining_rows,
+            slice,
             file_metadata,
             schema,
             predicate,
@@ -196,10 +204,329 @@ fn rg_to_dfs(
             projection,
             use_statistics,
             hive_partition_columns,
+        ),
+    }
+}
+
+/// Collect a HashSet of the projected columns.
+///  Returns `None` if all columns are projected.
+fn projected_columns_set<'a>(
+    schema: &'a ArrowSchema,
+    projection: &[usize],
+) -> Option<PlHashSet<&'a str>> {
+    if projection.len() == schema.len() {
+        None
+    } else {
+        Some(
+            projection
+                .iter()
+                .map(|i| schema.fields[*i].name.as_str())
+                .collect::<PlHashSet<_>>(),
         )
     }
 }
 
+#[allow(clippy::too_many_arguments)]
+fn rg_to_dfs_prefiltered(
+    store: &mmap::ColumnStore,
+    previous_row_count: &mut IdxSize,
+    row_group_start: usize,
+    row_group_end: usize,
+    file_metadata: &FileMetaData,
+    schema: &ArrowSchemaRef,
+    live_variables: Vec<Arc<str>>,
+    predicate: &dyn PhysicalIoExpr,
+    row_index: Option<RowIndex>,
+    projection: &[usize],
+    use_statistics: bool,
+    hive_partition_columns: Option<&[Series]>,
+) -> PolarsResult<Vec<DataFrame>> {
+    struct RowGroupInfo {
+        index: u32,
+        row_offset: IdxSize,
+    }
+
+    if row_group_end > u32::MAX as usize {
+        polars_bail!(ComputeError: "Parquet file contains too many row groups (> {})", u32::MAX);
+    }
+
+    let projected_columns = projected_columns_set(schema, projection);
+
+    let part_mds = POOL.install(|| {
+        file_metadata
+            .row_groups
+            .par_iter()
+            .map(|rg| {
+                let mut part_md = PartitionedColumnChunkMD::new(rg);
+                part_md.set_partitions(projected_columns.as_ref());
+                part_md
+            })
+            .collect::<Vec<_>>()
+    });
+
+    let mut row_offset = *previous_row_count;
+    let mut row_groups: Vec<RowGroupInfo> = (row_group_start..row_group_end)
+        .filter_map(|index| {
+            let part_md = &part_mds[index];
+            let md = &file_metadata.row_groups[index];
+
+            let current_offset = row_offset;
+            let current_row_count = md.num_rows() as IdxSize;
+            row_offset += current_row_count;
+
+            if use_statistics {
+                match read_this_row_group(Some(predicate), part_md, schema) {
+                    Ok(false) => return None,
+                    Ok(true) => {},
+                    Err(e) => return Some(Err(e)),
+                }
+            }
+
+            Some(Ok(RowGroupInfo {
+                index: index as u32,
+                row_offset: current_offset,
+            }))
+        })
+        .collect::<PolarsResult<Vec<_>>>()?;
+
+    // Deduplicate the live variables
+    let live_variables = live_variables
+        .iter()
+        .map(Deref::deref)
+        .collect::<PlHashSet<_>>();
+
+    // Get the number of live columns
+    let num_live_columns = live_variables.len();
+    let num_dead_columns = projection.len() - num_live_columns;
+
+    // We create two look-up tables that map indexes offsets into the live- and dead-set onto
+    // column indexes of the schema.
+    let mut live_idx_to_col_idx = Vec::with_capacity(num_live_columns);
+    let mut dead_idx_to_col_idx = Vec::with_capacity(num_dead_columns);
+    for (i, field) in schema.fields.iter().enumerate() {
+        if live_variables.contains(&field.name[..]) {
+            live_idx_to_col_idx.push(i);
+        } else {
+            dead_idx_to_col_idx.push(i);
+        }
+    }
+
+    debug_assert_eq!(live_idx_to_col_idx.len(), num_live_columns);
+    debug_assert_eq!(dead_idx_to_col_idx.len(), num_dead_columns);
+
+    enum MaskSetting {
+        Auto,
+        Pre,
+        Post,
+    }
+
+    let mask_setting =
+        std::env::var("POLARS_PQ_PREFILTERED_MASK").map_or(MaskSetting::Auto, |v| match &v[..] {
+            "auto" => MaskSetting::Auto,
+            "pre" => MaskSetting::Pre,
+            "post" => MaskSetting::Post,
+            _ => panic!("Invalid `POLARS_PQ_PREFILTERED_MASK` value '{v}'."),
+        });
+
+    POOL.install(|| {
+        // Set partitioned fields to prevent quadratic behavior.
+        // Ensure all row groups are partitioned.
+
+        // Collect the data for the live columns
+        let mut live_columns = (0..row_groups.len() * num_live_columns)
+            .into_par_iter()
+            .map(|i| {
+                let col_idx = live_idx_to_col_idx[i % num_live_columns];
+
+                let name = &schema.fields[col_idx].name;
+                let rg_idx = row_groups[i / num_live_columns].index;
+                let field_md = part_mds[rg_idx as usize].get_partitions(name).unwrap();
+
+                column_idx_to_series(col_idx, field_md.as_slice(), None, schema, store)
+            })
+            .collect::<PolarsResult<Vec<_>>>()?;
+
+        // Apply the predicate to the live columns and save the dataframe and the bitmask
+        let mut dfs = live_columns
+            .par_chunks_exact_mut(num_live_columns)
+            .enumerate()
+            .map(|(i, columns)| {
+                let rg = &row_groups[i];
+                let rg_idx = rg.index as usize;
+
+                let columns = columns.iter_mut().map(std::mem::take).collect::<Vec<_>>();
+
+                let md = &file_metadata.row_groups[rg_idx];
+                let mut df = unsafe { DataFrame::new_no_checks(columns) };
+
+                materialize_hive_partitions(
+                    &mut df,
+                    schema.as_ref(),
+                    hive_partition_columns,
+                    md.num_rows(),
+                );
+                let s = predicate.evaluate_io(&df)?;
+                let mask = s.bool().expect("filter predicates was not of type boolean");
+
+                if let Some(rc) = &row_index {
+                    df.with_row_index_mut(&rc.name, Some(rg.row_offset + rc.offset));
+                }
+                df = df.filter(mask)?;
+
+                let mut bitmap = MutableBitmap::with_capacity(mask.len());
+
+                // We need to account for the validity of the items
+                for chunk in mask.downcast_iter() {
+                    match chunk.validity() {
+                        None => bitmap.extend_from_bitmap(chunk.values()),
+                        Some(validity) => bitmap.extend_from_bitmap(&(validity & chunk.values())),
+                    }
+                }
+
+                let bitmap = bitmap.freeze();
+
+                debug_assert_eq!(md.num_rows(), bitmap.len());
+                debug_assert_eq!(df.height(), bitmap.set_bits());
+
+                Ok((bitmap, df))
+            })
+            .collect::<PolarsResult<Vec<(Bitmap, DataFrame)>>>()?;
+
+        // Filter out the row-groups that do not include any rows that match the predicate.
+        inplace_zip_filtermap(&mut dfs, &mut row_groups, |(mask, df), rg| {
+            (mask.set_bits() > 0).then_some(((mask, df), rg))
+        });
+
+        for (_, df) in &dfs {
+            let height = IdxSize::try_from(df.height()).map_err(|_| ROW_COUNT_OVERFLOW_ERR)?;
+            *previous_row_count = previous_row_count
+                .checked_add(height)
+                .ok_or(ROW_COUNT_OVERFLOW_ERR)?;
+        }
+
+        // We don't need to do any further work if there are no dead columns
+        if num_dead_columns == 0 {
+            return Ok(dfs.into_iter().map(|(_, df)| df).collect());
+        }
+
+        let rg_prefilter_costs = matches!(mask_setting, MaskSetting::Auto)
+            .then(|| {
+                dfs.par_iter()
+                    .map(|(mask, _)| {
+                        let num_edges = mask.num_edges() as f64;
+                        let rg_len = mask.len() as f64;
+
+                        // @GB: I did quite some analysis on this.
+                        //
+                        // Pre-filtered and Post-filtered can both be faster in certain scenarios.
+                        //
+                        // - Pre-filtered is faster when there is some amount of clustering or
+                        // sorting involved or if the number of values selected is small.
+                        // - Post-filtering is faster when the predicate selects a somewhat random
+                        // elements throughout the row group.
+                        //
+                        // The following is a heuristic value to try and estimate which one is
+                        // faster. Essentially, it sees how many times it needs to switch between
+                        // skipping items and collecting items and compares it against the number
+                        // of values that it will collect.
+                        //
+                        // Closer to 0: post-filtering is probably better.
+                        // Closer to 1: pre-filtering is probably better.
+                        (num_edges / rg_len).clamp(0.0, 1.0)
+                    })
+                    .collect::<Vec<_>>()
+            })
+            .unwrap_or_default();
+
+        let mut rg_columns = (0..dfs.len() * num_dead_columns)
+            .into_par_iter()
+            .map(|i| {
+                let col_idx = dead_idx_to_col_idx[i % num_dead_columns];
+                let name = &schema.fields[col_idx].name;
+
+                let (mask, _) = &dfs[i / num_dead_columns];
+
+                let rg_idx = row_groups[i / num_dead_columns].index;
+
+                #[cfg(debug_assertions)]
+                {
+                    let md = &file_metadata.row_groups[rg_idx as usize];
+                    debug_assert_eq!(md.num_rows(), mask.len());
+                }
+                let field_md = part_mds[rg_idx as usize].get_partitions(name).unwrap();
+
+                let pre = || {
+                    column_idx_to_series(
+                        col_idx,
+                        field_md.as_slice(),
+                        Some(Filter::new_masked(mask.clone())),
+                        schema,
+                        store,
+                    )
+                };
+                let post = || {
+                    let array =
+                        column_idx_to_series(col_idx, field_md.as_slice(), None, schema, store)?;
+
+                    debug_assert_eq!(array.len(), mask.len());
+
+                    let mask_arr = BooleanArray::new(ArrowDataType::Boolean, mask.clone(), None);
+                    let mask_arr = BooleanChunked::from(mask_arr);
+                    array.filter(&mask_arr)
+                };
+
+                let array = match mask_setting {
+                    MaskSetting::Auto => {
+                        // Prefiltering is more expensive for nested types so we make the cut-off
+                        // higher.
+                        let is_nested = schema.fields[col_idx].data_type.is_nested();
+                        let prefilter_cost = rg_prefilter_costs[i / num_dead_columns];
+
+                        // We empirically selected these numbers.
+                        let do_prefilter = (is_nested && prefilter_cost <= 0.01)
+                            || (!is_nested && prefilter_cost <= 0.02);
+
+                        if do_prefilter {
+                            pre()?
+                        } else {
+                            post()?
+                        }
+                    },
+                    MaskSetting::Pre => pre()?,
+                    MaskSetting::Post => post()?,
+                };
+
+                debug_assert_eq!(array.len(), mask.set_bits());
+
+                Ok(array)
+            })
+            .collect::<PolarsResult<Vec<_>>>()?;
+
+        let Some(df) = dfs.first().map(|(_, df)| df) else {
+            return Ok(Vec::new());
+        };
+        let mut rearranged_schema = df.schema();
+        rearranged_schema.merge(Schema::from(schema));
+
+        rg_columns
+            .par_chunks_exact_mut(num_dead_columns)
+            .zip(dfs)
+            .map(|(rg_cols, (_, mut df))| {
+                let rg_cols = rg_cols.iter_mut().map(std::mem::take).collect::<Vec<_>>();
+
+                debug_assert!(rg_cols.iter().all(|v| v.len() == df.height()));
+
+                // We first add the columns with the live columns at the start. Then, we do a
+                // projections that puts the columns at the right spot.
+                df._add_columns(rg_cols, &rearranged_schema)?;
+                let df = df.select(schema.get_names())?;
+
+                PolarsResult::Ok(df)
+            })
+            .collect::<PolarsResult<Vec<DataFrame>>>()
+    })
+}
+
 #[allow(clippy::too_many_arguments)]
 // might parallelize over columns
 fn rg_to_dfs_optionally_par_over_columns(
@@ -207,7 +534,7 @@ fn rg_to_dfs_optionally_par_over_columns(
     previous_row_count: &mut IdxSize,
     row_group_start: usize,
     row_group_end: usize,
-    remaining_rows: &mut usize,
+    slice: (usize, usize),
     file_metadata: &FileMetaData,
     schema: &ArrowSchemaRef,
     predicate: Option<&dyn PhysicalIoExpr>,
@@ -219,14 +546,25 @@ fn rg_to_dfs_optionally_par_over_columns(
 ) -> PolarsResult<Vec<DataFrame>> {
     let mut dfs = Vec::with_capacity(row_group_end - row_group_start);
 
+    let mut n_rows_processed: usize = (0..row_group_start)
+        .map(|i| file_metadata.row_groups[i].num_rows())
+        .sum();
+    let slice_end = slice.0 + slice.1;
+
     for rg_idx in row_group_start..row_group_end {
         let md = &file_metadata.row_groups[rg_idx];
+
+        // Set partitioned fields to prevent quadratic behavior.
+        let projected_columns = projected_columns_set(schema, projection);
+        let mut part_md = PartitionedColumnChunkMD::new(md);
+        part_md.set_partitions(projected_columns.as_ref());
+
+        let rg_slice =
+            split_slice_at_file(&mut n_rows_processed, md.num_rows(), slice.0, slice_end);
         let current_row_count = md.num_rows() as IdxSize;
 
-        if use_statistics
-            && !read_this_row_group(predicate, &file_metadata.row_groups[rg_idx], schema)?
-        {
-            *previous_row_count += current_row_count;
+        if use_statistics && !read_this_row_group(predicate, &part_md, schema)? {
+            *previous_row_count += rg_slice.1 as IdxSize;
             continue;
         }
         // test we don't read the parquet file if this env var is set
@@ -235,20 +573,20 @@ fn rg_to_dfs_optionally_par_over_columns(
             assert!(std::env::var("POLARS_PANIC_IF_PARQUET_PARSED").is_err())
         }
 
-        let projection_height = (*remaining_rows).min(md.num_rows());
-        let chunk_size = md.num_rows();
         let columns = if let ParallelStrategy::Columns = parallel {
             POOL.install(|| {
                 projection
                     .par_iter()
                     .map(|column_i| {
+                        let name = &schema.fields[*column_i].name;
+                        let part = part_md.get_partitions(name).unwrap();
+
                         column_idx_to_series(
                             *column_i,
-                            md,
-                            projection_height,
+                            part.as_slice(),
+                            Some(Filter::new_ranged(rg_slice.0, rg_slice.0 + rg_slice.1)),
                             schema,
                             store,
-                            chunk_size,
                         )
                     })
                     .collect::<PolarsResult<Vec<_>>>()
@@ -257,40 +595,42 @@ fn rg_to_dfs_optionally_par_over_columns(
             projection
                 .iter()
                 .map(|column_i| {
+                    let name = &schema.fields[*column_i].name;
+                    let part = part_md.get_partitions(name).unwrap();
+
                     column_idx_to_series(
                         *column_i,
-                        md,
-                        projection_height,
+                        part.as_slice(),
+                        Some(Filter::new_ranged(rg_slice.0, rg_slice.0 + rg_slice.1)),
                         schema,
                         store,
-                        chunk_size,
                     )
                 })
                 .collect::<PolarsResult<Vec<_>>>()?
         };
 
-        *remaining_rows -= projection_height;
-
         let mut df = unsafe { DataFrame::new_no_checks(columns) };
         if let Some(rc) = &row_index {
             df.with_row_index_mut(&rc.name, Some(*previous_row_count + rc.offset));
         }
 
-        materialize_hive_partitions(
-            &mut df,
-            schema.as_ref(),
-            hive_partition_columns,
-            projection_height,
-        );
+        materialize_hive_partitions(&mut df, schema.as_ref(), hive_partition_columns, rg_slice.1);
         apply_predicate(&mut df, predicate, true)?;
 
-        *previous_row_count += current_row_count;
+        *previous_row_count = previous_row_count.checked_add(current_row_count).ok_or_else(||
+            polars_err!(
+                ComputeError: "Parquet file produces more than pow(2, 32) rows; \
+                consider compiling with polars-bigidx feature (polars-u64-idx package on python), \
+                or set 'streaming'"
+            ),
+        )?;
         dfs.push(df);
 
-        if *remaining_rows == 0 {
+        if *previous_row_count as usize >= slice_end {
             break;
         }
     }
+
     Ok(dfs)
 }
 
@@ -301,7 +641,7 @@ fn rg_to_dfs_par_over_rg(
     row_group_start: usize,
     row_group_end: usize,
     previous_row_count: &mut IdxSize,
-    remaining_rows: &mut usize,
+    slice: (usize, usize),
     file_metadata: &FileMetaData,
     schema: &ArrowSchemaRef,
     predicate: Option<&dyn PhysicalIoExpr>,
@@ -311,34 +651,52 @@ fn rg_to_dfs_par_over_rg(
     hive_partition_columns: Option<&[Series]>,
 ) -> PolarsResult<Vec<DataFrame>> {
     // compute the limits per row group and the row count offsets
-    let row_groups = file_metadata
-        .row_groups
-        .iter()
-        .enumerate()
-        .skip(row_group_start)
-        .take(row_group_end - row_group_start)
-        .map(|(rg_idx, rg_md)| {
-            let row_count_start = *previous_row_count;
-            let num_rows = rg_md.num_rows();
-            *previous_row_count += num_rows as IdxSize;
-            let projection_height = (*remaining_rows).min(num_rows);
-            *remaining_rows -= projection_height;
+    let mut row_groups = Vec::with_capacity(row_group_end - row_group_start);
 
-            (rg_idx, rg_md, projection_height, row_count_start)
-        })
-        .collect::<Vec<_>>();
+    let mut n_rows_processed: usize = (0..row_group_start)
+        .map(|i| file_metadata.row_groups[i].num_rows())
+        .sum();
+    let slice_end = slice.0 + slice.1;
+
+    for i in row_group_start..row_group_end {
+        let row_count_start = *previous_row_count;
+        let rg_md = &file_metadata.row_groups[i];
+        let rg_slice =
+            split_slice_at_file(&mut n_rows_processed, rg_md.num_rows(), slice.0, slice_end);
+        *previous_row_count = previous_row_count
+            .checked_add(rg_slice.1 as IdxSize)
+            .ok_or(ROW_COUNT_OVERFLOW_ERR)?;
+
+        if rg_slice.1 == 0 {
+            continue;
+        }
+
+        row_groups.push((i, rg_md, rg_slice, row_count_start));
+    }
 
     let dfs = POOL.install(|| {
+        // Set partitioned fields to prevent quadratic behavior.
+        // Ensure all row groups are partitioned.
+        let part_mds = {
+            let projected_columns = projected_columns_set(schema, projection);
+            row_groups
+                .par_iter()
+                .map(|(_, rg, _, _)| {
+                    let mut ccmd = PartitionedColumnChunkMD::new(rg);
+                    ccmd.set_partitions(projected_columns.as_ref());
+                    ccmd
+                })
+                .collect::<Vec<_>>()
+        };
+
         row_groups
             .into_par_iter()
-            .map(|(rg_idx, md, projection_height, row_count_start)| {
-                if projection_height == 0
-                    || use_statistics
-                        && !read_this_row_group(
-                            predicate,
-                            &file_metadata.row_groups[rg_idx],
-                            schema,
-                        )?
+            .enumerate()
+            .map(|(iter_idx, (_rg_idx, _md, slice, row_count_start))| {
+                let part_md = &part_mds[iter_idx];
+
+                if slice.1 == 0
+                    || use_statistics && !read_this_row_group(predicate, part_md, schema)?
                 {
                     return Ok(None);
                 }
@@ -348,17 +706,18 @@ fn rg_to_dfs_par_over_rg(
                     assert!(std::env::var("POLARS_PANIC_IF_PARQUET_PARSED").is_err())
                 }
 
-                let chunk_size = md.num_rows();
                 let columns = projection
                     .iter()
                     .map(|column_i| {
+                        let name = &schema.fields[*column_i].name;
+                        let field_md = part_md.get_partitions(name).unwrap();
+
                         column_idx_to_series(
                             *column_i,
-                            md,
-                            projection_height,
+                            field_md.as_slice(),
+                            Some(Filter::new_ranged(slice.0, slice.0 + slice.1)),
                             schema,
                             store,
-                            chunk_size,
                         )
                     })
                     .collect::<PolarsResult<Vec<_>>>()?;
@@ -373,7 +732,7 @@ fn rg_to_dfs_par_over_rg(
                     &mut df,
                     schema.as_ref(),
                     hive_partition_columns,
-                    projection_height,
+                    slice.1,
                 );
                 apply_predicate(&mut df, predicate, false)?;
 
@@ -387,7 +746,7 @@ fn rg_to_dfs_par_over_rg(
 #[allow(clippy::too_many_arguments)]
 pub fn read_parquet<R: MmapBytesReader>(
     mut reader: R,
-    mut limit: usize,
+    slice: (usize, usize),
     projection: Option<&[usize]>,
     reader_schema: &ArrowSchemaRef,
     metadata: Option<FileMetaDataRef>,
@@ -398,7 +757,7 @@ pub fn read_parquet<R: MmapBytesReader>(
     hive_partition_columns: Option<&[Series]>,
 ) -> PolarsResult<DataFrame> {
     // Fast path.
-    if limit == 0 {
+    if slice.1 == 0 {
         return Ok(materialize_empty_df(
             projection,
             reader_schema,
@@ -446,15 +805,17 @@ pub fn read_parquet<R: MmapBytesReader>(
     }
 
     let reader = ReaderBytes::from(&mut reader);
-    let bytes = reader.deref();
-    let store = mmap::ColumnStore::Local(bytes);
+    let store = mmap::ColumnStore::Local(
+        unsafe { std::mem::transmute::<ReaderBytes<'_>, ReaderBytes<'static>>(reader) }
+            .into_mem_slice(),
+    );
 
     let dfs = rg_to_dfs(
         &store,
         &mut 0,
         0,
         n_row_groups,
-        &mut limit,
+        slice,
         &file_metadata,
         reader_schema,
         predicate,
@@ -492,8 +853,12 @@ impl FetchRowGroupsFromMmapReader {
         let reader_bytes = get_reader_bytes(reader_ptr)?;
         Ok(FetchRowGroupsFromMmapReader(reader_bytes))
     }
+
     fn fetch_row_groups(&mut self, _row_groups: Range<usize>) -> PolarsResult<ColumnStore> {
-        Ok(mmap::ColumnStore::Local(self.0.deref()))
+        // @TODO: we can something smarter here with mmap
+        Ok(mmap::ColumnStore::Local(MemSlice::from_vec(
+            self.0.deref().to_vec(),
+        )))
     }
 }
 
@@ -531,28 +896,51 @@ impl RowGroupFetcher {
 pub(super) fn compute_row_group_range(
     row_group_start: usize,
     row_group_end: usize,
-    limit: usize,
+    slice: (usize, usize),
     row_groups: &[RowGroupMetaData],
-) -> usize {
-    let mut row_group_end_truncated = row_group_start;
-    let mut acc_row_count = 0;
+) -> std::ops::Range<usize> {
+    let mut start = row_group_start;
+    let mut cum_rows: usize = (0..row_group_start).map(|i| row_groups[i].num_rows()).sum();
+    let row_group_end = row_groups.len().min(row_group_end);
 
-    #[allow(clippy::needless_range_loop)]
-    for rg_i in row_group_start..(std::cmp::min(row_group_end, row_groups.len())) {
-        if acc_row_count >= limit {
+    loop {
+        if start == row_group_end {
             break;
         }
-        row_group_end_truncated = rg_i + 1;
-        acc_row_count += row_groups[rg_i].num_rows();
+
+        cum_rows += row_groups[start].num_rows();
+
+        if cum_rows >= slice.0 {
+            break;
+        }
+
+        start += 1;
     }
-    row_group_end_truncated
+
+    let slice_end = slice.0 + slice.1;
+    let mut end = (1 + start).min(row_group_end);
+
+    loop {
+        if end == row_group_end {
+            break;
+        }
+
+        if cum_rows >= slice_end {
+            break;
+        }
+
+        cum_rows += row_groups[end].num_rows();
+        end += 1;
+    }
+
+    start..end
 }
 
 pub struct BatchedParquetReader {
     // use to keep ownership
     #[allow(dead_code)]
     row_group_fetcher: RowGroupFetcher,
-    limit: usize,
+    slice: (usize, usize),
     projection: Arc<[usize]>,
     schema: ArrowSchemaRef,
     metadata: FileMetaDataRef,
@@ -577,7 +965,7 @@ impl BatchedParquetReader {
         row_group_fetcher: RowGroupFetcher,
         metadata: FileMetaDataRef,
         schema: ArrowSchemaRef,
-        limit: usize,
+        slice: (usize, usize),
         projection: Option<Vec<usize>>,
         predicate: Option<Arc<dyn PhysicalIoExpr>>,
         row_index: Option<RowIndex>,
@@ -609,7 +997,7 @@ impl BatchedParquetReader {
 
         Ok(BatchedParquetReader {
             row_group_fetcher,
-            limit,
+            slice,
             projection,
             schema,
             metadata,
@@ -629,10 +1017,6 @@ impl BatchedParquetReader {
         })
     }
 
-    pub fn limit_reached(&self) -> bool {
-        self.limit == 0
-    }
-
     pub fn schema(&self) -> &ArrowSchemaRef {
         &self.schema
     }
@@ -646,7 +1030,7 @@ impl BatchedParquetReader {
     }
 
     pub async fn next_batches(&mut self, n: usize) -> PolarsResult<Option<Vec<DataFrame>>> {
-        if self.limit == 0 && self.has_returned {
+        if self.rows_read as usize == self.slice.0 + self.slice.1 && self.has_returned {
             return if self.chunks_fifo.is_empty() {
                 Ok(None)
             } else {
@@ -658,28 +1042,30 @@ impl BatchedParquetReader {
 
         let mut skipped_all_rgs = false;
         // fill up fifo stack
-        if self.row_group_offset < self.n_row_groups && self.chunks_fifo.len() < n {
+        if (self.rows_read as usize) < self.slice.0 + self.slice.1
+            && self.row_group_offset < self.n_row_groups
+            && self.chunks_fifo.len() < n
+        {
             // Ensure we apply the limit on the metadata, before we download the row-groups.
-            let row_group_start = self.row_group_offset;
-            let row_group_end = compute_row_group_range(
-                row_group_start,
-                row_group_start + n,
-                self.limit,
+            let row_group_range = compute_row_group_range(
+                self.row_group_offset,
+                self.row_group_offset + n,
+                self.slice,
                 &self.metadata.row_groups,
             );
 
             let store = self
                 .row_group_fetcher
-                .fetch_row_groups(row_group_start..row_group_end)
+                .fetch_row_groups(row_group_range.clone())
                 .await?;
 
             let mut dfs = match store {
                 ColumnStore::Local(_) => rg_to_dfs(
                     &store,
                     &mut self.rows_read,
-                    row_group_start,
-                    row_group_end,
-                    &mut self.limit,
+                    row_group_range.start,
+                    row_group_range.end,
+                    self.slice,
                     &self.metadata,
                     &self.schema,
                     self.predicate.as_deref(),
@@ -700,7 +1086,6 @@ impl BatchedParquetReader {
 
                     // Make everything 'static.
                     let mut rows_read = self.rows_read;
-                    let mut limit = self.limit;
                     let row_index = self.row_index.clone();
                     let predicate = self.predicate.clone();
                     let schema = self.schema.clone();
@@ -709,14 +1094,15 @@ impl BatchedParquetReader {
                     let projection = self.projection.clone();
                     let use_statistics = self.use_statistics;
                     let hive_partition_columns = self.hive_partition_columns.clone();
+                    let slice = self.slice;
 
                     let f = move || {
                         let dfs = rg_to_dfs(
                             &store,
                             &mut rows_read,
-                            row_group_start,
-                            row_group_end,
-                            &mut limit,
+                            row_group_range.start,
+                            row_group_range.end,
+                            slice,
                             &metadata,
                             &schema,
                             predicate.as_deref(),
@@ -728,7 +1114,7 @@ impl BatchedParquetReader {
                         );
 
                         // Don't unwrap send attempt - async task could be cancelled.
-                        let _ = tx.send((dfs, rows_read, limit));
+                        let _ = tx.send((dfs, rows_read));
                     };
 
                     // Spawn the task and wait on it asynchronously.
@@ -741,10 +1127,8 @@ impl BatchedParquetReader {
                         POOL.spawn(f);
                     };
 
-                    let (dfs, rows_read, limit) = rx.await.unwrap();
-
+                    let (dfs, rows_read) = rx.await.unwrap();
                     self.rows_read = rows_read;
-                    self.limit = limit;
                     dfs
                 },
             }?;
@@ -762,7 +1146,7 @@ impl BatchedParquetReader {
 
                 // Re-use the same ChunkedArray
                 if ca.len() < max_len {
-                    *ca = ca.new_from_index(max_len, 0);
+                    *ca = ca.new_from_index(0, max_len);
                 }
 
                 for df in &mut dfs {
