@@ -88,7 +88,7 @@ pub fn expand_paths(
     paths: &[PathBuf],
     glob: bool,
     #[allow(unused_variables)] cloud_options: Option<&CloudOptions>,
-) -> PolarsResult<Arc<[PathBuf]>> {
+) -> PolarsResult<Arc<Vec<PathBuf>>> {
     expand_paths_hive(paths, glob, cloud_options, false).map(|x| x.0)
 }
 
@@ -129,7 +129,7 @@ pub fn expand_paths_hive(
     glob: bool,
     #[allow(unused_variables)] cloud_options: Option<&CloudOptions>,
     check_directory_level: bool,
-) -> PolarsResult<(Arc<[PathBuf]>, usize)> {
+) -> PolarsResult<(Arc<Vec<PathBuf>>, usize)> {
     let Some(first_path) = paths.first() else {
         return Ok((vec![].into(), 0));
     };
@@ -146,13 +146,18 @@ pub fn expand_paths_hive(
     if is_cloud || { cfg!(not(target_family = "windows")) && config::force_async() } {
         #[cfg(feature = "cloud")]
         {
-            use crate::cloud::object_path_from_string;
+            use polars_utils::_limit_path_len_io_err;
+
+            use crate::cloud::object_path_from_str;
 
             if first_path.starts_with("hf://") {
-                let (expand_start_idx, paths) =
-                    crate::pl_async::get_runtime().block_on_potential_spawn(
-                        hugging_face::expand_paths_hf(paths, check_directory_level),
-                    )?;
+                let (expand_start_idx, paths) = crate::pl_async::get_runtime()
+                    .block_on_potential_spawn(hugging_face::expand_paths_hf(
+                        paths,
+                        check_directory_level,
+                        cloud_options,
+                        glob,
+                    ))?;
 
                 return Ok((Arc::from(paths), expand_start_idx));
             }
@@ -169,25 +174,25 @@ pub fn expand_paths_hive(
                                      cloud_options: Option<&CloudOptions>|
              -> PolarsResult<(usize, Vec<PathBuf>)> {
                 crate::pl_async::get_runtime().block_on_potential_spawn(async {
-                    if path.starts_with("http") {
-                        return Ok((0, vec![PathBuf::from(path)]));
-                    }
-
                     let (cloud_location, store) =
-                        crate::cloud::build_object_store(path, cloud_options).await?;
-
-                    let prefix = object_path_from_string(cloud_location.prefix.clone())?;
+                        crate::cloud::build_object_store(path, cloud_options, glob).await?;
+                    let prefix = object_path_from_str(&cloud_location.prefix)?;
 
                     let out = if !path.ends_with("/")
-                        && cloud_location.expansion.is_none()
-                        && store.head(&prefix).await.is_ok()
-                    {
+                        && (!glob || cloud_location.expansion.is_none())
+                        && {
+                            // We need to check if it is a directory for local paths (we can be here due
+                            // to FORCE_ASYNC). For cloud paths the convention is that the user must add
+                            // a trailing slash `/` to scan directories. We don't infer it as that would
+                            // mean sending one network request per path serially (very slow).
+                            is_cloud || PathBuf::from(path).is_file()
+                        } {
                         (
                             0,
                             vec![PathBuf::from(format_path(
                                 &cloud_location.scheme,
                                 &cloud_location.bucket,
-                                &cloud_location.prefix,
+                                prefix.as_ref(),
                             ))],
                         )
                     } else {
@@ -200,14 +205,8 @@ pub fn expand_paths_hive(
                             // indistinguishable from an empty directory.
                             let path = PathBuf::from(path);
                             if !path.is_dir() {
-                                path.metadata().map_err(|err| {
-                                    let msg =
-                                        Some(format!("{}: {}", err, path.to_str().unwrap()).into());
-                                    PolarsError::IO {
-                                        error: err.into(),
-                                        msg,
-                                    }
-                                })?;
+                                path.metadata()
+                                    .map_err(|err| _limit_path_len_io_err(&path, err))?;
                             }
                         }
 
@@ -248,9 +247,15 @@ pub fn expand_paths_hive(
             };
 
             for (path_idx, path) in paths.iter().enumerate() {
+                if path.to_str().unwrap().starts_with("http") {
+                    out_paths.push(path.clone());
+                    hive_idx_tracker.update(0, path_idx)?;
+                    continue;
+                }
+
                 let glob_start_idx = get_glob_start_idx(path.to_str().unwrap().as_bytes());
 
-                let path = if glob_start_idx.is_some() {
+                let path = if glob && glob_start_idx.is_some() {
                     path.clone()
                 } else {
                     let (expand_start_idx, paths) =
@@ -351,12 +356,12 @@ pub fn expand_paths_hive(
 
                 Ok(path)
             })
-            .collect::<PolarsResult<Arc<[_]>>>()?
+            .collect::<PolarsResult<Vec<_>>>()?
     } else {
-        Arc::<[_]>::from(out_paths)
+        out_paths
     };
 
-    Ok((out_paths, hive_idx_tracker.idx))
+    Ok((Arc::new(out_paths), hive_idx_tracker.idx))
 }
 
 /// Ignores errors from `std::fs::create_dir_all` if the directory exists.
@@ -414,4 +419,18 @@ mod tests {
         assert!(resolved[1].is_absolute());
         assert!(resolved[2].is_absolute());
     }
+
+    #[test]
+    fn test_http_path_with_query_parameters_is_not_expanded_as_glob() {
+        // Don't confuse HTTP URL's with query parameters for globs.
+        // See https://github.com/pola-rs/polars/pull/17774
+        use std::path::PathBuf;
+
+        use super::expand_paths;
+
+        let path = "https://pola.rs/test.csv?token=bear";
+        let paths = &[PathBuf::from(path)];
+        let out = expand_paths(paths, true, None).unwrap();
+        assert_eq!(out.as_ref(), paths);
+    }
 }
