@@ -35,7 +35,7 @@ use polars_expr::{create_physical_expr, ExpressionConversionState};
 use polars_io::RowIndex;
 use polars_mem_engine::{create_physical_plan, Executor};
 use polars_ops::frame::JoinCoalesce;
-pub use polars_plan::frame::{AllowedOptimizations, OptState};
+pub use polars_plan::frame::{AllowedOptimizations, OptFlags};
 use polars_plan::global::FETCH_ROWS;
 use smartstring::alias::String as SmartString;
 
@@ -67,13 +67,14 @@ impl IntoLazy for LazyFrame {
 }
 
 /// Lazy abstraction over an eager `DataFrame`.
+///
 /// It really is an abstraction over a logical plan. The methods of this struct will incrementally
 /// modify a logical plan until output is requested (via [`collect`](crate::frame::LazyFrame::collect)).
 #[derive(Clone, Default)]
 #[must_use]
 pub struct LazyFrame {
     pub logical_plan: DslPlan,
-    pub(crate) opt_state: OptState,
+    pub(crate) opt_state: OptFlags,
     pub(crate) cached_arena: Arc<Mutex<Option<CachedArena>>>,
 }
 
@@ -81,10 +82,7 @@ impl From<DslPlan> for LazyFrame {
     fn from(plan: DslPlan) -> Self {
         Self {
             logical_plan: plan,
-            opt_state: OptState {
-                file_caching: true,
-                ..Default::default()
-            },
+            opt_state: OptFlags::default() | OptFlags::FILE_CACHING,
             cached_arena: Default::default(),
         }
     }
@@ -93,7 +91,7 @@ impl From<DslPlan> for LazyFrame {
 impl LazyFrame {
     pub(crate) fn from_inner(
         logical_plan: DslPlan,
-        opt_state: OptState,
+        opt_state: OptFlags,
         cached_arena: Arc<Mutex<Option<CachedArena>>>,
     ) -> Self {
         Self {
@@ -107,11 +105,11 @@ impl LazyFrame {
         DslBuilder::from(self.logical_plan)
     }
 
-    fn get_opt_state(&self) -> OptState {
+    fn get_opt_state(&self) -> OptFlags {
         self.opt_state
     }
 
-    fn from_logical_plan(logical_plan: DslPlan, opt_state: OptState) -> Self {
+    fn from_logical_plan(logical_plan: DslPlan, opt_state: OptFlags) -> Self {
         LazyFrame {
             logical_plan,
             opt_state,
@@ -120,109 +118,91 @@ impl LazyFrame {
     }
 
     /// Get current optimizations.
-    pub fn get_current_optimizations(&self) -> OptState {
+    pub fn get_current_optimizations(&self) -> OptFlags {
         self.opt_state
     }
 
     /// Set allowed optimizations.
-    pub fn with_optimizations(mut self, opt_state: OptState) -> Self {
+    pub fn with_optimizations(mut self, opt_state: OptFlags) -> Self {
         self.opt_state = opt_state;
         self
     }
 
     /// Turn off all optimizations.
     pub fn without_optimizations(self) -> Self {
-        self.with_optimizations(OptState {
-            projection_pushdown: false,
-            predicate_pushdown: false,
-            cluster_with_columns: false,
-            type_coercion: true,
-            simplify_expr: false,
-            slice_pushdown: false,
-            // will be toggled by a scan operation such as csv scan or parquet scan
-            file_caching: false,
-            #[cfg(feature = "cse")]
-            comm_subplan_elim: false,
-            #[cfg(feature = "cse")]
-            comm_subexpr_elim: false,
-            streaming: false,
-            eager: false,
-            fast_projection: false,
-            row_estimate: false,
-            new_streaming: false,
-        })
+        self.with_optimizations(OptFlags::from_bits_truncate(0) | OptFlags::TYPE_COERCION)
     }
 
     /// Toggle projection pushdown optimization.
     pub fn with_projection_pushdown(mut self, toggle: bool) -> Self {
-        self.opt_state.projection_pushdown = toggle;
+        self.opt_state.set(OptFlags::PROJECTION_PUSHDOWN, toggle);
         self
     }
 
     /// Toggle cluster with columns optimization.
     pub fn with_cluster_with_columns(mut self, toggle: bool) -> Self {
-        self.opt_state.cluster_with_columns = toggle;
+        self.opt_state.set(OptFlags::CLUSTER_WITH_COLUMNS, toggle);
         self
     }
 
     /// Toggle predicate pushdown optimization.
     pub fn with_predicate_pushdown(mut self, toggle: bool) -> Self {
-        self.opt_state.predicate_pushdown = toggle;
+        self.opt_state.set(OptFlags::PREDICATE_PUSHDOWN, toggle);
         self
     }
 
     /// Toggle type coercion optimization.
     pub fn with_type_coercion(mut self, toggle: bool) -> Self {
-        self.opt_state.type_coercion = toggle;
+        self.opt_state.set(OptFlags::TYPE_COERCION, toggle);
         self
     }
 
     /// Toggle expression simplification optimization on or off.
     pub fn with_simplify_expr(mut self, toggle: bool) -> Self {
-        self.opt_state.simplify_expr = toggle;
+        self.opt_state.set(OptFlags::SIMPLIFY_EXPR, toggle);
         self
     }
 
     /// Toggle common subplan elimination optimization on or off
     #[cfg(feature = "cse")]
     pub fn with_comm_subplan_elim(mut self, toggle: bool) -> Self {
-        self.opt_state.comm_subplan_elim = toggle;
+        self.opt_state.set(OptFlags::COMM_SUBPLAN_ELIM, toggle);
         self
     }
 
     /// Toggle common subexpression elimination optimization on or off
     #[cfg(feature = "cse")]
     pub fn with_comm_subexpr_elim(mut self, toggle: bool) -> Self {
-        self.opt_state.comm_subexpr_elim = toggle;
+        self.opt_state.set(OptFlags::COMM_SUBEXPR_ELIM, toggle);
         self
     }
 
     /// Toggle slice pushdown optimization.
     pub fn with_slice_pushdown(mut self, toggle: bool) -> Self {
-        self.opt_state.slice_pushdown = toggle;
+        self.opt_state.set(OptFlags::SLICE_PUSHDOWN, toggle);
         self
     }
 
     /// Run nodes that are capably of doing so on the streaming engine.
     pub fn with_streaming(mut self, toggle: bool) -> Self {
-        self.opt_state.streaming = toggle;
+        self.opt_state.set(OptFlags::STREAMING, toggle);
         self
     }
 
     pub fn with_new_streaming(mut self, toggle: bool) -> Self {
-        self.opt_state.new_streaming = toggle;
+        self.opt_state.set(OptFlags::NEW_STREAMING, toggle);
         self
     }
 
     /// Try to estimate the number of rows so that joins can determine which side to keep in memory.
     pub fn with_row_estimate(mut self, toggle: bool) -> Self {
-        self.opt_state.row_estimate = toggle;
+        self.opt_state.set(OptFlags::ROW_ESTIMATE, toggle);
         self
     }
 
     /// Run every node eagerly. This turns off multi-node optimizations.
     pub fn _with_eager(mut self, toggle: bool) -> Self {
-        self.opt_state.eager = toggle;
+        self.opt_state.set(OptFlags::EAGER, toggle);
         self
     }
 
@@ -342,28 +322,20 @@ impl LazyFrame {
     /// fn example(df: DataFrame) -> LazyFrame {
     ///       df.lazy()
     ///         .sort_by_exprs(vec![col("sepal_width")], Default::default())
-    ///         .unwrap()
     /// }
     /// ```
     pub fn sort_by_exprs<E: AsRef<[Expr]>>(
         self,
         by_exprs: E,
         sort_options: SortMultipleOptions,
-    ) -> PolarsResult<Self> {
+    ) -> Self {
         let by_exprs = by_exprs.as_ref().to_vec();
         if by_exprs.is_empty() {
-            Ok(self)
+            self
         } else {
-            let is_include_literal = by_exprs.iter().any(|expr| matches!(expr, Expr::Literal(_)));
-
-            polars_ensure!(
-                !is_include_literal,
-                ComputeError: "literal expressions are not allowed for sorting"
-            );
-
             let opt_state = self.get_opt_state();
             let lp = self.get_plan_builder().sort(by_exprs, sort_options).build();
-            Ok(Self::from_logical_plan(lp, opt_state))
+            Self::from_logical_plan(lp, opt_state)
         }
     }
 
@@ -372,14 +344,13 @@ impl LazyFrame {
         k: IdxSize,
         by_exprs: E,
         sort_options: SortMultipleOptions,
-    ) -> PolarsResult<Self> {
+    ) -> Self {
         // this will optimize to top-k
-        Ok(self
-            .sort_by_exprs(
-                by_exprs,
-                sort_options.with_order_reversed().with_nulls_last(true),
-            )?
-            .slice(0, k))
+        self.sort_by_exprs(
+            by_exprs,
+            sort_options.with_order_reversed().with_nulls_last(true),
+        )
+        .slice(0, k)
     }
 
     pub fn bottom_k<E: AsRef<[Expr]>>(
@@ -387,11 +358,10 @@ impl LazyFrame {
         k: IdxSize,
         by_exprs: E,
         sort_options: SortMultipleOptions,
-    ) -> PolarsResult<Self> {
+    ) -> Self {
         // this will optimize to bottom-k
-        Ok(self
-            .sort_by_exprs(by_exprs, sort_options.with_nulls_last(true))?
-            .slice(0, k))
+        self.sort_by_exprs(by_exprs, sort_options.with_nulls_last(true))
+            .slice(0, k)
     }
 
     /// Reverse the `DataFrame` from top to bottom.
@@ -457,12 +427,9 @@ impl LazyFrame {
     fn _drop<I, T>(self, columns: I, strict: bool) -> Self
     where
         I: IntoIterator<Item = T>,
-        T: AsRef<str>,
+        T: Into<Selector>,
     {
-        let to_drop = columns
-            .into_iter()
-            .map(|s| s.as_ref().to_string())
-            .collect::<PlHashSet<_>>();
+        let to_drop = columns.into_iter().map(|c| c.into()).collect();
 
         let opt_state = self.get_opt_state();
         let lp = self.get_plan_builder().drop(to_drop, strict).build();
@@ -475,11 +442,10 @@ impl LazyFrame {
     ///
     /// Any given columns that are not in the schema will give a [`PolarsError::ColumnNotFound`]
     /// error while materializing the [`LazyFrame`].
-    #[inline]
     pub fn drop<I, T>(self, columns: I) -> Self
     where
         I: IntoIterator<Item = T>,
-        T: AsRef<str>,
+        T: Into<Selector>,
     {
         self._drop(columns, true)
     }
@@ -489,11 +455,10 @@ impl LazyFrame {
     /// and let the projection pushdown optimize away the unneeded columns.
     ///
     /// If a column name does not exist in the schema, it will quietly be ignored.
-    #[inline]
     pub fn drop_no_validate<I, T>(self, columns: I) -> Self
     where
         I: IntoIterator<Item = T>,
-        T: AsRef<str>,
+        T: Into<Selector>,
     {
         self._drop(columns, false)
     }
@@ -601,8 +566,7 @@ impl LazyFrame {
             self.logical_plan,
             &mut expr_arena,
             &mut lp_arena,
-            true,
-            true,
+            &mut self.opt_state,
         )?;
         let plan = IRPlan::new(node, lp_arena, expr_arena);
         Ok(plan)
@@ -617,11 +581,20 @@ impl LazyFrame {
     ) -> PolarsResult<Node> {
         #[allow(unused_mut)]
         let mut opt_state = self.opt_state;
-        let streaming = self.opt_state.streaming;
+        let streaming = self.opt_state.contains(OptFlags::STREAMING);
+        let new_streaming = self.opt_state.contains(OptFlags::NEW_STREAMING);
         #[cfg(feature = "cse")]
-        if streaming && self.opt_state.comm_subplan_elim {
-            opt_state.comm_subplan_elim = false;
+        if streaming && !new_streaming {
+            opt_state &= !OptFlags::COMM_SUBPLAN_ELIM;
         }
+
+        // The new streaming engine can't deal with the way the common
+        // subexpression elimination adds length-incorrect with_columns.
+        #[cfg(feature = "cse")]
+        if new_streaming {
+            opt_state &= !OptFlags::COMM_SUBEXPR_ELIM;
+        }
+
         let lp_top = optimize(
             self.logical_plan,
             opt_state,
@@ -652,7 +625,7 @@ impl LazyFrame {
                     scratch,
                     enable_fmt,
                     true,
-                    opt_state.row_estimate,
+                    opt_state.contains(OptFlags::ROW_ESTIMATE),
                 )?;
             }
             #[cfg(not(feature = "streaming"))]
@@ -730,20 +703,55 @@ impl LazyFrame {
     pub fn collect(self) -> PolarsResult<DataFrame> {
         #[cfg(feature = "new_streaming")]
         {
-            if self.opt_state.new_streaming {
-                let alp_plan = self.to_alp_optimized()?;
-                let lp_top = alp_plan.lp_top;
-                let mut ir_arena = alp_plan.lp_arena;
-                let expr_arena = alp_plan.expr_arena;
-
-                let lp_top = ir_arena.add(IR::Sink {
-                    input: lp_top,
+            let auto_new_streaming =
+                std::env::var("POLARS_AUTO_NEW_STREAMING").as_deref() == Ok("1");
+            if self.opt_state.contains(OptFlags::NEW_STREAMING) || auto_new_streaming {
+                // Try to run using the new streaming engine, falling back
+                // if it fails in a todo!() error if auto_new_streaming is set.
+                let mut new_stream_lazy = self.clone();
+                new_stream_lazy.opt_state |= OptFlags::NEW_STREAMING;
+                new_stream_lazy.opt_state &= !OptFlags::STREAMING;
+                let mut alp_plan = new_stream_lazy.to_alp_optimized()?;
+                let stream_lp_top = alp_plan.lp_arena.add(IR::Sink {
+                    input: alp_plan.lp_top,
                     payload: SinkType::Memory,
                 });
 
-                return polars_stream::run_query(lp_top, ir_arena, expr_arena);
+                let f = || {
+                    polars_stream::run_query(
+                        stream_lp_top,
+                        alp_plan.lp_arena,
+                        &mut alp_plan.expr_arena,
+                    )
+                };
+                match std::panic::catch_unwind(std::panic::AssertUnwindSafe(f)) {
+                    Ok(r) => return r,
+                    Err(e) => {
+                        // Fallback to normal engine if error is due to not being implemented
+                        // and auto_new_streaming is set, otherwise propagate error.
+                        if auto_new_streaming
+                            && e.downcast_ref::<&str>() == Some(&"not yet implemented")
+                        {
+                            if polars_core::config::verbose() {
+                                eprintln!("caught unimplemented error in new streaming engine, falling back to normal engine");
+                            }
+                        } else {
+                            std::panic::resume_unwind(e);
+                        }
+                    },
+                }
             }
+
+            let mut alp_plan = self.to_alp_optimized()?;
+            let mut physical_plan = create_physical_plan(
+                alp_plan.lp_top,
+                &mut alp_plan.lp_arena,
+                &alp_plan.expr_arena,
+            )?;
+            let mut state = ExecutionState::new();
+            physical_plan.execute(&mut state)
         }
+        #[cfg(not(feature = "new_streaming"))]
         self._collect_post_opt(|_, _, _| Ok(()))
     }
 
@@ -827,7 +835,7 @@ impl LazyFrame {
         cloud_options: Option<polars_io::cloud::CloudOptions>,
         ipc_options: IpcWriterOptions,
     ) -> PolarsResult<()> {
-        self.opt_state.streaming = true;
+        self.opt_state |= OptFlags::STREAMING;
         self.logical_plan = DslPlan::Sink {
             input: Arc::new(self.logical_plan),
             payload: SinkType::Cloud {
@@ -882,7 +890,7 @@ impl LazyFrame {
         feature = "json",
     ))]
     fn sink(mut self, payload: SinkType, msg_alternative: &str) -> Result<(), PolarsError> {
-        self.opt_state.streaming = true;
+        self.opt_state |= OptFlags::STREAMING;
         self.logical_plan = DslPlan::Sink {
             input: Arc::new(self.logical_plan),
             payload,
@@ -1044,7 +1052,7 @@ impl LazyFrame {
             options.index_column = name.as_ref().into();
         } else {
             let output_field = index_column
-                .to_field(&self.schema().unwrap(), Context::Default)
+                .to_field(&self.collect_schema().unwrap(), Context::Default)
                 .unwrap();
             return self.with_column(index_column).rolling(
                 Expr::Column(Arc::from(output_field.name().as_str())),
@@ -1089,7 +1097,7 @@ impl LazyFrame {
             options.index_column = name.as_ref().into();
         } else {
             let output_field = index_column
-                .to_field(&self.schema().unwrap(), Context::Default)
+                .to_field(&self.collect_schema().unwrap(), Context::Default)
                 .unwrap();
             return self.with_column(index_column).group_by_dynamic(
                 Expr::Column(Arc::from(output_field.name().as_str())),
@@ -1307,7 +1315,9 @@ impl LazyFrame {
         args: JoinArgs,
     ) -> LazyFrame {
         // if any of the nodes reads from files we must activate this this plan as well.
-        self.opt_state.file_caching |= other.opt_state.file_caching;
+        if other.opt_state.contains(OptFlags::FILE_CACHING) {
+            self.opt_state |= OptFlags::FILE_CACHING;
+        }
 
         let left_on = left_on.as_ref().to_vec();
         let right_on = right_on.as_ref().to_vec();
@@ -1509,14 +1519,26 @@ impl LazyFrame {
     }
 
     /// Apply explode operation. [See eager explode](polars_core::frame::DataFrame::explode).
-    pub fn explode<E: AsRef<[IE]>, IE: Into<Expr> + Clone>(self, columns: E) -> LazyFrame {
+    pub fn explode<E: AsRef<[IE]>, IE: Into<Selector> + Clone>(self, columns: E) -> LazyFrame {
+        self.explode_impl(columns, false)
+    }
+
+    /// Apply explode operation. [See eager explode](polars_core::frame::DataFrame::explode).
+    fn explode_impl<E: AsRef<[IE]>, IE: Into<Selector> + Clone>(
+        self,
+        columns: E,
+        allow_empty: bool,
+    ) -> LazyFrame {
         let columns = columns
             .as_ref()
             .iter()
             .map(|e| e.clone().into())
             .collect::<Vec<_>>();
         let opt_state = self.get_opt_state();
-        let lp = self.get_plan_builder().explode(columns).build();
+        let lp = self
+            .get_plan_builder()
+            .explode(columns, allow_empty)
+            .build();
         Self::from_logical_plan(lp, opt_state)
     }
 
@@ -1534,12 +1556,30 @@ impl LazyFrame {
         subset: Option<Vec<String>>,
         keep_strategy: UniqueKeepStrategy,
     ) -> LazyFrame {
+        self.unique_stable_generic(subset, keep_strategy)
+    }
+
+    pub fn unique_stable_generic<E, IE>(
+        self,
+        subset: Option<E>,
+        keep_strategy: UniqueKeepStrategy,
+    ) -> LazyFrame
+    where
+        E: AsRef<[IE]>,
+        IE: Into<Selector> + Clone,
+    {
+        let subset = subset.map(|s| {
+            s.as_ref()
+                .iter()
+                .map(|e| e.clone().into())
+                .collect::<Vec<_>>()
+        });
+
         let opt_state = self.get_opt_state();
-        let options = DistinctOptions {
-            subset: subset.map(Arc::new),
+        let options = DistinctOptionsDSL {
+            subset,
             maintain_order: true,
             keep_strategy,
-            ..Default::default()
         };
         let lp = self.get_plan_builder().distinct(options).build();
         Self::from_logical_plan(lp, opt_state)
@@ -1557,12 +1597,25 @@ impl LazyFrame {
         subset: Option<Vec<String>>,
         keep_strategy: UniqueKeepStrategy,
     ) -> LazyFrame {
+        self.unique_generic(subset, keep_strategy)
+    }
+
+    pub fn unique_generic<E: AsRef<[IE]>, IE: Into<Selector> + Clone>(
+        self,
+        subset: Option<E>,
+        keep_strategy: UniqueKeepStrategy,
+    ) -> LazyFrame {
+        let subset = subset.map(|s| {
+            s.as_ref()
+                .iter()
+                .map(|e| e.clone().into())
+                .collect::<Vec<_>>()
+        });
         let opt_state = self.get_opt_state();
-        let options = DistinctOptions {
-            subset: subset.map(Arc::new),
+        let options = DistinctOptionsDSL {
+            subset,
             maintain_order: false,
             keep_strategy,
-            ..Default::default()
         };
         let lp = self.get_plan_builder().distinct(options).build();
         Self::from_logical_plan(lp, opt_state)
@@ -1617,8 +1670,9 @@ impl LazyFrame {
 
     /// Unpivot the DataFrame from wide to long format.
     ///
-    /// See [`UnpivotArgs`] for information on how to unpivot a DataFrame.
-    pub fn unpivot(self, args: UnpivotArgs) -> LazyFrame {
+    /// See [`UnpivotArgsIR`] for information on how to unpivot a DataFrame.
+    #[cfg(feature = "pivot")]
+    pub fn unpivot(self, args: UnpivotArgsDSL) -> LazyFrame {
         let opt_state = self.get_opt_state();
         let lp = self.get_plan_builder().unpivot(args).build();
         Self::from_logical_plan(lp, opt_state)
@@ -1731,10 +1785,17 @@ impl LazyFrame {
     /// Unnest the given `Struct` columns: the fields of the `Struct` type will be
     /// inserted as columns.
     #[cfg(feature = "dtype-struct")]
-    pub fn unnest<I: IntoIterator<Item = S>, S: AsRef<str>>(self, cols: I) -> Self {
-        self.map_private(DslFunction::FunctionNode(FunctionNode::Unnest {
-            columns: cols.into_iter().map(|s| Arc::from(s.as_ref())).collect(),
-        }))
+    pub fn unnest<E, IE>(self, cols: E) -> Self
+    where
+        E: AsRef<[IE]>,
+        IE: Into<Selector> + Clone,
+    {
+        let cols = cols
+            .as_ref()
+            .iter()
+            .map(|ie| ie.clone().into())
+            .collect::<Vec<_>>();
+        self.map_private(DslFunction::Unnest(cols))
     }
 
     #[cfg(feature = "merge_sorted")]
@@ -1743,7 +1804,7 @@ impl LazyFrame {
         // this indicates until which chunk the data is from the left df
         // this trick allows us to reuse the `Union` architecture to get map over
         // two DataFrames
-        let left = self.map_private(DslFunction::FunctionNode(FunctionNode::Rechunk));
+        let left = self.map_private(DslFunction::FunctionIR(FunctionIR::Rechunk));
         let q = concat(
             &[left, other],
             UnionArgs {
@@ -1753,7 +1814,7 @@ impl LazyFrame {
             },
         )?;
         Ok(
-            q.map_private(DslFunction::FunctionNode(FunctionNode::MergeSorted {
+            q.map_private(DslFunction::FunctionIR(FunctionIR::MergeSorted {
                 column: Arc::from(key),
             })),
         )
@@ -1764,7 +1825,7 @@ impl LazyFrame {
 #[derive(Clone)]
 pub struct LazyGroupBy {
     pub logical_plan: DslPlan,
-    opt_state: OptState,
+    opt_state: OptFlags,
     keys: Vec<Expr>,
     maintain_order: bool,
     #[cfg(feature = "dynamic_group_by")]
@@ -1835,7 +1896,7 @@ impl LazyGroupBy {
             .collect::<Vec<_>>();
 
         self.agg([col("*").exclude(&keys).head(n)])
-            .explode([col("*").exclude(&keys)])
+            .explode_impl([col("*").exclude(&keys)], true)
     }
 
     /// Return last n rows of each group
@@ -1847,7 +1908,7 @@ impl LazyGroupBy {
             .collect::<Vec<_>>();
 
         self.agg([col("*").exclude(&keys).tail(n)])
-            .explode([col("*").exclude(&keys)])
+            .explode_impl([col("*").exclude(&keys)], true)
     }
 
     /// Apply a function over the groups as a new DataFrame.
@@ -1991,8 +2052,10 @@ impl JoinBuilder {
         let mut opt_state = self.lf.opt_state;
         let other = self.other.expect("with not set");
 
-        // if any of the nodes reads from files we must activate this this plan as well.
-        opt_state.file_caching |= other.opt_state.file_caching;
+        // If any of the nodes reads from files we must activate this this plan as well.
+        if other.opt_state.contains(OptFlags::FILE_CACHING) {
+            opt_state |= OptFlags::FILE_CACHING;
+        }
 
         let args = JoinArgs {
             how: self.how,
