@@ -1,3 +1,5 @@
+//! Module containing implementation of the pivot operation.
+//!
 //! Polars lazy does not implement a pivot because it is impossible to know the schema without
 //! materializing the whole dataset. This makes a pivot quite a terrible operation for performant
 //! workflows. An optimization can never be pushed down passed a pivot.
@@ -32,8 +34,8 @@ impl PhysicalAggExpr for PivotExpr {
 
 pub fn pivot<I0, I1, I2, S0, S1, S2>(
     df: &DataFrame,
-    index: I0,
-    columns: I1,
+    on: I0,
+    index: Option<I1>,
     values: Option<I2>,
     sort_columns: bool,
     agg_expr: Option<Expr>,
@@ -53,21 +55,13 @@ where
         let expr = prepare_eval_expr(agg_expr);
         PivotAgg::Expr(Arc::new(PivotExpr(expr)))
     });
-    polars_ops::pivot::pivot(
-        df,
-        index,
-        columns,
-        values,
-        sort_columns,
-        agg_expr,
-        separator,
-    )
+    polars_ops::pivot::pivot(df, on, index, values, sort_columns, agg_expr, separator)
 }
 
 pub fn pivot_stable<I0, I1, I2, S0, S1, S2>(
     df: &DataFrame,
-    index: I0,
-    columns: I1,
+    on: I0,
+    index: Option<I1>,
     values: Option<I2>,
     sort_columns: bool,
     agg_expr: Option<Expr>,
@@ -87,13 +81,5 @@ where
         let expr = prepare_eval_expr(agg_expr);
         PivotAgg::Expr(Arc::new(PivotExpr(expr)))
     });
-    polars_ops::pivot::pivot_stable(
-        df,
-        index,
-        columns,
-        values,
-        sort_columns,
-        agg_expr,
-        separator,
-    )
+    polars_ops::pivot::pivot_stable(df, on, index, values, sort_columns, agg_expr, separator)
 }
