@@ -26,6 +26,10 @@ impl PhysicalIoExpr for Wrap {
         };
         h.evaluate_io(df)
     }
+    fn live_variables(&self) -> Option<Vec<Arc<str>>> {
+        // @TODO: This should not unwrap
+        Some(expr_to_leaf_column_names(self.0.as_expression()?))
+    }
     fn as_stats_evaluator(&self) -> Option<&dyn StatsEvaluator> {
         self.0.as_stats_evaluator()
     }
@@ -246,7 +250,7 @@ fn get_pipeline_node(
     });
 
     IR::MapFunction {
-        function: FunctionNode::Pipeline {
+        function: FunctionIR::Pipeline {
             function: Arc::new(Mutex::new(move |_df: DataFrame| {
                 let state = ExecutionState::new();
                 if state.verbose() {
