@@ -3,6 +3,7 @@ use std::path::{Path, PathBuf};
 use std::sync::{Arc, Mutex, RwLock};
 
 use polars_core::prelude::*;
+use polars_io::cloud::CloudOptions;
 use polars_io::RowIndex;
 use polars_plan::plans::{DslPlan, FileScan};
 use polars_plan::prelude::{FileScanOptions, NDJsonReadOptions};
@@ -12,35 +13,39 @@ use crate::scan::file_list_reader::LazyFileListReader;
 
 #[derive(Clone)]
 pub struct LazyJsonLineReader {
-    pub(crate) paths: Arc<[PathBuf]>,
+    pub(crate) paths: Arc<Vec<PathBuf>>,
     pub(crate) batch_size: Option<NonZeroUsize>,
     pub(crate) low_memory: bool,
     pub(crate) rechunk: bool,
     pub(crate) schema: Option<SchemaRef>,
+    pub(crate) schema_overwrite: Option<SchemaRef>,
     pub(crate) row_index: Option<RowIndex>,
     pub(crate) infer_schema_length: Option<NonZeroUsize>,
     pub(crate) n_rows: Option<usize>,
     pub(crate) ignore_errors: bool,
     pub(crate) include_file_paths: Option<Arc<str>>,
+    pub(crate) cloud_options: Option<CloudOptions>,
 }
 
 impl LazyJsonLineReader {
-    pub fn new_paths(paths: Arc<[PathBuf]>) -> Self {
+    pub fn new_paths(paths: Arc<Vec<PathBuf>>) -> Self {
         Self::new(PathBuf::new()).with_paths(paths)
     }
 
     pub fn new(path: impl AsRef<Path>) -> Self {
         LazyJsonLineReader {
-            paths: Arc::new([path.as_ref().to_path_buf()]),
+            paths: Arc::new(vec![path.as_ref().to_path_buf()]),
             batch_size: None,
             low_memory: false,
             rechunk: false,
             schema: None,
+            schema_overwrite: None,
             row_index: None,
             infer_schema_length: NonZeroUsize::new(100),
             ignore_errors: false,
             n_rows: None,
             include_file_paths: None,
+            cloud_options: None,
         }
     }
     /// Add a row index column.
@@ -79,6 +84,13 @@ impl LazyJsonLineReader {
         self
     }
 
+    /// Set the JSON file's schema
+    #[must_use]
+    pub fn with_schema_overwrite(mut self, schema_overwrite: Option<SchemaRef>) -> Self {
+        self.schema_overwrite = schema_overwrite;
+        self
+    }
+
     /// Reduce memory usage at the expense of performance
     #[must_use]
     pub fn low_memory(mut self, toggle: bool) -> Self {
@@ -92,6 +104,11 @@ impl LazyJsonLineReader {
         self
     }
 
+    pub fn with_cloud_options(mut self, cloud_options: Option<CloudOptions>) -> Self {
+        self.cloud_options = cloud_options;
+        self
+    }
+
     pub fn with_include_file_paths(mut self, include_file_paths: Option<Arc<str>>) -> Self {
         self.include_file_paths = include_file_paths;
         self
@@ -100,14 +117,10 @@ impl LazyJsonLineReader {
 
 impl LazyFileListReader for LazyJsonLineReader {
     fn finish(self) -> PolarsResult<LazyFrame> {
-        if !self.glob() {
-            return self.finish_no_glob();
-        }
-
         let paths = Arc::new(Mutex::new((self.paths, false)));
 
         let file_options = FileScanOptions {
-            n_rows: self.n_rows,
+            slice: self.n_rows.map(|x| (0, x)),
             with_columns: None,
             cache: false,
             row_index: self.row_index,
@@ -125,9 +138,13 @@ impl LazyFileListReader for LazyJsonLineReader {
             low_memory: self.low_memory,
             ignore_errors: self.ignore_errors,
             schema: self.schema,
+            schema_overwrite: self.schema_overwrite,
         };
 
-        let scan_type = FileScan::NDJson { options };
+        let scan_type = FileScan::NDJson {
+            options,
+            cloud_options: self.cloud_options,
+        };
 
         Ok(LazyFrame::from(DslPlan::Scan {
             paths,
@@ -140,46 +157,14 @@ impl LazyFileListReader for LazyJsonLineReader {
     }
 
     fn finish_no_glob(self) -> PolarsResult<LazyFrame> {
-        let paths = Arc::new(Mutex::new((self.paths, false)));
-
-        let file_options = FileScanOptions {
-            n_rows: self.n_rows,
-            with_columns: None,
-            cache: false,
-            row_index: self.row_index.clone(),
-            rechunk: self.rechunk,
-            file_counter: 0,
-            hive_options: Default::default(),
-            glob: false,
-            include_file_paths: None,
-        };
-
-        let options = NDJsonReadOptions {
-            n_threads: None,
-            infer_schema_length: self.infer_schema_length,
-            chunk_size: NonZeroUsize::new(1 << 18).unwrap(),
-            low_memory: self.low_memory,
-            ignore_errors: self.ignore_errors,
-            schema: self.schema,
-        };
-
-        let scan_type = FileScan::NDJson { options };
-
-        Ok(LazyFrame::from(DslPlan::Scan {
-            paths,
-            file_info: Arc::new(RwLock::new(None)),
-            hive_parts: None,
-            predicate: None,
-            file_options,
-            scan_type,
-        }))
+        unreachable!();
     }
 
     fn paths(&self) -> &[PathBuf] {
         &self.paths
     }
 
-    fn with_paths(mut self, paths: Arc<[PathBuf]>) -> Self {
+    fn with_paths(mut self, paths: Arc<Vec<PathBuf>>) -> Self {
         self.paths = paths;
         self
     }
@@ -215,4 +200,9 @@ impl LazyFileListReader for LazyJsonLineReader {
     fn row_index(&self) -> Option<&RowIndex> {
         self.row_index.as_ref()
     }
+
+    /// [CloudOptions] used to list files.
+    fn cloud_options(&self) -> Option<&CloudOptions> {
+        self.cloud_options.as_ref()
+    }
 }
