@@ -399,7 +399,7 @@ fn test_scan_parquet_limit_9001() {
             let sliced = options.slice.unwrap();
             sliced.1 == 3
         },
-        IR::Scan { file_options, .. } => file_options.n_rows == Some(3),
+        IR::Scan { file_options, .. } => file_options.slice == Some((0, 3)),
         _ => true,
     });
 }
@@ -419,6 +419,8 @@ fn test_ipc_globbing() -> PolarsResult<()> {
             row_index: None,
             memory_map: true,
             cloud_options: None,
+            hive_options: Default::default(),
+            include_file_paths: None,
         },
     )?
     .collect()?;
@@ -648,17 +650,41 @@ fn scan_predicate_on_set_null_values() -> PolarsResult<()> {
 }
 
 #[test]
-fn scan_anonymous_fn() -> PolarsResult<()> {
-    let function = Arc::new(|_scan_opts: AnonymousScanArgs| Ok(fruits_cars()));
+fn scan_anonymous_fn_with_options() -> PolarsResult<()> {
+    struct MyScan {}
+
+    impl AnonymousScan for MyScan {
+        fn as_any(&self) -> &dyn std::any::Any {
+            self
+        }
+
+        fn allows_projection_pushdown(&self) -> bool {
+            true
+        }
+
+        fn scan(&self, scan_opts: AnonymousScanArgs) -> PolarsResult<DataFrame> {
+            assert_eq!(scan_opts.with_columns.clone().unwrap().len(), 2);
+            assert_eq!(scan_opts.n_rows, Some(3));
+            let out = fruits_cars().select(scan_opts.with_columns.unwrap().as_ref())?;
+            Ok(out.slice(0, scan_opts.n_rows.unwrap()))
+        }
+    }
+
+    let function = Arc::new(MyScan {});
 
     let args = ScanArgsAnonymous {
         schema: Some(Arc::new(fruits_cars().schema())),
         ..ScanArgsAnonymous::default()
     };
 
-    let df = LazyFrame::anonymous_scan(function, args)?.collect()?;
+    let q = LazyFrame::anonymous_scan(function, args)?
+        .with_column((col("A") * lit(2)).alias("A2"))
+        .select([col("A2"), col("fruits")])
+        .limit(3);
 
-    assert_eq!(df.shape(), (5, 4));
+    let df = q.collect()?;
+
+    assert_eq!(df.shape(), (3, 2));
     Ok(())
 }
 
