@@ -19,7 +19,7 @@ fn test_multiple_roots() -> PolarsResult<()> {
     // and that we don't have any filter node
     assert!(!(&lp_arena)
         .iter(root)
-        .any(|(_, lp)| matches!(lp, ALogicalPlan::Selection { .. })));
+        .any(|(_, lp)| matches!(lp, IR::Filter { .. })));
 
     Ok(())
 }
@@ -102,10 +102,7 @@ fn filter_added_column_issue_2470() -> PolarsResult<()> {
 fn filter_blocked_by_map() -> PolarsResult<()> {
     let df = fruits_cars();
 
-    let allowed = AllowedOptimizations {
-        predicate_pushdown: false,
-        ..Default::default()
-    };
+    let allowed = OptFlags::default() & !OptFlags::PREDICATE_PUSHDOWN;
     let q = df
         .lazy()
         .map(Ok, allowed, None, None)
