@@ -46,10 +46,11 @@ fn test_lazy_alias() {
 }
 
 #[test]
+#[cfg(feature = "pivot")]
 fn test_lazy_unpivot() {
     let df = get_df();
 
-    let args = UnpivotArgs {
+    let args = UnpivotArgsDSL {
         on: vec!["sepal_length".into(), "sepal_width".into()],
         index: vec!["petal_width".into(), "petal_length".into()],
         ..Default::default()
@@ -557,7 +558,15 @@ fn test_simplify_expr() {
 
     let mut expr_arena = Arena::new();
     let mut lp_arena = Arena::new();
-    let lp_top = to_alp(plan, &mut expr_arena, &mut lp_arena, true, false).unwrap();
+
+    #[allow(const_item_mutation)]
+    let lp_top = to_alp(
+        plan,
+        &mut expr_arena,
+        &mut lp_arena,
+        &mut OptFlags::SIMPLIFY_EXPR,
+    )
+    .unwrap();
     let plan = node_to_lp(lp_top, &expr_arena, &mut lp_arena);
     assert!(
         matches!(plan, DslPlan::Select{ expr, ..} if matches!(&expr[0], Expr::BinaryExpr{left, ..} if **left == Expr::Literal(LiteralValue::Float(2.0))))
@@ -636,7 +645,7 @@ fn test_type_coercion() {
 
     let mut expr_arena = Arena::new();
     let mut lp_arena = Arena::new();
-    let lp_top = to_alp(lp, &mut expr_arena, &mut lp_arena, true, true).unwrap();
+    let lp_top = to_alp(lp, &mut expr_arena, &mut lp_arena, &mut OptFlags::default()).unwrap();
     let lp = node_to_lp(lp_top, &expr_arena, &mut lp_arena);
 
     if let DslPlan::Select { expr, .. } = lp {
@@ -1439,7 +1448,7 @@ fn test_when_then_schema() -> PolarsResult<()> {
         .select([when(col("A").gt(lit(1)))
             .then(Null {}.lit())
             .otherwise(col("A"))])
-        .schema();
+        .collect_schema();
     assert_ne!(schema?.get_at_index(0).unwrap().1, &DataType::Null);
 
     Ok(())
@@ -1911,7 +1920,7 @@ fn test_sort_maintain_order_true() -> PolarsResult<()> {
             SortMultipleOptions::default()
                 .with_maintain_order(true)
                 .with_nulls_last(true),
-        )?
+        )
         .slice(0, 3)
         .collect()?;
     println!("{:?}", res);
