@@ -2,7 +2,7 @@ use super::*;
 
 pub(crate) struct UniqueExec {
     pub(crate) input: Box<dyn Executor>,
-    pub(crate) options: DistinctOptions,
+    pub(crate) options: DistinctOptionsIR,
 }
 
 impl Executor for UniqueExec {
@@ -15,7 +15,11 @@ impl Executor for UniqueExec {
             }
         }
         let df = self.input.execute(state)?;
-        let subset = self.options.subset.as_ref().map(|v| &***v);
+        let subset = self
+            .options
+            .subset
+            .as_ref()
+            .map(|v| v.iter().map(|n| n.to_string()).collect::<Vec<_>>());
         let keep = self.options.keep_strategy;
 
         state.record(
@@ -25,8 +29,8 @@ impl Executor for UniqueExec {
                 }
 
                 match self.options.maintain_order {
-                    true => df.unique_stable(subset, keep, self.options.slice),
-                    false => df.unique(subset, keep, self.options.slice),
+                    true => df.unique_stable(subset.as_deref(), keep, self.options.slice),
+                    false => df.unique(subset.as_deref(), keep, self.options.slice),
                 }
             },
             Cow::Borrowed("unique()"),
