@@ -158,7 +158,52 @@ fn create_physical_plan_impl(
     let logical_plan = lp_arena.take(root);
     match logical_plan {
         #[cfg(feature = "python")]
-        PythonScan { options, .. } => Ok(Box::new(executors::PythonScanExec { options })),
+        PythonScan { mut options } => {
+            let mut predicate_serialized = None;
+
+            let predicate = if let PythonPredicate::Polars(e) = &options.predicate {
+                let phys_expr = || {
+                    let mut state = ExpressionConversionState::new(true, state.expr_depth);
+                    create_physical_expr(
+                        e,
+                        Context::Default,
+                        expr_arena,
+                        Some(&options.schema),
+                        &mut state,
+                    )
+                };
+
+                // Convert to a pyarrow eval string.
+                if matches!(options.python_source, PythonScanSource::Pyarrow) {
+                    if let Some(eval_str) = polars_plan::plans::python::pyarrow::predicate_to_pa(
+                        e.node(),
+                        expr_arena,
+                        Default::default(),
+                    ) {
+                        options.predicate = PythonPredicate::PyArrow(eval_str);
+                        // We don't have to use a physical expression as pyarrow deals with the filter.
+                        None
+                    } else {
+                        Some(phys_expr()?)
+                    }
+                }
+                // Convert to physical expression for the case the reader cannot consume the predicate.
+                else {
+                    let dsl_expr = e.to_expr(expr_arena);
+                    predicate_serialized =
+                        polars_plan::plans::python::predicate::serialize(&dsl_expr)?;
+
+                    Some(phys_expr()?)
+                }
+            } else {
+                None
+            };
+            Ok(Box::new(executors::PythonScanExec {
+                options,
+                predicate,
+                predicate_serialized,
+            }))
+        },
         Sink { payload, .. } => match payload {
             SinkType::Memory => {
                 polars_bail!(InvalidOperation: "memory sink not supported in the standard engine")
@@ -239,7 +284,12 @@ fn create_physical_plan_impl(
             predicate,
             mut file_options,
         } => {
-            file_options.n_rows = _set_n_rows_for_scan(file_options.n_rows);
+            file_options.slice = if let Some((offset, len)) = file_options.slice {
+                Some((offset, _set_n_rows_for_scan(Some(len)).unwrap()))
+            } else {
+                _set_n_rows_for_scan(None).map(|x| (0, x))
+            };
+
             let mut state = ExpressionConversionState::new(true, state.expr_depth);
             let predicate = predicate
                 .map(|pred| {
@@ -292,7 +342,7 @@ fn create_physical_plan_impl(
                     metadata,
                 ))),
                 #[cfg(feature = "json")]
-                FileScan::NDJson { options } => Ok(Box::new(executors::JsonExec::new(
+                FileScan::NDJson { options, .. } => Ok(Box::new(executors::JsonExec::new(
                     paths,
                     options,
                     file_options,
@@ -325,7 +375,8 @@ fn create_physical_plan_impl(
                 state.expr_depth,
             );
 
-            let streamable = all_streamable(&expr, expr_arena, Context::Default);
+            let streamable =
+                options.should_broadcast && all_streamable(&expr, expr_arena, Context::Default);
             let phys_expr = create_physical_expressions_from_irs(
                 &expr,
                 Context::Default,
@@ -579,7 +630,8 @@ fn create_physical_plan_impl(
             let input_schema = lp_arena.get(input).schema(lp_arena).into_owned();
             let input = create_physical_plan_impl(input, lp_arena, expr_arena, state)?;
 
-            let streamable = all_streamable(&exprs, expr_arena, Context::Default);
+            let streamable =
+                options.should_broadcast && all_streamable(&exprs, expr_arena, Context::Default);
 
             let mut state = ExpressionConversionState::new(
                 POOL.current_num_threads() > exprs.len(),
