@@ -80,7 +80,7 @@ pub trait ToStruct: AsList {
                 .collect::<PolarsResult<Vec<_>>>()
         })?;
 
-        StructChunked::new(ca.name(), &fields)
+        StructChunked::from_series(ca.name(), &fields)
     }
 }
 
