@@ -1,9 +1,10 @@
 use arrow::array::{Utf8Array, ValueSize};
 use arrow::compute::cast::utf8_to_utf8view;
+use polars_core::prelude::arity::unary_elementwise;
 use polars_core::prelude::*;
 
 // Vertically concatenate all strings in a StringChunked.
-pub fn str_concat(ca: &StringChunked, delimiter: &str, ignore_nulls: bool) -> StringChunked {
+pub fn str_join(ca: &StringChunked, delimiter: &str, ignore_nulls: bool) -> StringChunked {
     if ca.is_empty() {
         return StringChunked::new(ca.name(), &[""]);
     }
@@ -67,7 +68,7 @@ pub fn hor_str_concat(
         return if !ignore_nulls || ca.null_count() == 0 {
             Ok(ca.clone())
         } else {
-            Ok(ca.apply_generic(|val| Some(val.unwrap_or(""))))
+            Ok(unary_elementwise(ca, |val| Some(val.unwrap_or(""))))
         };
     }
 
@@ -88,12 +89,10 @@ pub fn hor_str_concat(
     // Broadcast if appropriate.
     let mut cols: Vec<_> = cas
         .iter()
-        .map(|ca| {
-            if ca.len() > 1 {
-                ColumnIter::Iter(ca.iter())
-            } else {
-                ColumnIter::Broadcast(ca.get(0))
-            }
+        .map(|ca| match ca.len() {
+            0 => ColumnIter::Broadcast(None),
+            1 => ColumnIter::Broadcast(ca.get(0)),
+            _ => ColumnIter::Iter(ca.iter()),
         })
         .collect();
 
@@ -144,7 +143,7 @@ mod test {
     fn test_str_concat() {
         let ca = Int32Chunked::new("foo", &[Some(1), None, Some(3)]);
         let ca_str = ca.cast(&DataType::String).unwrap();
-        let out = str_concat(ca_str.str().unwrap(), "-", true);
+        let out = str_join(ca_str.str().unwrap(), "-", true);
 
         let out = out.get(0);
         assert_eq!(out, Some("1-3"));
