@@ -63,25 +63,47 @@ pub trait StringNameSpaceImpl: AsString {
 
     #[cfg(feature = "string_to_integer")]
     // Parse a string number with base _radix_ into a decimal (i64)
-    fn to_integer(&self, base: u32, strict: bool) -> PolarsResult<Int64Chunked> {
+    fn to_integer(&self, base: &UInt32Chunked, strict: bool) -> PolarsResult<Int64Chunked> {
         let ca = self.as_string();
-        let f = |opt_s: Option<&str>| -> Option<i64> {
-            opt_s.and_then(|s| <i64 as Num>::from_str_radix(s, base).ok())
+        let f = |opt_s: Option<&str>, opt_base: Option<u32>| -> Option<i64> {
+            match (opt_s, opt_base) {
+                (Some(s), Some(base)) => <i64 as Num>::from_str_radix(s, base).ok(),
+                _ => None,
+            }
         };
-        let out: Int64Chunked = ca.apply_generic(f);
-
+        let out = broadcast_binary_elementwise(ca, base, f);
         if strict && ca.null_count() != out.null_count() {
-            let failure_mask = !ca.is_null() & out.is_null();
+            let failure_mask = ca.is_not_null() & out.is_null() & base.is_not_null();
             let all_failures = ca.filter(&failure_mask)?;
+            if all_failures.is_empty() {
+                return Ok(out);
+            }
             let n_failures = all_failures.len();
             let some_failures = all_failures.unique()?.slice(0, 10).sort(false);
-            let some_error_msg = some_failures
-                .get(0)
-                .and_then(|s| <i64 as Num>::from_str_radix(s, base).err())
-                .map_or_else(
-                    || unreachable!("failed to extract ParseIntError"),
-                    |e| format!("{}", e),
-                );
+            let some_error_msg = match base.len() {
+                1 => {
+                    // we can ensure that base is not null.
+                    let base = base.get(0).unwrap();
+                    some_failures
+                        .get(0)
+                        .and_then(|s| <i64 as Num>::from_str_radix(s, base).err())
+                        .map_or_else(
+                            || unreachable!("failed to extract ParseIntError"),
+                            |e| format!("{}", e),
+                        )
+                },
+                _ => {
+                    let base_filures = base.filter(&failure_mask)?;
+                    some_failures
+                        .get(0)
+                        .zip(base_filures.get(0))
+                        .and_then(|(s, base)| <i64 as Num>::from_str_radix(s, base).err())
+                        .map_or_else(
+                            || unreachable!("failed to extract ParseIntError"),
+                            |e| format!("{}", e),
+                        )
+                },
+            };
             polars_bail!(
                 ComputeError:
                 "strict integer parsing failed for {} value(s): {}; error message for the \
@@ -243,7 +265,7 @@ pub trait StringNameSpaceImpl: AsString {
         let res_reg = Regex::new(pat);
         let opt_reg = if strict { Some(res_reg?) } else { res_reg.ok() };
         let out: BooleanChunked = if let Some(reg) = opt_reg {
-            ca.apply_values_generic(|s| reg.is_match(s))
+            unary_elementwise_values(ca, |s| reg.is_match(s))
         } else {
             BooleanChunked::full_null(ca.name(), ca.len())
         };
@@ -267,11 +289,9 @@ pub trait StringNameSpaceImpl: AsString {
     fn find(&self, pat: &str, strict: bool) -> PolarsResult<UInt32Chunked> {
         let ca = self.as_string();
         match Regex::new(pat) {
-            Ok(rx) => {
-                Ok(ca.apply_generic(|opt_s| {
-                    opt_s.and_then(|s| rx.find(s)).map(|m| m.start() as u32)
-                }))
-            },
+            Ok(rx) => Ok(unary_elementwise(ca, |opt_s| {
+                opt_s.and_then(|s| rx.find(s)).map(|m| m.start() as u32)
+            })),
             Err(_) if !strict => Ok(UInt32Chunked::full_null(ca.name(), ca.len())),
             Err(e) => Err(PolarsError::ComputeError(
                 format!("Invalid regular expression: {}", e).into(),
@@ -397,7 +417,7 @@ pub trait StringNameSpaceImpl: AsString {
     fn strip_chars(&self, pat: &Series) -> PolarsResult<StringChunked> {
         let ca = self.as_string();
         if pat.dtype() == &DataType::Null {
-            Ok(ca.apply_generic(|opt_s| opt_s.map(|s| s.trim())))
+            Ok(unary_elementwise(ca, |opt_s| opt_s.map(|s| s.trim())))
         } else {
             Ok(strip_chars(ca, pat.str()?))
         }
@@ -406,7 +426,7 @@ pub trait StringNameSpaceImpl: AsString {
     fn strip_chars_start(&self, pat: &Series) -> PolarsResult<StringChunked> {
         let ca = self.as_string();
         if pat.dtype() == &DataType::Null {
-            return Ok(ca.apply_generic(|opt_s| opt_s.map(|s| s.trim_start())));
+            return Ok(unary_elementwise(ca, |opt_s| opt_s.map(|s| s.trim_start())));
         } else {
             Ok(strip_chars_start(ca, pat.str()?))
         }
@@ -415,7 +435,7 @@ pub trait StringNameSpaceImpl: AsString {
     fn strip_chars_end(&self, pat: &Series) -> PolarsResult<StringChunked> {
         let ca = self.as_string();
         if pat.dtype() == &DataType::Null {
-            return Ok(ca.apply_generic(|opt_s| opt_s.map(|s| s.trim_end())));
+            return Ok(unary_elementwise(ca, |opt_s| opt_s.map(|s| s.trim_end())));
         } else {
             Ok(strip_chars_end(ca, pat.str()?))
         }
@@ -435,21 +455,21 @@ pub trait StringNameSpaceImpl: AsString {
     fn split_exact(&self, by: &StringChunked, n: usize) -> PolarsResult<StructChunked> {
         let ca = self.as_string();
 
-        split_to_struct(ca, by, n + 1, |s, by| s.split(by))
+        split_to_struct(ca, by, n + 1, str::split, false)
     }
 
     #[cfg(feature = "dtype-struct")]
     fn split_exact_inclusive(&self, by: &StringChunked, n: usize) -> PolarsResult<StructChunked> {
         let ca = self.as_string();
 
-        split_to_struct(ca, by, n + 1, |s, by| s.split_inclusive(by))
+        split_to_struct(ca, by, n + 1, str::split_inclusive, false)
     }
 
     #[cfg(feature = "dtype-struct")]
     fn splitn(&self, by: &StringChunked, n: usize) -> PolarsResult<StructChunked> {
         let ca = self.as_string();
 
-        split_to_struct(ca, by, n, |s, by| s.splitn(n, by))
+        split_to_struct(ca, by, n, |s, by| s.splitn(n, by), true)
     }
 
     fn split(&self, by: &StringChunked) -> ListChunked {
@@ -502,7 +522,9 @@ pub trait StringNameSpaceImpl: AsString {
             Regex::new(pat)?
         };
 
-        Ok(ca.apply_generic(|opt_s| opt_s.map(|s| reg.find_iter(s).count() as u32)))
+        Ok(unary_elementwise(ca, |opt_s| {
+            opt_s.map(|s| reg.find_iter(s).count() as u32)
+        }))
     }
 
     /// Count all successive non-overlapping regex matches.
@@ -590,6 +612,29 @@ pub trait StringNameSpaceImpl: AsString {
 
         Ok(substring::substring(ca, offset.i64()?, length.u64()?))
     }
+
+    /// Slice the first `n` values of the string.
+    ///
+    /// Determines a substring starting at the beginning of the string up to offset `n` of each
+    /// element in `array`. `n` can be negative, in which case the slice ends `n` characters from
+    /// the end of the string.
+    fn str_head(&self, n: &Series) -> PolarsResult<StringChunked> {
+        let ca = self.as_string();
+        let n = n.strict_cast(&DataType::Int64)?;
+
+        substring::head(ca, n.i64()?)
+    }
+
+    /// Slice the last `n` values of the string.
+    ///
+    /// Determines a substring starting at offset `n` of each element in `array`. `n` can be
+    /// negative, in which case the slice begins `n` characters from the start of the string.
+    fn str_tail(&self, n: &Series) -> PolarsResult<StringChunked> {
+        let ca = self.as_string();
+        let n = n.strict_cast(&DataType::Int64)?;
+
+        substring::tail(ca, n.i64()?)
+    }
 }
 
 impl StringNameSpaceImpl for StringChunked {}
