@@ -22,13 +22,15 @@ pub fn floor_div_series(a: &Series, b: &Series) -> PolarsResult<Series> {
     match (a.dtype(), b.dtype()) {
         #[cfg(feature = "dtype-struct")]
         (DataType::Struct(_), DataType::Struct(_)) => {
-            return Ok(_struct_arithmetic(a, b, |a, b| {
-                floor_div_series(a, b).unwrap()
-            }))
+            return _struct_arithmetic(a, b, floor_div_series);
         },
         _ => {},
     }
 
+    if !a.dtype().is_numeric() {
+        polars_bail!(op = "floor_div", a.dtype());
+    }
+
     let logical_type = a.dtype();
 
     let a = a.to_physical_repr();
