@@ -8,26 +8,56 @@ use crate::parquet::statistics::{BinaryStatistics, ParquetStatistics};
 use crate::read::schema::is_nullable;
 use crate::write::binary::encode_non_null_values;
 use crate::write::utils::invalid_encoding;
-use crate::write::{utils, Encoding, Page, StatisticsOptions, WriteOptions};
+use crate::write::{utils, EncodeNullability, Encoding, Page, StatisticsOptions, WriteOptions};
 
-pub(crate) fn encode_plain(array: &BinaryViewArray, buffer: &mut Vec<u8>) {
-    let capacity =
-        array.total_bytes_len() + (array.len() - array.null_count()) * std::mem::size_of::<u32>();
+pub(crate) fn encode_plain(
+    array: &BinaryViewArray,
+    options: EncodeNullability,
+    buffer: &mut Vec<u8>,
+) {
+    if options.is_optional() && array.validity().is_some() {
+        let capacity = array.total_bytes_len()
+            + (array.len() - array.null_count()) * std::mem::size_of::<u32>();
 
-    let len_before = buffer.len();
-    buffer.reserve(capacity);
+        let len_before = buffer.len();
+        buffer.reserve(capacity);
 
-    encode_non_null_values(array.non_null_values_iter(), buffer);
-    // Append the non-null values.
-    debug_assert_eq!(buffer.len() - len_before, capacity);
+        encode_non_null_values(array.non_null_values_iter(), buffer);
+        // Append the non-null values.
+        debug_assert_eq!(buffer.len() - len_before, capacity);
+    } else {
+        let capacity = array.total_bytes_len() + array.len() * std::mem::size_of::<u32>();
+
+        let len_before = buffer.len();
+        buffer.reserve(capacity);
+
+        encode_non_null_values(array.values_iter(), buffer);
+        // Append the non-null values.
+        debug_assert_eq!(buffer.len() - len_before, capacity);
+    }
 }
 
-pub(crate) fn encode_delta(array: &BinaryViewArray, buffer: &mut Vec<u8>) {
-    let lengths = array.non_null_views_iter().map(|v| v.length as i64);
-    delta_bitpacked::encode(lengths, buffer);
+pub(crate) fn encode_delta(
+    array: &BinaryViewArray,
+    options: EncodeNullability,
+    buffer: &mut Vec<u8>,
+) {
+    if options.is_optional() && array.validity().is_some() {
+        let lengths = utils::ExactSizedIter::new(
+            array.non_null_views_iter().map(|v| v.length as i64),
+            array.len() - array.null_count(),
+        );
+        delta_bitpacked::encode(lengths, buffer, 1);
 
-    for slice in array.non_null_values_iter() {
-        buffer.extend_from_slice(slice)
+        for slice in array.non_null_values_iter() {
+            buffer.extend_from_slice(slice)
+        }
+    } else {
+        let lengths =
+            utils::ExactSizedIter::new(array.views().iter().map(|v| v.length as i64), array.len());
+        delta_bitpacked::encode(lengths, buffer, 1);
+
+        buffer.extend(array.values_iter().flatten());
     }
 }
 
@@ -38,6 +68,7 @@ pub fn array_to_page(
     encoding: Encoding,
 ) -> PolarsResult<Page> {
     let is_optional = is_nullable(&type_.field_info);
+    let encode_options = EncodeNullability::new(is_optional);
 
     let mut buffer = vec![];
     // TODO! reserve capacity
@@ -52,8 +83,8 @@ pub fn array_to_page(
     let definition_levels_byte_length = buffer.len();
 
     match encoding {
-        Encoding::Plain => encode_plain(array, &mut buffer),
-        Encoding::DeltaLengthByteArray => encode_delta(array, &mut buffer),
+        Encoding::Plain => encode_plain(array, encode_options, &mut buffer),
+        Encoding::DeltaLengthByteArray => encode_delta(array, encode_options, &mut buffer),
         _ => return Err(invalid_encoding(encoding, array.data_type())),
     }
 
