@@ -4,7 +4,10 @@ use crate::parquet::encoding::delta_bitpacked;
 /// # Implementation
 /// This encoding is equivalent to call [`delta_bitpacked::encode`] on the lengths of the items
 /// of the iterator followed by extending the buffer from each item of the iterator.
-pub fn encode<A: AsRef<[u8]>, I: Iterator<Item = A> + Clone>(iterator: I, buffer: &mut Vec<u8>) {
+pub fn encode<A: AsRef<[u8]>, I: ExactSizeIterator<Item = A> + Clone>(
+    iterator: I,
+    buffer: &mut Vec<u8>,
+) {
     let mut total_length = 0;
     delta_bitpacked::encode(
         iterator.clone().map(|x| {
@@ -13,6 +16,7 @@ pub fn encode<A: AsRef<[u8]>, I: Iterator<Item = A> + Clone>(iterator: I, buffer
             len as i64
         }),
         buffer,
+        1,
     );
     buffer.reserve(total_length);
     iterator.for_each(|x| buffer.extend(x.as_ref()))
