@@ -4,7 +4,6 @@ use std::sync::Mutex;
 
 use hashbrown::hash_map::RawEntryMut;
 use num_traits::NumCast;
-use polars_core::export::ahash::RandomState;
 use polars_core::frame::row::AnyValueBuffer;
 use polars_core::prelude::*;
 use polars_core::utils::_set_partition_size;
@@ -52,7 +51,7 @@ impl Hash for Key {
 // those indexes point into the keys buffer and the values buffer
 // the keys buffer are buffers of AnyValue per partition
 // and the values are buffer of Aggregation functions per partition
-pub struct Utf8GroupbySink {
+pub struct StringGroupbySink {
     thread_no: usize,
     // idx is the offset in the array with keys
     // idx is the offset in the array with aggregators
@@ -68,7 +67,7 @@ pub struct Utf8GroupbySink {
     key_column: Arc<dyn PhysicalPipedExpr>,
     // the columns that will be aggregated
     aggregation_columns: Arc<Vec<Arc<dyn PhysicalPipedExpr>>>,
-    hb: RandomState,
+    hb: PlRandomState,
     // Initializing Aggregation functions. If we aggregate by 2 columns
     // this vec will have two functions. We will use these functions
     // to populate the buffer where the hashmap points to
@@ -83,7 +82,7 @@ pub struct Utf8GroupbySink {
     ooc_state: OocState,
 }
 
-impl Utf8GroupbySink {
+impl StringGroupbySink {
     pub(crate) fn new(
         key_column: Arc<dyn PhysicalPipedExpr>,
         aggregation_columns: Arc<Vec<Arc<dyn PhysicalPipedExpr>>>,
@@ -166,7 +165,7 @@ impl Utf8GroupbySink {
                     .zip(slices.par_iter())
                     .filter_map(|(agg_map, slice)| {
                         let ptr = aggregators as *mut AggregateFunction;
-                        // safety:
+                        // SAFETY:
                         // we will not alias.
                         let aggregators =
                             unsafe { std::slice::from_raw_parts_mut(ptr, aggregators_len) };
@@ -187,7 +186,7 @@ impl Utf8GroupbySink {
                             .collect::<Vec<_>>();
 
                         let cap = std::cmp::min(slice_len, agg_map.len());
-                        let mut key_builder = Utf8ChunkedBuilder::new("", cap, cap * 8);
+                        let mut key_builder = StringChunkedBuilder::new("", cap);
                         agg_map.into_iter().skip(offset).take(slice_len).for_each(
                             |(k, &offset)| {
                                 let key_offset = k.idx as usize;
@@ -213,7 +212,7 @@ impl Utf8GroupbySink {
                         cols.push(key_builder.finish().into_series());
                         cols.extend(buffers.into_iter().map(|buf| buf.into_series()));
                         physical_agg_to_logical(&mut cols, &self.output_schema);
-                        Some(DataFrame::new_no_checks(cols))
+                        Some(unsafe { DataFrame::new_no_checks(cols) })
                     })
                     .collect::<Vec<_>>();
 
@@ -225,15 +224,13 @@ impl Utf8GroupbySink {
         context: &PExecutionContext,
         chunk: &DataChunk,
     ) -> PolarsResult<Series> {
-        let s = self
-            .key_column
-            .evaluate(chunk, context.execution_state.as_any())?;
+        let s = self.key_column.evaluate(chunk, &context.execution_state)?;
         let s = s.to_physical_repr();
         let s = prepare_key(&s, chunk);
 
         // todo! ammortize allocation
         for phys_e in self.aggregation_columns.iter() {
-            let s = phys_e.evaluate(chunk, context.execution_state.as_any())?;
+            let s = phys_e.evaluate(chunk, &context.execution_state)?;
             let s = s.to_physical_repr();
             self.aggregation_series.push(s.rechunk());
         }
@@ -271,7 +268,7 @@ impl Utf8GroupbySink {
         // already read/taken. So we write on the slots we just read
         let agg_idx_ptr = hashes.as_ptr() as *mut u64 as *mut IdxSize;
         // array of the keys
-        let keys_arr = s.utf8().unwrap().downcast_iter().next().unwrap().clone();
+        let keys_arr = s.str().unwrap().downcast_iter().next().unwrap().clone();
 
         // set all bits to false
         self.ooc_state.reset_ooc_filter_rows(chunk.data.height());
@@ -284,7 +281,7 @@ impl Utf8GroupbySink {
             match entry {
                 RawEntryMut::Vacant(_) => {
                     // set this row to true: e.g. processed ooc
-                    // safety: we correctly set the length with `reset_ooc_filter_rows`
+                    // SAFETY: we correctly set the length with `reset_ooc_filter_rows`
                     unsafe {
                         self.ooc_state.set_row_as_ooc(iteration_idx);
                     }
@@ -324,7 +321,7 @@ impl Utf8GroupbySink {
     }
 }
 
-impl Sink for Utf8GroupbySink {
+impl Sink for StringGroupbySink {
     fn sink(&mut self, context: &PExecutionContext, chunk: DataChunk) -> PolarsResult<SinkResult> {
         if chunk.is_empty() {
             return Ok(SinkResult::CanHaveMoreInput);
@@ -349,7 +346,7 @@ impl Sink for Utf8GroupbySink {
         // already read/taken. So we write on the slots we just read
         let agg_idx_ptr = hashes.as_ptr() as *mut u64 as *mut IdxSize;
         // array of the keys
-        let keys_arr = s.utf8().unwrap().downcast_iter().next().unwrap().clone();
+        let keys_arr = s.str().unwrap().downcast_iter().next().unwrap().clone();
 
         for (iteration_idx, (key_val, &h)) in keys_arr.iter().zip(&hashes).enumerate() {
             let current_partition = self.get_partitions(h);
@@ -426,7 +423,7 @@ impl Sink for Utf8GroupbySink {
                             // the offset in the keys of self
                             let idx_self = k_self.idx as usize;
                             // slice to the keys of self
-                            // safety:
+                            // SAFETY:
                             // in bounds
                             let key_self = unsafe { self.keys.get_unchecked_release(idx_self) };
                             // compare the keys
@@ -516,7 +513,7 @@ impl Sink for Utf8GroupbySink {
         self
     }
     fn fmt(&self) -> &str {
-        "utf8_group_by"
+        "string_group_by"
     }
 }
 
@@ -537,19 +534,19 @@ pub(super) fn apply_aggregate(
                 ($self:expr, $macro:ident $(, $opt_args:expr)*) => {{
                     match $self.dtype() {
                         #[cfg(feature = "dtype-u8")]
-                        DataType::UInt8 => $macro!($self.u8().unwrap(), pre_agg_u8 $(, $opt_args)*),
+                        DataType::UInt8 => $macro!($self.u8().unwrap(), pre_agg_primitive $(, $opt_args)*),
                         #[cfg(feature = "dtype-u16")]
-                        DataType::UInt16 => $macro!($self.u16().unwrap(), pre_agg_u16 $(, $opt_args)*),
-                        DataType::UInt32 => $macro!($self.u32().unwrap(), pre_agg_u32 $(, $opt_args)*),
-                        DataType::UInt64 => $macro!($self.u64().unwrap(), pre_agg_u64 $(, $opt_args)*),
+                        DataType::UInt16 => $macro!($self.u16().unwrap(), pre_agg_primitive $(, $opt_args)*),
+                        DataType::UInt32 => $macro!($self.u32().unwrap(), pre_agg_primitive $(, $opt_args)*),
+                        DataType::UInt64 => $macro!($self.u64().unwrap(), pre_agg_primitive $(, $opt_args)*),
                         #[cfg(feature = "dtype-i8")]
-                        DataType::Int8 => $macro!($self.i8().unwrap(), pre_agg_i8 $(, $opt_args)*),
+                        DataType::Int8 => $macro!($self.i8().unwrap(), pre_agg_primitive $(, $opt_args)*),
                         #[cfg(feature = "dtype-i16")]
-                        DataType::Int16 => $macro!($self.i16().unwrap(), pre_agg_i16 $(, $opt_args)*),
-                        DataType::Int32 => $macro!($self.i32().unwrap(), pre_agg_i32 $(, $opt_args)*),
-                        DataType::Int64 => $macro!($self.i64().unwrap(), pre_agg_i64 $(, $opt_args)*),
-                        DataType::Float32 => $macro!($self.f32().unwrap(), pre_agg_f32 $(, $opt_args)*),
-                        DataType::Float64 => $macro!($self.f64().unwrap(), pre_agg_f64 $(, $opt_args)*),
+                        DataType::Int16 => $macro!($self.i16().unwrap(), pre_agg_primitive $(, $opt_args)*),
+                        DataType::Int32 => $macro!($self.i32().unwrap(), pre_agg_primitive $(, $opt_args)*),
+                        DataType::Int64 => $macro!($self.i64().unwrap(), pre_agg_primitive $(, $opt_args)*),
+                        DataType::Float32 => $macro!($self.f32().unwrap(), pre_agg_primitive $(, $opt_args)*),
+                        DataType::Float64 => $macro!($self.f64().unwrap(), pre_agg_primitive $(, $opt_args)*),
                         dt => panic!("not implemented for {:?}", dt),
                     }
                 }};
