@@ -1,7 +1,6 @@
 use std::borrow::Cow;
 
 use arrow::array::{Array, BinaryArray};
-use polars_core::export::ahash::RandomState;
 use polars_core::prelude::*;
 use polars_core::series::IsSorted;
 use polars_ops::chunked_array::DfTake;
@@ -31,7 +30,7 @@ pub struct GenericJoinProbe<K: ExtraPayload> {
     ///      * end = (offset + n_join_keys)
     materialized_join_cols: Arc<[BinaryArray<i64>]>,
     suffix: Arc<str>,
-    hb: RandomState,
+    hb: PlRandomState,
     /// partitioned tables that will be used for probing
     /// stores the key and the chunk_idx, df_idx of the left table
     hash_tables: Arc<PartitionedMap<K>>,
@@ -59,7 +58,7 @@ impl<K: ExtraPayload> GenericJoinProbe<K> {
         mut df_a: DataFrame,
         materialized_join_cols: Arc<[BinaryArray<i64>]>,
         suffix: Arc<str>,
-        hb: RandomState,
+        hb: PlRandomState,
         hash_tables: Arc<PartitionedMap<K>>,
         join_columns_left: Arc<Vec<Arc<dyn PhysicalPipedExpr>>>,
         join_columns_right: Arc<Vec<Arc<dyn PhysicalPipedExpr>>>,
