@@ -144,40 +144,41 @@ pub struct ParquetCloudSink {}
 #[cfg(feature = "cloud")]
 impl ParquetCloudSink {
     #[allow(clippy::new_ret_no_self)]
-    #[tokio::main(flavor = "current_thread")]
-    pub async fn new(
+    pub fn new(
         uri: &str,
         cloud_options: Option<&polars_io::cloud::CloudOptions>,
         parquet_options: ParquetWriteOptions,
         schema: &Schema,
     ) -> PolarsResult<FilesSink> {
-        let cloud_writer = polars_io::cloud::CloudWriter::new(uri, cloud_options).await?;
-        let writer = ParquetWriter::new(cloud_writer)
-            .with_compression(parquet_options.compression)
-            .with_data_page_size(parquet_options.data_page_size)
-            .with_statistics(parquet_options.statistics)
-            .with_row_group_size(parquet_options.row_group_size)
-            // This is important! Otherwise we will deadlock
-            // See: #7074
-            .set_parallel(false)
-            .batched(schema)?;
+        polars_io::pl_async::get_runtime().block_on_potential_spawn(async {
+            let cloud_writer = polars_io::cloud::CloudWriter::new(uri, cloud_options).await?;
+            let writer = ParquetWriter::new(cloud_writer)
+                .with_compression(parquet_options.compression)
+                .with_data_page_size(parquet_options.data_page_size)
+                .with_statistics(parquet_options.statistics)
+                .with_row_group_size(parquet_options.row_group_size)
+                // This is important! Otherwise we will deadlock
+                // See: #7074
+                .set_parallel(false)
+                .batched(schema)?;
 
-        let writer = Box::new(writer) as Box<dyn SinkWriter + Send>;
+            let writer = Box::new(writer) as Box<dyn SinkWriter + Send>;
 
-        let morsels_per_sink = morsels_per_sink();
-        let backpressure = morsels_per_sink * 2;
-        let (sender, receiver) = bounded(backpressure);
+            let morsels_per_sink = morsels_per_sink();
+            let backpressure = morsels_per_sink * 2;
+            let (sender, receiver) = bounded(backpressure);
 
-        let io_thread_handle = Arc::new(Some(init_writer_thread(
-            receiver,
-            writer,
-            true,
-            morsels_per_sink,
-        )));
+            let io_thread_handle = Arc::new(Some(init_writer_thread(
+                receiver,
+                writer,
+                true,
+                morsels_per_sink,
+            )));
 
-        Ok(FilesSink {
-            sender,
-            io_thread_handle,
+            Ok(FilesSink {
+                sender,
+                io_thread_handle,
+            })
         })
     }
 }
