@@ -1,10 +1,13 @@
-use arrow::temporal_conversions::{MICROSECONDS, MILLISECONDS, NANOSECONDS, SECONDS_IN_DAY};
 #[cfg(feature = "timezones")]
 use chrono_tz::Tz;
 #[cfg(feature = "timezones")]
+use polars_core::chunked_array::temporal::validate_time_zone;
+#[cfg(feature = "timezones")]
 use polars_time::base_utc_offset as base_utc_offset_fn;
 #[cfg(feature = "timezones")]
 use polars_time::dst_offset as dst_offset_fn;
+#[cfg(feature = "offset_by")]
+use polars_time::impl_offset_by;
 #[cfg(feature = "serde")]
 use serde::{Deserialize, Serialize};
 
@@ -47,16 +50,18 @@ pub enum TemporalFunction {
     #[cfg(feature = "timezones")]
     ConvertTimeZone(TimeZone),
     TimeStamp(TimeUnit),
-    Truncate(String),
-    #[cfg(feature = "date_offset")]
+    Truncate,
+    #[cfg(feature = "offset_by")]
+    OffsetBy,
+    #[cfg(feature = "month_start")]
     MonthStart,
-    #[cfg(feature = "date_offset")]
+    #[cfg(feature = "month_end")]
     MonthEnd,
     #[cfg(feature = "timezones")]
     BaseUtcOffset,
     #[cfg(feature = "timezones")]
     DSTOffset,
-    Round(String),
+    Round,
     #[cfg(feature = "timezones")]
     ReplaceTimeZone(Option<TimeZone>, NonExistent),
     Combine(TimeUnit),
@@ -100,16 +105,18 @@ impl TemporalFunction {
                 DataType::Datetime(tu, _) => Ok(DataType::Datetime(*tu, None)),
                 dtype => polars_bail!(ComputeError: "expected Datetime, got {}", dtype),
             }),
-            Truncate(_) => mapper.with_same_dtype(),
-            #[cfg(feature = "date_offset")]
+            Truncate => mapper.with_same_dtype(),
+            #[cfg(feature = "offset_by")]
+            OffsetBy => mapper.with_same_dtype(),
+            #[cfg(feature = "month_start")]
             MonthStart => mapper.with_same_dtype(),
-            #[cfg(feature = "date_offset")]
+            #[cfg(feature = "month_end")]
             MonthEnd => mapper.with_same_dtype(),
             #[cfg(feature = "timezones")]
             BaseUtcOffset => mapper.with_dtype(DataType::Duration(TimeUnit::Milliseconds)),
             #[cfg(feature = "timezones")]
             DSTOffset => mapper.with_dtype(DataType::Duration(TimeUnit::Milliseconds)),
-            Round(..) => mapper.with_same_dtype(),
+            Round => mapper.with_same_dtype(),
             #[cfg(feature = "timezones")]
             ReplaceTimeZone(tz, _non_existent) => mapper.map_datetime_dtype_timezone(tz.as_ref()),
             DatetimeFunction {
@@ -168,16 +175,18 @@ impl Display for TemporalFunction {
             CastTimeUnit(_) => "cast_time_unit",
             WithTimeUnit(_) => "with_time_unit",
             TimeStamp(tu) => return write!(f, "dt.timestamp({tu})"),
-            Truncate(..) => "truncate",
-            #[cfg(feature = "date_offset")]
+            Truncate => "truncate",
+            #[cfg(feature = "offset_by")]
+            OffsetBy => "offset_by",
+            #[cfg(feature = "month_start")]
             MonthStart => "month_start",
-            #[cfg(feature = "date_offset")]
+            #[cfg(feature = "month_end")]
             MonthEnd => "month_end",
             #[cfg(feature = "timezones")]
             BaseUtcOffset => "base_utc_offset",
             #[cfg(feature = "timezones")]
             DSTOffset => "dst_offset",
-            Round(..) => "round",
+            Round => "round",
             #[cfg(feature = "timezones")]
             ReplaceTimeZone(_, _) => "replace_time_zone",
             DatetimeFunction { .. } => return write!(f, "dt.datetime"),
@@ -302,24 +311,31 @@ pub(super) fn microsecond(s: &Series) -> PolarsResult<Series> {
 pub(super) fn nanosecond(s: &Series) -> PolarsResult<Series> {
     s.nanosecond().map(|ca| ca.into_series())
 }
+#[cfg(feature = "dtype-duration")]
 pub(super) fn total_days(s: &Series) -> PolarsResult<Series> {
     s.duration().map(|ca| ca.days().into_series())
 }
+#[cfg(feature = "dtype-duration")]
 pub(super) fn total_hours(s: &Series) -> PolarsResult<Series> {
     s.duration().map(|ca| ca.hours().into_series())
 }
+#[cfg(feature = "dtype-duration")]
 pub(super) fn total_minutes(s: &Series) -> PolarsResult<Series> {
     s.duration().map(|ca| ca.minutes().into_series())
 }
+#[cfg(feature = "dtype-duration")]
 pub(super) fn total_seconds(s: &Series) -> PolarsResult<Series> {
     s.duration().map(|ca| ca.seconds().into_series())
 }
+#[cfg(feature = "dtype-duration")]
 pub(super) fn total_milliseconds(s: &Series) -> PolarsResult<Series> {
     s.duration().map(|ca| ca.milliseconds().into_series())
 }
+#[cfg(feature = "dtype-duration")]
 pub(super) fn total_microseconds(s: &Series) -> PolarsResult<Series> {
     s.duration().map(|ca| ca.microseconds().into_series())
 }
+#[cfg(feature = "dtype-duration")]
 pub(super) fn total_nanoseconds(s: &Series) -> PolarsResult<Series> {
     s.duration().map(|ca| ca.nanoseconds().into_series())
 }
@@ -335,6 +351,7 @@ pub(super) fn convert_time_zone(s: &Series, time_zone: &TimeZone) -> PolarsResul
     match s.dtype() {
         DataType::Datetime(_, _) => {
             let mut ca = s.datetime()?.clone();
+            validate_time_zone(time_zone)?;
             ca.set_time_zone(time_zone.clone())?;
             Ok(ca.into_series())
         },
@@ -372,7 +389,7 @@ pub(super) fn cast_time_unit(s: &Series, tu: TimeUnit) -> PolarsResult<Series> {
     }
 }
 
-pub(super) fn truncate(s: &[Series], offset: &str) -> PolarsResult<Series> {
+pub(super) fn truncate(s: &[Series]) -> PolarsResult<Series> {
     let time_series = &s[0];
     let every = s[1].str()?;
 
@@ -381,24 +398,23 @@ pub(super) fn truncate(s: &[Series], offset: &str) -> PolarsResult<Series> {
             #[cfg(feature = "timezones")]
             Some(tz) => time_series
                 .datetime()?
-                .truncate(tz.parse::<Tz>().ok().as_ref(), every, offset)?
-                .into_series(),
-            _ => time_series
-                .datetime()?
-                .truncate(None, every, offset)?
+                .truncate(tz.parse::<Tz>().ok().as_ref(), every)?
                 .into_series(),
+            _ => time_series.datetime()?.truncate(None, every)?.into_series(),
         },
-        DataType::Date => time_series
-            .date()?
-            .truncate(None, every, offset)?
-            .into_series(),
+        DataType::Date => time_series.date()?.truncate(None, every)?.into_series(),
         dt => polars_bail!(opq = round, got = dt, expected = "date/datetime"),
     };
     out.set_sorted_flag(time_series.is_sorted_flag());
     Ok(out)
 }
 
-#[cfg(feature = "date_offset")]
+#[cfg(feature = "offset_by")]
+pub(super) fn offset_by(s: &[Series]) -> PolarsResult<Series> {
+    impl_offset_by(&s[0], &s[1])
+}
+
+#[cfg(feature = "month_start")]
 pub(super) fn month_start(s: &Series) -> PolarsResult<Series> {
     Ok(match s.dtype() {
         DataType::Datetime(_, tz) => match tz {
@@ -415,7 +431,7 @@ pub(super) fn month_start(s: &Series) -> PolarsResult<Series> {
     })
 }
 
-#[cfg(feature = "date_offset")]
+#[cfg(feature = "month_end")]
 pub(super) fn month_end(s: &Series) -> PolarsResult<Series> {
     Ok(match s.dtype() {
         DataType::Datetime(_, tz) => match tz {
@@ -465,9 +481,7 @@ pub(super) fn dst_offset(s: &Series) -> PolarsResult<Series> {
     }
 }
 
-pub(super) fn round(s: &[Series], offset: &str) -> PolarsResult<Series> {
-    let offset = Duration::parse(offset);
-
+pub(super) fn round(s: &[Series]) -> PolarsResult<Series> {
     let time_series = &s[0];
     let every = s[1].str()?;
 
@@ -477,106 +491,19 @@ pub(super) fn round(s: &[Series], offset: &str) -> PolarsResult<Series> {
             Some(tz) => time_series
                 .datetime()
                 .unwrap()
-                .round(every, offset, tz.parse::<Tz>().ok().as_ref())?
+                .round(every, tz.parse::<Tz>().ok().as_ref())?
                 .into_series(),
             _ => time_series
                 .datetime()
                 .unwrap()
-                .round(every, offset, None)?
+                .round(every, None)?
                 .into_series(),
         },
         DataType::Date => time_series
             .date()
             .unwrap()
-            .round(every, offset, None)?
+            .round(every, None)?
             .into_series(),
         dt => polars_bail!(opq = round, got = dt, expected = "date/datetime"),
     })
 }
-
-pub(super) fn duration(s: &[Series], time_unit: TimeUnit) -> PolarsResult<Series> {
-    if s.iter().any(|s| s.is_empty()) {
-        return Ok(Series::new_empty(
-            s[0].name(),
-            &DataType::Duration(time_unit),
-        ));
-    }
-
-    // TODO: Handle overflow for UInt64
-    let weeks = s[0].cast(&DataType::Int64).unwrap();
-    let days = s[1].cast(&DataType::Int64).unwrap();
-    let hours = s[2].cast(&DataType::Int64).unwrap();
-    let minutes = s[3].cast(&DataType::Int64).unwrap();
-    let seconds = s[4].cast(&DataType::Int64).unwrap();
-    let mut milliseconds = s[5].cast(&DataType::Int64).unwrap();
-    let mut microseconds = s[6].cast(&DataType::Int64).unwrap();
-    let mut nanoseconds = s[7].cast(&DataType::Int64).unwrap();
-
-    let is_scalar = |s: &Series| s.len() == 1;
-    let is_zero_scalar = |s: &Series| is_scalar(s) && s.get(0).unwrap() == AnyValue::Int64(0);
-
-    // Process subseconds
-    let max_len = s.iter().map(|s| s.len()).max().unwrap();
-    let mut duration = match time_unit {
-        TimeUnit::Microseconds => {
-            if is_scalar(&microseconds) {
-                microseconds = microseconds.new_from_index(0, max_len);
-            }
-            if !is_zero_scalar(&nanoseconds) {
-                microseconds = microseconds + (nanoseconds.wrapping_trunc_div_scalar(1_000));
-            }
-            if !is_zero_scalar(&milliseconds) {
-                microseconds = microseconds + (milliseconds * 1_000);
-            }
-            microseconds
-        },
-        TimeUnit::Nanoseconds => {
-            if is_scalar(&nanoseconds) {
-                nanoseconds = nanoseconds.new_from_index(0, max_len);
-            }
-            if !is_zero_scalar(&microseconds) {
-                nanoseconds = nanoseconds + (microseconds * 1_000);
-            }
-            if !is_zero_scalar(&milliseconds) {
-                nanoseconds = nanoseconds + (milliseconds * 1_000_000);
-            }
-            nanoseconds
-        },
-        TimeUnit::Milliseconds => {
-            if is_scalar(&milliseconds) {
-                milliseconds = milliseconds.new_from_index(0, max_len);
-            }
-            if !is_zero_scalar(&nanoseconds) {
-                milliseconds = milliseconds + (nanoseconds.wrapping_trunc_div_scalar(1_000_000));
-            }
-            if !is_zero_scalar(&microseconds) {
-                milliseconds = milliseconds + (microseconds.wrapping_trunc_div_scalar(1_000));
-            }
-            milliseconds
-        },
-    };
-
-    // Process other duration specifiers
-    let multiplier = match time_unit {
-        TimeUnit::Nanoseconds => NANOSECONDS,
-        TimeUnit::Microseconds => MICROSECONDS,
-        TimeUnit::Milliseconds => MILLISECONDS,
-    };
-    if !is_zero_scalar(&seconds) {
-        duration = duration + seconds * multiplier;
-    }
-    if !is_zero_scalar(&minutes) {
-        duration = duration + minutes * (multiplier * 60);
-    }
-    if !is_zero_scalar(&hours) {
-        duration = duration + hours * (multiplier * 60 * 60);
-    }
-    if !is_zero_scalar(&days) {
-        duration = duration + days * (multiplier * SECONDS_IN_DAY);
-    }
-    if !is_zero_scalar(&weeks) {
-        duration = duration + weeks * (multiplier * SECONDS_IN_DAY * 7);
-    }
-
-    duration.cast(&DataType::Duration(time_unit))
-}
