@@ -51,8 +51,6 @@ impl FunctionExpr {
             SearchSorted(_) => mapper.with_dtype(IDX_DTYPE),
             #[cfg(feature = "range")]
             Range(func) => func.get_field(mapper),
-            #[cfg(feature = "date_offset")]
-            DateOffset { .. } => mapper.with_same_dtype(),
             #[cfg(feature = "trigonometry")]
             Trigonometry(_) => mapper.map_to_float_dtype(),
             #[cfg(feature = "trigonometry")]
@@ -109,10 +107,16 @@ impl FunctionExpr {
                 sort: _,
                 parallel: _,
                 name,
+                normalize,
             } => mapper.map_dtype(|dt| {
+                let count_dt = if *normalize {
+                    DataType::Float64
+                } else {
+                    IDX_DTYPE
+                };
                 DataType::Struct(vec![
                     Field::new(fields[0].name().as_str(), dt.clone()),
-                    Field::new(name, IDX_DTYPE),
+                    Field::new(name, count_dt),
                 ])
             }),
             #[cfg(feature = "unique_counts")]
@@ -139,7 +143,7 @@ impl FunctionExpr {
                 if *include_breakpoint || *include_category {
                     let mut fields = Vec::with_capacity(3);
                     if *include_breakpoint {
-                        fields.push(Field::new("break_point", DataType::Float64));
+                        fields.push(Field::new("breakpoint", DataType::Float64));
                     }
                     if *include_category {
                         fields.push(Field::new(
@@ -226,14 +230,9 @@ impl FunctionExpr {
                 include_breaks: true,
                 ..
             } => {
-                let name = fields[0].name();
-                let name_bin = format!("{}_bin", name);
                 let struct_dt = DataType::Struct(vec![
-                    Field::new("brk", DataType::Float64),
-                    Field::new(
-                        name_bin.as_str(),
-                        DataType::Categorical(None, Default::default()),
-                    ),
+                    Field::new("breakpoint", DataType::Float64),
+                    Field::new("category", DataType::Categorical(None, Default::default())),
                 ]);
                 mapper.with_dtype(struct_dt)
             },
@@ -269,26 +268,21 @@ impl FunctionExpr {
                 include_breaks: true,
                 ..
             } => {
-                let name = fields[0].name();
-                let name_bin = format!("{}_bin", name);
                 let struct_dt = DataType::Struct(vec![
-                    Field::new("brk", DataType::Float64),
-                    Field::new(
-                        name_bin.as_str(),
-                        DataType::Categorical(None, Default::default()),
-                    ),
+                    Field::new("breakpoint", DataType::Float64),
+                    Field::new("category", DataType::Categorical(None, Default::default())),
                 ]);
                 mapper.with_dtype(struct_dt)
             },
             #[cfg(feature = "rle")]
             RLE => mapper.map_dtype(|dt| {
                 DataType::Struct(vec![
-                    Field::new("lengths", DataType::Int32),
-                    Field::new("values", dt.clone()),
+                    Field::new("len", IDX_DTYPE),
+                    Field::new("value", dt.clone()),
                 ])
             }),
             #[cfg(feature = "rle")]
-            RLEID => mapper.with_dtype(DataType::UInt32),
+            RLEID => mapper.with_dtype(IDX_DTYPE),
             ToPhysical => mapper.to_physical_type(),
             #[cfg(feature = "random")]
             Random { .. } => mapper.with_same_dtype(),
@@ -320,7 +314,9 @@ impl FunctionExpr {
             #[cfg(feature = "ewma")]
             EwmVar { .. } => mapper.map_to_float_dtype(),
             #[cfg(feature = "replace")]
-            Replace { return_dtype } => mapper.replace_dtype(return_dtype.clone()),
+            Replace => mapper.with_same_dtype(),
+            #[cfg(feature = "replace")]
+            ReplaceStrict { return_dtype } => mapper.replace_dtype(return_dtype.clone()),
             FillNullWithStrategy(_) => mapper.with_same_dtype(),
             GatherEvery { .. } => mapper.with_same_dtype(),
             #[cfg(feature = "reinterpret")]
@@ -538,11 +534,13 @@ impl<'a> FieldsMapper<'a> {
     pub fn replace_dtype(&self, return_dtype: Option<DataType>) -> PolarsResult<Field> {
         let dtype = match return_dtype {
             Some(dtype) => dtype,
-            // Supertype of `new` and `default`
             None => {
-                let column_type = &self.fields[0];
                 let new = &self.fields[2];
-                try_get_supertype(column_type.data_type(), new.data_type())?
+                let default = self.fields.get(3);
+                match default {
+                    Some(default) => try_get_supertype(default.data_type(), new.data_type())?,
+                    None => new.data_type().clone(),
+                }
             },
         };
         self.with_dtype(dtype)
