@@ -1,10 +1,3 @@
-#[cfg(feature = "date_offset")]
-use arrow::legacy::time_zone::Tz;
-#[cfg(feature = "date_offset")]
-use polars_core::chunked_array::ops::arity::try_binary_elementwise;
-#[cfg(feature = "date_offset")]
-use polars_time::prelude::*;
-
 use super::*;
 use crate::{map, map_as_slice};
 
@@ -21,7 +14,7 @@ impl From<TemporalFunction> for SpecialEq<Arc<dyn SeriesUdf>> {
             Quarter => map!(datetime::quarter),
             Week => map!(datetime::week),
             WeekDay => map!(datetime::weekday),
-            Duration(tu) => map_as_slice!(datetime::duration, tu),
+            Duration(tu) => map_as_slice!(impl_duration, tu),
             Day => map!(datetime::day),
             OrdinalDay => map!(datetime::ordinal_day),
             Time => map!(datetime::time),
@@ -46,18 +39,22 @@ impl From<TemporalFunction> for SpecialEq<Arc<dyn SeriesUdf>> {
             ConvertTimeZone(tz) => map!(datetime::convert_time_zone, &tz),
             WithTimeUnit(tu) => map!(datetime::with_time_unit, tu),
             CastTimeUnit(tu) => map!(datetime::cast_time_unit, tu),
-            Truncate(offset) => {
-                map_as_slice!(datetime::truncate, &offset)
+            Truncate => {
+                map_as_slice!(datetime::truncate)
             },
-            #[cfg(feature = "date_offset")]
+            #[cfg(feature = "offset_by")]
+            OffsetBy => {
+                map_as_slice!(datetime::offset_by)
+            },
+            #[cfg(feature = "month_start")]
             MonthStart => map!(datetime::month_start),
-            #[cfg(feature = "date_offset")]
+            #[cfg(feature = "month_end")]
             MonthEnd => map!(datetime::month_end),
             #[cfg(feature = "timezones")]
             BaseUtcOffset => map!(datetime::base_utc_offset),
             #[cfg(feature = "timezones")]
             DSTOffset => map!(datetime::dst_offset),
-            Round(offset) => map_as_slice!(datetime::round, &offset),
+            Round => map_as_slice!(datetime::round),
             #[cfg(feature = "timezones")]
             ReplaceTimeZone(tz, non_existent) => {
                 map_as_slice!(dispatch::replace_time_zone, tz.as_deref(), non_existent)
@@ -185,115 +182,6 @@ pub(super) fn datetime(
     Ok(s)
 }
 
-#[cfg(feature = "date_offset")]
-fn apply_offsets_to_datetime(
-    datetime: &Logical<DatetimeType, Int64Type>,
-    offsets: &StringChunked,
-    offset_fn: fn(&Duration, i64, Option<&Tz>) -> PolarsResult<i64>,
-    time_zone: Option<&Tz>,
-) -> PolarsResult<Int64Chunked> {
-    match (datetime.len(), offsets.len()) {
-        (1, _) => match datetime.0.get(0) {
-            Some(dt) => offsets.try_apply_nonnull_values_generic(|offset| {
-                offset_fn(&Duration::parse(offset), dt, time_zone)
-            }),
-            _ => Ok(Int64Chunked::full_null(datetime.0.name(), offsets.len())),
-        },
-        (_, 1) => match offsets.get(0) {
-            Some(offset) => {
-                let offset = &Duration::parse(offset);
-                datetime
-                    .0
-                    .try_apply_nonnull_values_generic(|v| offset_fn(offset, v, time_zone))
-            },
-            _ => Ok(datetime.0.apply(|_| None)),
-        },
-        _ => try_binary_elementwise(datetime, offsets, |timestamp_opt, offset_opt| {
-            match (timestamp_opt, offset_opt) {
-                (Some(timestamp), Some(offset)) => {
-                    offset_fn(&Duration::parse(offset), timestamp, time_zone).map(Some)
-                },
-                _ => Ok(None),
-            }
-        }),
-    }
-}
-
-#[cfg(feature = "date_offset")]
-pub(super) fn date_offset(s: &[Series]) -> PolarsResult<Series> {
-    let ts = &s[0];
-    let offsets = &s[1].str()?;
-
-    let preserve_sortedness: bool;
-    let out = match ts.dtype() {
-        DataType::Date => {
-            let ts = ts
-                .cast(&DataType::Datetime(TimeUnit::Milliseconds, None))
-                .unwrap();
-            let datetime = ts.datetime().unwrap();
-            let out = apply_offsets_to_datetime(datetime, offsets, Duration::add_ms, None)?;
-            // sortedness is only guaranteed to be preserved if a constant offset is being added to every datetime
-            preserve_sortedness = match offsets.len() {
-                1 => offsets.get(0).is_some(),
-                _ => false,
-            };
-            out.cast(&DataType::Datetime(TimeUnit::Milliseconds, None))
-                .unwrap()
-                .cast(&DataType::Date)
-        },
-        DataType::Datetime(tu, tz) => {
-            let datetime = ts.datetime().unwrap();
-
-            let offset_fn = match tu {
-                TimeUnit::Nanoseconds => Duration::add_ns,
-                TimeUnit::Microseconds => Duration::add_us,
-                TimeUnit::Milliseconds => Duration::add_ms,
-            };
-
-            let out = match tz {
-                #[cfg(feature = "timezones")]
-                Some(ref tz) => apply_offsets_to_datetime(
-                    datetime,
-                    offsets,
-                    offset_fn,
-                    tz.parse::<Tz>().ok().as_ref(),
-                )?,
-                _ => apply_offsets_to_datetime(datetime, offsets, offset_fn, None)?,
-            };
-            // Sortedness may not be preserved when crossing daylight savings time boundaries
-            // for calendar-aware durations.
-            // Constant durations (e.g. 2 hours) always preserve sortedness.
-            preserve_sortedness = match offsets.len() {
-                1 => match offsets.get(0) {
-                    Some(offset) => {
-                        let offset = Duration::parse(offset);
-                        tz.is_none()
-                            || tz.as_deref() == Some("UTC")
-                            || offset.is_constant_duration(tz.as_deref())
-                    },
-                    None => false,
-                },
-                _ => false,
-            };
-            out.cast(&DataType::Datetime(*tu, tz.clone()))
-        },
-        dt => polars_bail!(
-            ComputeError: "cannot use 'date_offset' on Series of datatype {}", dt,
-        ),
-    };
-    if preserve_sortedness {
-        out.map(|mut out| {
-            out.set_sorted_flag(ts.is_sorted_flag());
-            out
-        })
-    } else {
-        out.map(|mut out| {
-            out.set_sorted_flag(IsSorted::Not);
-            out
-        })
-    }
-}
-
 pub(super) fn combine(s: &[Series], tu: TimeUnit) -> PolarsResult<Series> {
     let date = &s[0];
     let time = &s[1];
@@ -314,12 +202,12 @@ pub(super) fn combine(s: &[Series], tu: TimeUnit) -> PolarsResult<Series> {
     match tz {
         #[cfg(feature = "timezones")]
         Some(tz) => Ok(polars_ops::prelude::replace_time_zone(
-            result_naive.datetime().unwrap(),
+            result_naive?.datetime().unwrap(),
             Some(tz),
             &StringChunked::from_iter(std::iter::once("raise")),
             NonExistent::Raise,
         )?
         .into()),
-        _ => Ok(result_naive),
+        _ => result_naive,
     }
 }
