@@ -2,25 +2,25 @@ use std::fmt::Display;
 use std::ops::BitAnd;
 
 use super::*;
-use crate::logical_plan::alp::tree_format::TreeFmtVisitor;
-use crate::logical_plan::conversion::is_regex_projection;
-use crate::logical_plan::visitor::{AexprNode, TreeWalker};
+use crate::plans::conversion::is_regex_projection;
+use crate::plans::ir::tree_format::TreeFmtVisitor;
+use crate::plans::visitor::{AexprNode, TreeWalker};
 
 /// Specialized expressions for Categorical dtypes.
 pub struct MetaNameSpace(pub(crate) Expr);
 
 impl MetaNameSpace {
     /// Pop latest expression and return the input(s) of the popped expression.
-    pub fn pop(self) -> Vec<Expr> {
+    pub fn pop(self) -> PolarsResult<Vec<Expr>> {
         let mut arena = Arena::with_capacity(8);
-        let node = to_aexpr(self.0, &mut arena);
+        let node = to_aexpr(self.0, &mut arena)?;
         let ae = arena.get(node);
         let mut inputs = Vec::with_capacity(2);
         ae.nodes(&mut inputs);
-        inputs
+        Ok(inputs
             .iter()
             .map(|node| node_to_expr(*node, &arena))
-            .collect()
+            .collect())
     }
 
     /// Get the root column names.
@@ -31,8 +31,9 @@ impl MetaNameSpace {
     /// A projection that only takes a column or a column + alias.
     pub fn is_simple_projection(&self) -> bool {
         let mut arena = Arena::with_capacity(8);
-        let node = to_aexpr(self.0.clone(), &mut arena);
-        aexpr_is_simple_projection(node, &arena)
+        to_aexpr(self.0.clone(), &mut arena)
+            .map(|node| aexpr_is_simple_projection(node, &arena))
+            .unwrap_or(false)
     }
 
     /// Get the output name of this expression.
@@ -109,6 +110,19 @@ impl MetaNameSpace {
         }
     }
 
+    pub fn _selector_and(self, other: Expr) -> PolarsResult<Expr> {
+        if let Expr::Selector(mut s) = self.0 {
+            if let Expr::Selector(s_other) = other {
+                s = s.bitand(s_other);
+            } else {
+                s = s.bitand(Selector::Root(Box::new(other)))
+            }
+            Ok(Expr::Selector(s))
+        } else {
+            polars_bail!(ComputeError: "expected selector, got {:?}", self.0)
+        }
+    }
+
     pub fn _selector_sub(self, other: Expr) -> PolarsResult<Expr> {
         if let Expr::Selector(mut s) = self.0 {
             if let Expr::Selector(s_other) = other {
@@ -122,12 +136,12 @@ impl MetaNameSpace {
         }
     }
 
-    pub fn _selector_and(self, other: Expr) -> PolarsResult<Expr> {
+    pub fn _selector_xor(self, other: Expr) -> PolarsResult<Expr> {
         if let Expr::Selector(mut s) = self.0 {
             if let Expr::Selector(s_other) = other {
-                s = s.bitand(s_other);
+                s = s ^ s_other;
             } else {
-                s = s.bitand(Selector::Root(Box::new(other)))
+                s = s ^ Selector::Root(Box::new(other))
             }
             Ok(Expr::Selector(s))
         } else {
@@ -147,7 +161,7 @@ impl MetaNameSpace {
     /// the expression as a tree
     pub fn into_tree_formatter(self) -> PolarsResult<impl Display> {
         let mut arena = Default::default();
-        let node = to_aexpr(self.0, &mut arena);
+        let node = to_aexpr(self.0, &mut arena)?;
         let mut visitor = TreeFmtVisitor::default();
 
         AexprNode::new(node).visit(&mut visitor, &arena)?;
