@@ -62,8 +62,10 @@ use polars_core::prelude::*;
 #[cfg(feature = "diff")]
 use polars_core::series::ops::NullBehavior;
 use polars_core::series::IsSorted;
-use polars_core::utils::try_get_supertype;
-pub(crate) use selector::Selector;
+#[cfg(any(feature = "search_sorted", feature = "is_between"))]
+use polars_core::utils::SuperTypeFlags;
+use polars_core::utils::{try_get_supertype, SuperTypeOptions};
+pub use selector::Selector;
 #[cfg(feature = "dtype-struct")]
 pub use struct_::*;
 pub use udf::UserDefinedFunction;
@@ -186,7 +188,15 @@ impl Expr {
 
     /// Drop null values.
     pub fn drop_nulls(self) -> Self {
-        self.apply_private(FunctionExpr::DropNulls)
+        Expr::Function {
+            input: vec![self],
+            function: FunctionExpr::DropNulls,
+            options: FunctionOptions {
+                collect_groups: ApplyOptions::GroupWise,
+                flags: FunctionFlags::default() | FunctionFlags::ALLOW_EMPTY_INPUTS,
+                ..Default::default()
+            },
+        }
     }
 
     /// Drop NaN values.
@@ -304,7 +314,7 @@ impl Expr {
     pub fn arg_min(self) -> Self {
         let options = FunctionOptions {
             collect_groups: ApplyOptions::GroupWise,
-            returns_scalar: true,
+            flags: FunctionFlags::default() | FunctionFlags::RETURNS_SCALAR,
             fmt_str: "arg_min",
             ..Default::default()
         };
@@ -325,7 +335,7 @@ impl Expr {
     pub fn arg_max(self) -> Self {
         let options = FunctionOptions {
             collect_groups: ApplyOptions::GroupWise,
-            returns_scalar: true,
+            flags: FunctionFlags::default() | FunctionFlags::RETURNS_SCALAR,
             fmt_str: "arg_max",
             ..Default::default()
         };
@@ -366,9 +376,11 @@ impl Expr {
             function: FunctionExpr::SearchSorted(side),
             options: FunctionOptions {
                 collect_groups: ApplyOptions::GroupWise,
-                returns_scalar: true,
+                flags: FunctionFlags::default() | FunctionFlags::RETURNS_SCALAR,
                 fmt_str: "search_sorted",
-                cast_to_supertypes: Some(Default::default()),
+                cast_to_supertypes: Some(
+                    (SuperTypeFlags::default() & !SuperTypeFlags::ALLOW_PRIMITIVE_TO_STRING).into(),
+                ),
                 ..Default::default()
             },
         }
@@ -531,6 +543,7 @@ impl Expr {
             options: FunctionOptions {
                 collect_groups: ApplyOptions::ElementWise,
                 fmt_str: "map",
+                flags: FunctionFlags::default() | FunctionFlags::OPTIONAL_RE_ENTRANT,
                 ..Default::default()
             },
         }
@@ -691,12 +704,17 @@ impl Expr {
             None
         };
 
+        let mut flags = FunctionFlags::default();
+        if returns_scalar {
+            flags |= FunctionFlags::RETURNS_SCALAR;
+        }
+
         Expr::Function {
             input,
             function: function_expr,
             options: FunctionOptions {
                 collect_groups: ApplyOptions::GroupWise,
-                returns_scalar,
+                flags,
                 cast_to_supertypes,
                 ..Default::default()
             },
@@ -708,24 +726,23 @@ impl Expr {
         function_expr: FunctionExpr,
         arguments: &[Expr],
         returns_scalar: bool,
-        cast_to_supertypes: bool,
+        cast_to_supertypes: Option<SuperTypeOptions>,
     ) -> Self {
         let mut input = Vec::with_capacity(arguments.len() + 1);
         input.push(self);
         input.extend_from_slice(arguments);
 
-        let cast_to_supertypes = if cast_to_supertypes {
-            Some(Default::default())
-        } else {
-            None
-        };
+        let mut flags = FunctionFlags::default();
+        if returns_scalar {
+            flags |= FunctionFlags::RETURNS_SCALAR;
+        }
 
         Expr::Function {
             input,
             function: function_expr,
             options: FunctionOptions {
                 collect_groups: ApplyOptions::ElementWise,
-                returns_scalar,
+                flags,
                 cast_to_supertypes,
                 ..Default::default()
             },
@@ -803,7 +820,7 @@ impl Expr {
     pub fn product(self) -> Self {
         let options = FunctionOptions {
             collect_groups: ApplyOptions::GroupWise,
-            returns_scalar: true,
+            flags: FunctionFlags::default() | FunctionFlags::RETURNS_SCALAR,
             fmt_str: "product",
             ..Default::default()
         };
@@ -873,7 +890,7 @@ impl Expr {
             },
             &[min, max],
             false,
-            false,
+            None,
         )
     }
 
@@ -887,7 +904,7 @@ impl Expr {
             },
             &[max],
             false,
-            false,
+            None,
         )
     }
 
@@ -901,7 +918,7 @@ impl Expr {
             },
             &[min],
             false,
-            false,
+            None,
         )
     }
 
@@ -1068,7 +1085,7 @@ impl Expr {
             BooleanFunction::IsBetween { closed }.into(),
             &[lower.into(), upper.into()],
             false,
-            true,
+            Some((SuperTypeFlags::default() & !SuperTypeFlags::ALLOW_PRIMITIVE_TO_STRING).into()),
         )
     }
 
@@ -1084,7 +1101,7 @@ impl Expr {
     pub fn approx_n_unique(self) -> Self {
         self.apply_private(FunctionExpr::ApproxNUnique)
             .with_function_options(|mut options| {
-                options.returns_scalar = true;
+                options.flags |= FunctionFlags::RETURNS_SCALAR;
                 options
             })
     }
@@ -1145,7 +1162,7 @@ impl Expr {
                 BooleanFunction::IsIn.into(),
                 arguments,
                 returns_scalar,
-                true,
+                Some(Default::default()),
             )
         } else {
             self.apply_many_private(
@@ -1519,7 +1536,7 @@ impl Expr {
         let args = [old, new];
 
         if literal_searchers {
-            self.map_many_private(FunctionExpr::Replace, &args, false, false)
+            self.map_many_private(FunctionExpr::Replace, &args, false, None)
         } else {
             self.apply_many_private(FunctionExpr::Replace, &args, false, false)
         }
@@ -1550,7 +1567,7 @@ impl Expr {
                 FunctionExpr::ReplaceStrict { return_dtype },
                 &args,
                 false,
-                false,
+                None,
             )
         } else {
             self.apply_many_private(
@@ -1578,7 +1595,7 @@ impl Expr {
             include_breaks,
         })
         .with_function_options(|mut opt| {
-            opt.pass_name_to_apply = true;
+            opt.flags |= FunctionFlags::PASS_NAME_TO_APPLY;
             opt
         })
     }
@@ -1601,7 +1618,7 @@ impl Expr {
             include_breaks,
         })
         .with_function_options(|mut opt| {
-            opt.pass_name_to_apply = true;
+            opt.flags |= FunctionFlags::PASS_NAME_TO_APPLY;
             opt
         })
     }
@@ -1625,7 +1642,7 @@ impl Expr {
             include_breaks,
         })
         .with_function_options(|mut opt| {
-            opt.pass_name_to_apply = true;
+            opt.flags |= FunctionFlags::PASS_NAME_TO_APPLY;
             opt
         })
     }
@@ -1667,7 +1684,7 @@ impl Expr {
     pub fn skew(self, bias: bool) -> Expr {
         self.apply_private(FunctionExpr::Skew(bias))
             .with_function_options(|mut options| {
-                options.returns_scalar = true;
+                options.flags |= FunctionFlags::RETURNS_SCALAR;
                 options
             })
     }
@@ -1683,19 +1700,27 @@ impl Expr {
     pub fn kurtosis(self, fisher: bool, bias: bool) -> Expr {
         self.apply_private(FunctionExpr::Kurtosis(fisher, bias))
             .with_function_options(|mut options| {
-                options.returns_scalar = true;
+                options.flags |= FunctionFlags::RETURNS_SCALAR;
                 options
             })
     }
 
     /// Get maximal value that could be hold by this dtype.
     pub fn upper_bound(self) -> Expr {
-        self.map_private(FunctionExpr::UpperBound)
+        self.apply_private(FunctionExpr::UpperBound)
+            .with_function_options(|mut options| {
+                options.flags |= FunctionFlags::RETURNS_SCALAR;
+                options
+            })
     }
 
     /// Get minimal value that could be hold by this dtype.
     pub fn lower_bound(self) -> Expr {
-        self.map_private(FunctionExpr::LowerBound)
+        self.apply_private(FunctionExpr::LowerBound)
+            .with_function_options(|mut options| {
+                options.flags |= FunctionFlags::RETURNS_SCALAR;
+                options
+            })
     }
 
     pub fn reshape(self, dimensions: &[i64], nested_type: NestedType) -> Self {
@@ -1742,7 +1767,7 @@ impl Expr {
     pub fn any(self, ignore_nulls: bool) -> Self {
         self.apply_private(BooleanFunction::Any { ignore_nulls }.into())
             .with_function_options(|mut opt| {
-                opt.returns_scalar = true;
+                opt.flags |= FunctionFlags::RETURNS_SCALAR;
                 opt
             })
     }
@@ -1757,7 +1782,7 @@ impl Expr {
     pub fn all(self, ignore_nulls: bool) -> Self {
         self.apply_private(BooleanFunction::All { ignore_nulls }.into())
             .with_function_options(|mut opt| {
-                opt.returns_scalar = true;
+                opt.flags |= FunctionFlags::RETURNS_SCALAR;
                 opt
             })
     }
@@ -1780,7 +1805,7 @@ impl Expr {
             normalize,
         })
         .with_function_options(|mut opts| {
-            opts.pass_name_to_apply = true;
+            opts.flags |= FunctionFlags::PASS_NAME_TO_APPLY;
             opts
         })
     }
@@ -1817,7 +1842,7 @@ impl Expr {
     pub fn entropy(self, base: f64, normalize: bool) -> Self {
         self.apply_private(FunctionExpr::Entropy { base, normalize })
             .with_function_options(|mut options| {
-                options.returns_scalar = true;
+                options.flags |= FunctionFlags::RETURNS_SCALAR;
                 options
             })
     }
@@ -1825,7 +1850,7 @@ impl Expr {
     pub fn null_count(self) -> Expr {
         self.apply_private(FunctionExpr::NullCount)
             .with_function_options(|mut options| {
-                options.returns_scalar = true;
+                options.flags |= FunctionFlags::RETURNS_SCALAR;
                 options
             })
     }
@@ -1963,8 +1988,8 @@ where
         output_type,
         options: FunctionOptions {
             collect_groups: ApplyOptions::ApplyList,
-            returns_scalar: true,
             fmt_str: "",
+            flags: FunctionFlags::default() | FunctionFlags::RETURNS_SCALAR,
             ..Default::default()
         },
     }
@@ -1990,6 +2015,10 @@ where
     E: AsRef<[Expr]>,
 {
     let input = expr.as_ref().to_vec();
+    let mut flags = FunctionFlags::default();
+    if returns_scalar {
+        flags |= FunctionFlags::RETURNS_SCALAR;
+    }
 
     Expr::AnonymousFunction {
         input,
@@ -1999,8 +2028,8 @@ where
             collect_groups: ApplyOptions::GroupWise,
             // don't set this to true
             // this is for the caller to decide
-            returns_scalar,
             fmt_str: "",
+            flags,
             ..Default::default()
         },
     }
